@@ -24,6 +24,7 @@ wiki_pages = [
  "TracAccessibility",
  "TracAdmin",
  "TracBackup",
+ "TracBatchModify",
  "TracBrowser",
  "TracCgi",
  "TracChangeset",
@@ -77,7 +78,7 @@ def get_page_from_file(prefix, pname):
         f = open(pname ,'r')
         d = f.read()
         f.close()
-    except:
+    except Exception:
         print "Missing page: %s" % pname
     return d
 
