@@ -1,7 +1,7 @@
 """Inserts the current time (in seconds) into the wiki page."""
 
-revision = "$Rev: 6326 $"
-url = "$URL: file:///var/svn/trac/tags/trac-0.12.5/sample-plugins/Timestamp.py $"
+revision = "$Rev: 10617 $"
+url = "$URL: https://svn.edgewall.org/repos/trac/tags/trac-1.0.1/sample-plugins/Timestamp.py $"
 
 #
 # The following shows the code for macro, old-style.
@@ -34,7 +34,7 @@ from trac.util.datefmt import format_datetime, utc
 from trac.wiki.macros import WikiMacroBase
 
 class TimestampMacro(WikiMacroBase):
-    """Inserts the current time (in seconds) into the wiki page."""
+    _description = "Inserts the current time (in seconds) into the wiki page."
 
     def expand_macro(self, formatter, name, args):
         t = datetime.now(utc)
