@@ -1,12 +1,12 @@
 from trac.core import *
 from trac.perm import IPermissionPolicy, IPermissionRequestor
 
-revision = "$Rev: 6669 $"
-url = "$URL: file:///var/svn/trac/tags/trac-0.12.5/sample-plugins/permissions/vulnerability_tickets.py $"
+revision = "$Rev: 11490 $"
+url = "$URL: https://svn.edgewall.org/repos/trac/tags/trac-1.0.1/sample-plugins/permissions/vulnerability_tickets.py $"
 
 class SecurityTicketsPolicy(Component):
     """Prevent public access to security sensitive tickets.
-    
+
     Add the VULNERABILITY_VIEW permission as a pre-requisite for any
     other permission check done on tickets that have the words
     "security" or "vulnerability" in the summary or keywords fields.
@@ -15,11 +15,11 @@ class SecurityTicketsPolicy(Component):
     place in your list of permission policies, e.g.
     {{{
     [trac]
-    permission_policies = SecurityTicketsPolicy, AuthzPolicy, 
+    permission_policies = SecurityTicketsPolicy, AuthzPolicy,
                           DefaultPermissionPolicy, LegacyAttachmentPolicy
     }}}
     """
-    
+
     implements(IPermissionPolicy, IPermissionRequestor)
 
     # IPermissionPolicy methods
@@ -30,7 +30,7 @@ class SecurityTicketsPolicy(Component):
         # to recursion.
         if action == 'VULNERABILITY_VIEW':
             return
-        
+
         # Check whether we're dealing with a ticket resource
         while resource:
             if resource.realm == 'ticket':
@@ -38,12 +38,10 @@ class SecurityTicketsPolicy(Component):
             resource = resource.parent
 
         if resource and resource.realm == 'ticket' and resource.id is not None:
-            db = self.env.get_db_cnx()
-            cursor = db.cursor()
-            cursor.execute("SELECT keywords, summary FROM ticket WHERE id=%s",
-                           (resource.id,))
-            for keywords, summary in cursor:
-                fields = ''.join([f for f in (keywords, summary) if f]).lower()
+            for keywords, summary in self.env.db_query(
+                    "SELECT keywords, summary FROM ticket WHERE id=%s",
+                    (resource.id,)):
+                fields = ''.join(f for f in (keywords, summary) if f).lower()
                 if 'security' in fields or 'vulnerability' in fields:
                     if 'VULNERABILITY_VIEW' not in perm:
                         return False
