@@ -4,8 +4,8 @@ from trac.core import Component, implements
 from trac.ticket.api import ITicketActionController, TicketSystem
 from trac.perm import IPermissionRequestor
 
-revision = "$Rev: 6326 $"
-url = "$URL: file:///var/svn/trac/tags/trac-0.12.5/sample-plugins/workflow/StatusFixer.py $"
+revision = "$Rev: 11075 $"
+url = "$URL: https://svn.edgewall.org/repos/trac/tags/trac-1.0.1/sample-plugins/workflow/StatusFixer.py $"
 
 class StatusFixerActionController(Component):
     """Provides the admin with a way to correct a ticket's status.
@@ -15,10 +15,11 @@ class StatusFixerActionController(Component):
     status can then be set to some valid state.
 
     Don't forget to add `StatusFixerActionController` to the workflow
-    option in [ticket].
-    If there is no workflow option, the line will look like this:
-
+    option in the `[ticket]` section in TracIni.
+    If there is no other workflow option, the line will look like this:
+    {{{
     workflow = ConfigurableTicketWorkflow,StatusFixerActionController
+    }}}
     """
 
     implements(ITicketActionController, IPermissionRequestor)
@@ -37,14 +38,11 @@ class StatusFixerActionController(Component):
         return actions
 
     def get_all_status(self):
-        """We return all the status that are used in the database so that the
-        user can query for used, but invalid, status."""
-        db = self.env.get_db_cnx()
-        cursor = db.cursor()
-        cursor.execute('SELECT DISTINCT status FROM ticket')
-        all_status = [row[0] for row in cursor]
-        cursor.close()
-        return all_status
+        """Return all the status that are present in the database,
+        so that queries for status no longer in use can be made.
+        """
+        return [status for status, in
+                self.env.db_query("SELECT DISTINCT status FROM ticket")]
 
     def render_ticket_action_control(self, req, ticket, action):
         # Need to use the list of all status so you can't manually set
