@@ -6,8 +6,8 @@ from trac.ticket.default_workflow import ConfigurableTicketWorkflow
 from trac.ticket.model import Priority, Ticket
 #from trac.perm import IPermissionRequestor # (TODO)
 
-revision = "$Rev: 6326 $"
-url = "$URL: file:///var/svn/trac/tags/trac-0.12.5/sample-plugins/workflow/VoteOperation.py $"
+revision = "$Rev: 11490 $"
+url = "$URL: https://svn.edgewall.org/repos/trac/tags/trac-1.0.1/sample-plugins/workflow/VoteOperation.py $"
 
 class VoteOperation(Component):
     """Provides a simplistic vote feature.
@@ -15,12 +15,14 @@ class VoteOperation(Component):
     This is a sample action controller illustrating how to create additional
     ''operations''.
 
-    Don't forget to add `VoteOperation` to the workflow option in [ticket].
-    If there is no workflow option, the line will look like this:
-
+    Don't forget to add `VoteOperation` to the workflow
+    option in the `[ticket]` section in TracIni.
+    If there is no other workflow option, the line will look like this:
+    {{{
     workflow = ConfigurableTicketWorkflow,VoteOperation
+    }}}
     """
-    
+
     implements(ITicketActionController)
 
     def get_ticket_actions(self, req, ticket):
