@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# 
+#
 # Copyright (C)2006-2009 Edgewall Software
 # All rights reserved.
 #
@@ -16,6 +16,7 @@ import sys
 import traceback
 
 from trac.core import *
+from trac.util.text import levenshtein_distance
 from trac.util.translation import _
 
 
@@ -31,14 +32,14 @@ class IAdminPanelProvider(Interface):
 
     def get_admin_panels(req):
         """Return a list of available admin panels.
-        
+
         The items returned by this function must be tuples of the form
         `(category, category_label, page, page_label)`.
         """
 
     def render_admin_panel(req, category, page, path_info):
         """Process a request for an admin panel.
-        
+
         This function should return a tuple of the form `(template, data)`,
         where `template` is the name of the template to use and `data` is the
         data to be passed to the template.
@@ -57,21 +58,21 @@ class IAdminCommandProvider(Interface):
     """Extension point interface for adding commands to the console
     administration interface `trac-admin`.
     """
-    
+
     def get_admin_commands():
         """Return a list of available admin commands.
-        
+
         The items returned by this function must be tuples of the form
         `(command, args, help, complete, execute)`, where `command` contains
         the space-separated command and sub-command names, `args` is a string
         describing the command arguments and `help` is the help text. The
         first paragraph of the help text is taken as a short help, shown in the
         list of commands.
-        
+
         `complete` is called to auto-complete the command arguments, with the
         current list of arguments as its only argument. It should return a list
         of relevant values for the last argument in the list.
-        
+
         `execute` is called to execute the command, with the command arguments
         passed as positional arguments.
         """
@@ -79,9 +80,9 @@ class IAdminCommandProvider(Interface):
 
 class AdminCommandManager(Component):
     """trac-admin command manager."""
-    
+
     providers = ExtensionPoint(IAdminCommandProvider)
-    
+
     def get_command_help(self, args=[]):
         """Return help information for a set of commands."""
         commands = []
@@ -92,7 +93,7 @@ class AdminCommandManager(Component):
                     commands.append(cmd[:3])
         commands.sort()
         return commands
-        
+
     def complete_command(self, args, cmd_only=False):
         """Perform auto-completion on the given arguments."""
         comp = []
@@ -109,7 +110,7 @@ class AdminCommandManager(Component):
                         return []
                     return cmd[3](args[len(parts):]) or []
         return comp
-        
+
     def execute_command(self, *args):
         """Execute a command given by a list of arguments."""
         args = list(args)
@@ -133,6 +134,23 @@ class AdminCommandManager(Component):
                         raise
         raise AdminCommandError(_("Command not found"), show_usage=True)
 
+    def get_similar_commands(self, arg, n=5):
+        if not arg:
+            return []
+
+        cmds = set()
+        for provider in self.providers:
+            for cmd in provider.get_admin_commands() or []:
+                cmds.add(cmd[0].split()[0]) # use only first token
+
+        def score(cmd, arg):
+            if cmd.startswith(arg):
+                return 0
+            return levenshtein_distance(cmd, arg) / float(len(cmd) + len(arg))
+        similars = sorted((score(cmd, arg), cmd) for cmd in cmds)
+        similars = [cmd for val, cmd in similars if val <= 0.5]
+        return similars[:n]
+
 
 class PrefixList(list):
     """A list of prefixes for command argument auto-completion."""
