@@ -12,20 +12,25 @@
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at http://trac.edgewall.org/log/.
 
+from __future__ import with_statement
+
 from ConfigParser import ConfigParser
 from copy import deepcopy
 import os.path
 
-from trac.core import ExtensionPoint, TracError
-from trac.util import AtomicFile
-from trac.util.compat import set, sorted
-from trac.util.text import to_unicode, CRLF
-from trac.util.translation import _
+from trac.admin import AdminCommandError, IAdminCommandProvider
+from trac.core import *
+from trac.util import AtomicFile, as_bool
+from trac.util.compat import cleandoc
+from trac.util.text import printout, to_unicode, CRLF
+from trac.util.translation import _, N_
 
-__all__ = ['Configuration', 'Option', 'BoolOption', 'IntOption', 'ListOption',
+__all__ = ['Configuration', 'ConfigSection', 'Option', 'BoolOption',
+           'IntOption', 'FloatOption', 'ListOption', 'ChoiceOption',
            'PathOption', 'ExtensionOption', 'OrderedExtensionsOption',
            'ConfigurationError']
 
+# Retained for backward-compatibility, use as_bool() instead
 _TRUE_VALUES = ('yes', 'true', 'enabled', 'on', 'aye', '1', 1, True)
 
 _use_default = object()
@@ -36,7 +41,7 @@ def _to_utf8(basestr):
 
 class ConfigurationError(TracError):
     """Exception raised when a value in the configuration file is not valid."""
-    title = 'Configuration Error'
+    title = N_('Configuration Error')
 
 
 class Configuration(object):
@@ -46,15 +51,14 @@ class Configuration(object):
     the last modification time of the configuration file, and reparses it
     when the file has changed.
     """
-    def __init__(self, filename):
+    def __init__(self, filename, params={}):
         self.filename = filename
         self.parser = ConfigParser()
         self._old_sections = {}
-        self.parent = None
-        self._base_filename = None
+        self.parents = []
         self._lastmtime = 0
         self._sections = {}
-        self.parse_if_needed()
+        self.parse_if_needed(force=True)
 
     def __contains__(self, name):
         """Return whether the configuration contains a section of the given
@@ -73,115 +77,135 @@ class Configuration(object):
 
     def get(self, section, key, default=''):
         """Return the value of the specified option.
-        
+
         Valid default input is a string. Returns a string.
         """
         return self[section].get(key, default)
 
     def getbool(self, section, key, default=''):
         """Return the specified option as boolean value.
-        
+
         If the value of the option is one of "yes", "true", "enabled", "on",
         or "1", this method wll return `True`, otherwise `False`.
-        
+
         Valid default input is a string or a bool. Returns a bool.
-        
+
         (since Trac 0.9.3, "enabled" added in 0.11)
         """
         return self[section].getbool(key, default)
 
     def getint(self, section, key, default=''):
         """Return the value of the specified option as integer.
-        
+
         If the specified option can not be converted to an integer, a
         `ConfigurationError` exception is raised.
-        
+
         Valid default input is a string or an int. Returns an int.
-        
+
         (since Trac 0.10)
         """
         return self[section].getint(key, default)
 
+    def getfloat(self, section, key, default=''):
+        """Return the value of the specified option as float.
+
+        If the specified option can not be converted to a float, a
+        `ConfigurationError` exception is raised.
+
+        Valid default input is a string, float or int. Returns a float.
+
+        (since Trac 0.12)
+        """
+        return self[section].getfloat(key, default)
+
     def getlist(self, section, key, default='', sep=',', keep_empty=False):
         """Return a list of values that have been specified as a single
         comma-separated option.
-        
+
         A different separator can be specified using the `sep` parameter. If
         the `keep_empty` parameter is set to `True`, empty elements are
         included in the list.
-        
+
         Valid default input is a string or a list. Returns a string.
-        
+
         (since Trac 0.10)
         """
         return self[section].getlist(key, default, sep, keep_empty)
 
     def getpath(self, section, key, default=''):
         """Return a configuration value as an absolute path.
-        
-        Relative paths are made absolute relative to the location of the file
-        the option is read from. A value read from defaults use location of
-        first configuration file as base.
 
-        Valid default input is a string. Returns a string as correct path.
-        
+        Relative paths are resolved relative to the location of this
+        configuration file.
+
+        Valid default input is a string. Returns a normalized path.
+
         (enabled since Trac 0.11.5)
         """
         return self[section].getpath(key, default)
 
     def set(self, section, key, value):
         """Change a configuration value.
-        
+
         These changes are not persistent unless saved with `save()`.
         """
         self[section].set(key, value)
 
-    def defaults(self):
-        """Returns a dictionary of the default configuration values.
-        
-        (since Trac 0.10)
+    def defaults(self, compmgr=None):
+        """Returns a dictionary of the default configuration values
+        (''since 0.10'').
+
+        If `compmgr` is specified, return only options declared in components
+        that are enabled in the given `ComponentManager`.
         """
         defaults = {}
-        for (section, key), option in Option.registry.items():
+        for (section, key), option in Option.get_registry(compmgr).items():
             defaults.setdefault(section, {})[key] = option.default
         return defaults
 
-    def options(self, section):
+    def options(self, section, compmgr=None):
         """Return a list of `(name, value)` tuples for every option in the
         specified section.
-        
+
         This includes options that have default values that haven't been
-        overridden.
+        overridden. If `compmgr` is specified, only return default option
+        values for components that are enabled in the given `ComponentManager`.
         """
-        return self[section].options()
+        return self[section].options(compmgr)
 
     def remove(self, section, key):
         """Remove the specified option."""
         self[section].remove(key)
 
-    def sections(self):
-        """Return a list of section names."""
+    def sections(self, compmgr=None, defaults=True):
+        """Return a list of section names.
+
+        If `compmgr` is specified, only the section names corresponding to
+        options declared in components that are enabled in the given
+        `ComponentManager` are returned.
+        """
         sections = set([to_unicode(s) for s in self.parser.sections()])
-        if self.parent:
-            sections.update(self.parent.sections())
-        else:
-            sections.update(self.defaults().keys())
+        for parent in self.parents:
+            sections.update(parent.sections(compmgr, defaults=False))
+        if defaults:
+            sections.update(self.defaults(compmgr))
         return sorted(sections)
 
-    def has_option(self, section, option):
-        """Returns True if option exists in section in either project or
-        parent trac.ini, or available through the Option registry.
-        
+    def has_option(self, section, option, defaults=True):
+        """Returns True if option exists in section in either the project
+        trac.ini or one of the parents, or is available through the Option
+        registry.
+
         (since Trac 0.11)
         """
         section_str = _to_utf8(section)
         if self.parser.has_section(section_str):
             if _to_utf8(option) in self.parser.options(section_str):
                 return True
-        if self.parent:
-            return self.parent.has_option(section, option)
-        else:
-            return (section, option) in Option.registry
+        for parent in self.parents:
+            if parent.has_option(section, option, defaults=False):
+                return True
+        return defaults and (section, option) in Option.registry
 
     def save(self):
         """Write the configuration options to the primary file."""
@@ -195,8 +219,10 @@ class Configuration(object):
             options = []
             for option in self[section]:
                 default_str = None
-                if self.parent:
-                    default_str = _to_utf8(self.parent.get(section, option))
+                for parent in self.parents:
+                    if parent.has_option(section, option, defaults=False):
+                        default_str = _to_utf8(parent.get(section, option))
+                        break
                 option_str = _to_utf8(option)
                 current_str = False
                 if self.parser.has_option(section_str, option_str):
@@ -208,8 +234,7 @@ class Configuration(object):
 
         # At this point, all the strings in `sections` are UTF-8 encoded `str`
         try:
-            fileobj = AtomicFile(self.filename, 'w')
-            try:
+            with AtomicFile(self.filename, 'w') as fileobj:
                 fileobj.write('# -*- coding: utf-8 -*-\n\n')
                 for section, options in sections:
                     fileobj.write('[%s]\n' % section)
@@ -221,44 +246,43 @@ class Configuration(object):
                                              .replace('\n', '\n ')
                             fileobj.write('%s = %s\n' % (key_str, val_str))
                     fileobj.write('\n')
-            finally:
-                fileobj.close()
             self._old_sections = deepcopy(self.parser._sections)
         except Exception:
             # Revert all changes to avoid inconsistencies
             self.parser._sections = deepcopy(self._old_sections)
             raise
 
-    def parse_if_needed(self):
+    def parse_if_needed(self, force=False):
         if not self.filename or not os.path.isfile(self.filename):
             return False
 
         changed = False
         modtime = os.path.getmtime(self.filename)
-        if modtime > self._lastmtime:
+        if force or modtime > self._lastmtime:
             self._sections = {}
             self.parser._sections = {}
-            self.parser.read(self.filename)
+            if not self.parser.read(self.filename):
+                raise TracError(_("Error reading '%(file)s', make sure it is "
+                                  "readable.", file=self.filename))
             self._lastmtime = modtime
             self._old_sections = deepcopy(self.parser._sections)
             changed = True
 
-        if self.parser.has_option('inherit', 'file'):
-            filename = to_unicode(self.parser.get('inherit', 'file'))
-            if not os.path.isabs(filename):
-                filename = os.path.join(os.path.dirname(self.filename),
-                                        filename)
-            if not self.parent or self.parent.filename != filename:
-                self.parent = Configuration(filename)
-                self.parent._base_filename = self._base_filename \
-                                                or self.filename
-                changed = True
-            else:
-                changed |= self.parent.parse_if_needed()
-        elif self.parent:
-            changed = True
-            self.parent = None
+        if changed:
+            self.parents = []
+            if self.parser.has_option('inherit', 'file'):
+                for filename in self.parser.get('inherit', 'file').split(','):
+                    filename = to_unicode(filename.strip())
+                    if not os.path.isabs(filename):
+                        filename = os.path.join(os.path.dirname(self.filename),
+                                                filename)
+                    self.parents.append(Configuration(filename))
+        else:
+            for parent in self.parents:
+                changed |= parent.parse_if_needed(force=force)
 
+        if changed:
+            self._cache = {}
         return changed
 
     def touch(self):
@@ -266,10 +290,25 @@ class Configuration(object):
            and os.access(self.filename, os.W_OK):
             os.utime(self.filename, None)
 
+    def set_defaults(self, compmgr=None):
+        """Retrieve all default values and store them explicitly in the
+        configuration, so that they can be saved to file.
+
+        Values already set in the configuration are not overridden.
+        """
+        for section, default_options in self.defaults(compmgr).items():
+            for name, value in default_options.items():
+                if not self.parser.has_option(_to_utf8(section),
+                                              _to_utf8(name)):
+                    if any(parent[section].contains(name, defaults=False)
+                           for parent in self.parents):
+                        value = None
+                    self.set(section, name, value)
+
 
 class Section(object):
     """Proxy for a specific configuration section.
-    
+
     Objects of this class should not be instantiated directly.
     """
     __slots__ = ['config', 'name', 'overridden', '_cache']
@@ -280,14 +319,22 @@ class Section(object):
         self.overridden = {}
         self._cache = {}
 
-    def __contains__(self, key):
+    def contains(self, key, defaults=True):
         if self.config.parser.has_option(_to_utf8(self.name), _to_utf8(key)):
             return True
-        if self.config.parent:
-            return key in self.config.parent[self.name]
-        return Option.registry.has_key((self.name, key))
+        for parent in self.config.parents:
+            if parent[self.name].contains(key, defaults=False):
+                return True
+        return defaults and Option.registry.has_key((self.name, key))
 
-    def __iter__(self):
+    __contains__ = contains
+
+    def iterate(self, compmgr=None, defaults=True):
+        """Iterate over the options in this section.
+
+        If `compmgr` is specified, only return default option values for
+        components that are enabled in the given `ComponentManager`.
+        """
         options = set()
         name_str = _to_utf8(self.name)
         if self.config.parser.has_section(name_str):
@@ -295,21 +342,25 @@ class Section(object):
                 option = to_unicode(option_str)
                 options.add(option.lower())
                 yield option
-        if self.config.parent:
-            for option in self.config.parent[self.name]:
-                if option.lower() not in options:
+        for parent in self.config.parents:
+            for option in parent[self.name].iterate(defaults=False):
+                loption = option.lower()
+                if loption not in options:
+                    options.add(loption)
                     yield option
-        else:
-            for section, option in Option.registry.keys():
+        if defaults:
+            for section, option in Option.get_registry(compmgr).keys():
                 if section == self.name and option.lower() not in options:
                     yield option
 
+    __iter__ = iterate
+
     def __repr__(self):
-        return '<Section [%s]>' % (self.name)
+        return '<%s [%s]>' % (self.__class__.__name__, self.name)
 
     def get(self, key, default=''):
         """Return the value of the specified option.
-        
+
         Valid default input is a string. Returns a string.
         """
         cached = self._cache.get(key, _use_default)
@@ -319,14 +370,17 @@ class Section(object):
         key_str = _to_utf8(key)
         if self.config.parser.has_option(name_str, key_str):
             value = self.config.parser.get(name_str, key_str)
-        elif self.config.parent:
-            value = self.config.parent[self.name].get(key, default)
         else:
-            option = Option.registry.get((self.name, key))
-            if option:
-                value = option.default or _use_default
+            for parent in self.config.parents:
+                value = parent[self.name].get(key, _use_default)
+                if value is not _use_default:
+                    break
             else:
-                value = _use_default
+                if default is not _use_default:
+                    option = Option.registry.get((self.name, key))
+                    value = option.default if option else _use_default
+                else:
+                    value = _use_default
         if value is _use_default:
             return default
         if not value:
@@ -338,23 +392,21 @@ class Section(object):
 
     def getbool(self, key, default=''):
         """Return the value of the specified option as boolean.
-        
+
         This method returns `True` if the option value is one of "yes", "true",
-        "enabled", "on", or "1", ignoring case. Otherwise `False` is returned.
+        "enabled", "on", or non-zero numbers, ignoring case. Otherwise `False`
+        is returned.
 
         Valid default input is a string or a bool. Returns a bool.
         """
-        value = self.get(key, default)
-        if isinstance(value, basestring):
-            value = value.lower() in _TRUE_VALUES
-        return bool(value)
+        return as_bool(self.get(key, default))
 
     def getint(self, key, default=''):
         """Return the value of the specified option as integer.
-        
+
         If the specified option can not be converted to an integer, a
         `ConfigurationError` exception is raised.
-        
+
         Valid default input is a string or an int. Returns an int.
         """
         value = self.get(key, default)
@@ -367,14 +419,32 @@ class Section(object):
                     _('[%(section)s] %(entry)s: expected integer, got %(value)s',
                       section=self.name, entry=key, value=repr(value)))
 
+    def getfloat(self, key, default=''):
+        """Return the value of the specified option as float.
+
+        If the specified option can not be converted to a float, a
+        `ConfigurationError` exception is raised.
+
+        Valid default input is a string, float or int. Returns a float.
+        """
+        value = self.get(key, default)
+        if not value:
+            return 0.0
+        try:
+            return float(value)
+        except ValueError:
+            raise ConfigurationError(
+                    _('[%(section)s] %(entry)s: expected float, got %(value)s',
+                      section=self.name, entry=key, value=repr(value)))
+
     def getlist(self, key, default='', sep=',', keep_empty=True):
         """Return a list of values that have been specified as a single
         comma-separated option.
-        
+
         A different separator can be specified using the `sep` parameter. If
         the `keep_empty` parameter is set to `False`, empty elements are omitted
         from the list.
-        
+
         Valid default input is a string or a list. Returns a list.
         """
         value = self.get(key, default)
@@ -385,43 +455,35 @@ class Section(object):
         else:
             items = list(value)
         if not keep_empty:
-            items = filter(None, items)
+            items = [item for item in items if item not in (None, '')]
         return items
 
     def getpath(self, key, default=''):
-        """Return the value of the specified option as a path name, relative to
-        the location of the configuration file the option is defined in.
+        """Return the value of the specified option as a path, relative to
+        the location of this configuration file.
 
-        Valid default input is a string. Returns a string with normalised path.
+        Valid default input is a string. Returns a normalized path.
         """
-        name_str = _to_utf8(self.name)
-        key_str = _to_utf8(key)
-        if self.config.parser.has_option(name_str, key_str):
-            path = to_unicode(self.config.parser.get(name_str, key_str))
-            if not path:
-                return default
-            if not os.path.isabs(path):
-                path = os.path.join(os.path.dirname(self.config.filename),
-                                    path)
-            return os.path.normcase(os.path.realpath(path))
-        elif self.config.parent:
-            return self.config.parent[self.name].getpath(key, default)
-        else:
-            base = self.config._base_filename or self.config.filename
-            path_opt = Option.registry.get((self.name, key), None)
-            path = path_opt and path_opt.default or default
-            if path and not os.path.isabs(path):
-                path = os.path.join(os.path.dirname(base), path)
-            return path
+        path = self.get(key, default)
+        if not path:
+            return default
+        if not os.path.isabs(path):
+            path = os.path.join(os.path.dirname(self.config.filename), path)
+        return os.path.normcase(os.path.realpath(path))
 
-    def options(self):
-        """Return `(key, value)` tuples for every option in the section."""
-        for key in self:
+    def options(self, compmgr=None):
+        """Return `(key, value)` tuples for every option in the section.
+
+        This includes options that have default values that haven't been
+        overridden. If `compmgr` is specified, only return default option
+        values for components that are enabled in the given `ComponentManager`.
+        """
+        for key in self.iterate(compmgr):
             yield key, self.get(key)
 
     def set(self, key, value):
         """Change a configuration value.
-        
+
         These changes are not persistent unless saved with `save()`.
         """
         self._cache.pop(key, None)
@@ -447,15 +509,80 @@ class Section(object):
             self.config.parser.remove_option(_to_utf8(self.name), _to_utf8(key))
 
 
+def _get_registry(cls, compmgr=None):
+    """Return the descriptor registry.
+
+    If `compmgr` is specified, only return descriptors for components that
+    are enabled in the given `ComponentManager`.
+    """
+    if compmgr is None:
+        return cls.registry
+
+    from trac.core import ComponentMeta
+    components = {}
+    for comp in ComponentMeta._components:
+        for attr in comp.__dict__.itervalues():
+            if isinstance(attr, cls):
+                components[attr] = comp
+
+    return dict(each for each in cls.registry.iteritems()
+                if each[1] not in components
+                   or compmgr.is_enabled(components[each[1]]))
+
+
+class ConfigSection(object):
+    """Descriptor for configuration sections."""
+
+    registry = {}
+
+    @staticmethod
+    def get_registry(compmgr=None):
+        """Return the section registry, as a `dict` mapping section names to
+        `ConfigSection` objects.
+
+        If `compmgr` is specified, only return sections for components that are
+        enabled in the given `ComponentManager`.
+        """
+        return _get_registry(ConfigSection, compmgr)
+
+    def __init__(self, name, doc, doc_domain='tracini'):
+        """Create the configuration section."""
+        self.name = name
+        self.registry[self.name] = self
+        self.__doc__ = cleandoc(doc)
+        self.doc_domain = doc_domain
+
+    def __get__(self, instance, owner):
+        if instance is None:
+            return self
+        config = getattr(instance, 'config', None)
+        if config and isinstance(config, Configuration):
+            return config[self.name]
+
+    def __repr__(self):
+        return '<%s [%s]>' % (self.__class__.__name__, self.name)
+
+
 class Option(object):
-    """Descriptor for configuration options on `Configurable` subclasses."""
+    """Descriptor for configuration options."""
 
     registry = {}
     accessor = Section.get
 
-    def __init__(self, section, name, default=None, doc=''):
-        """Create the extension point.
-        
+    @staticmethod
+    def get_registry(compmgr=None):
+        """Return the option registry, as a `dict` mapping `(section, key)`
+        tuples to `Option` objects.
+
+        If `compmgr` is specified, only return options for components that are
+        enabled in the given `ComponentManager`.
+        """
+        return _get_registry(Option, compmgr)
+
+    def __init__(self, section, name, default=None, doc='',
+                 doc_domain='tracini'):
+        """Create the configuration option.
+
         @param section: the name of the configuration section this option
             belongs to
         @param name: the name of the option
@@ -466,7 +593,8 @@ class Option(object):
         self.name = name
         self.default = default
         self.registry[(self.section, self.name)] = self
-        self.__doc__ = doc
+        self.__doc__ = cleandoc(doc)
+        self.doc_domain = doc_domain
 
     def __get__(self, instance, owner):
         if instance is None:
@@ -476,7 +604,6 @@ class Option(object):
             section = config[self.section]
             value = self.accessor(section, self.name, self.default)
             return value
-        return None
 
     def __set__(self, instance, value):
         raise AttributeError, 'can\'t set attribute'
@@ -496,28 +623,64 @@ class IntOption(Option):
     accessor = Section.getint
 
 
+class FloatOption(Option):
+    """Descriptor for float configuration options."""
+    accessor = Section.getfloat
+
+
 class ListOption(Option):
     """Descriptor for configuration options that contain multiple values
-    separated by a specific character."""
+    separated by a specific character.
+    """
 
     def __init__(self, section, name, default=None, sep=',', keep_empty=False,
-                 doc=''):
-        Option.__init__(self, section, name, default, doc)
+                 doc='', doc_domain='tracini'):
+        Option.__init__(self, section, name, default, doc, doc_domain)
         self.sep = sep
         self.keep_empty = keep_empty
 
     def accessor(self, section, name, default):
         return section.getlist(name, default, self.sep, self.keep_empty)
 
+
+class ChoiceOption(Option):
+    """Descriptor for configuration options providing a choice among a list
+    of items.
+
+    The default value is the first choice in the list.
+    """
+
+    def __init__(self, section, name, choices, doc='', doc_domain='tracini'):
+        Option.__init__(self, section, name, _to_utf8(choices[0]), doc,
+                        doc_domain)
+        self.choices = set(_to_utf8(choice).strip() for choice in choices)
+
+    def accessor(self, section, name, default):
+        value = section.get(name, default)
+        if value not in self.choices:
+            raise ConfigurationError(
+                    _('[%(section)s] %(entry)s: expected one of '
+                      '(%(choices)s), got %(value)s',
+                      section=section.name, entry=name, value=repr(value),
+                      choices=', '.join('"%s"' % c
+                                        for c in sorted(self.choices))))
+        return value
+
+
 class PathOption(Option):
-    """Descriptor for file system path configuration options."""
+    """Descriptor for file system path configuration options.
+
+    Relative paths are resolved to absolute paths using the directory
+    containing the configuration file as the reference.
+    """
     accessor = Section.getpath
 
 
 class ExtensionOption(Option):
 
-    def __init__(self, section, name, interface, default=None, doc=''):
-        Option.__init__(self, section, name, default, doc)
+    def __init__(self, section, name, interface, default=None, doc='',
+                 doc_domain='tracini'):
+        Option.__init__(self, section, name, default, doc, doc_domain)
         self.xtnpt = ExtensionPoint(interface)
 
     def __get__(self, instance, owner):
@@ -542,8 +705,9 @@ class OrderedExtensionsOption(ListOption):
     interface are returned, with those specified by the option ordered first."""
 
     def __init__(self, section, name, interface, default=None,
-                 include_missing=True, doc=''):
-        ListOption.__init__(self, section, name, default, doc=doc)
+                 include_missing=True, doc='', doc_domain='tracini'):
+        ListOption.__init__(self, section, name, default, doc=doc,
+                            doc_domain=doc_domain)
         self.xtnpt = ExtensionPoint(interface)
         self.include_missing = include_missing
 
@@ -565,3 +729,51 @@ class OrderedExtensionsOption(ListOption):
             return cmp(order.index(x), order.index(y))
         components.sort(compare)
         return components
+
+
+class ConfigurationAdmin(Component):
+    """trac-admin command provider for trac.ini administration."""
+
+    implements(IAdminCommandProvider)
+
+    # IAdminCommandProvider methods
+
+    def get_admin_commands(self):
+        yield ('config get', '<section> <option>',
+               'Get the value of the given option in "trac.ini"',
+               self._complete_config, self._do_get)
+        yield ('config remove', '<section> <option>',
+               'Remove the specified option from "trac.ini"',
+               self._complete_config, self._do_remove)
+        yield ('config set', '<section> <option> <value>',
+               'Set the value for the given option in "trac.ini"',
+               self._complete_config, self._do_set)
+
+    def _complete_config(self, args):
+        if len(args) == 1:
+            return self.config.sections()
+        elif len(args) == 2:
+            return [name for (name, value) in self.config[args[0]].options()]
+
+    def _do_get(self, section, option):
+        if not self.config.has_option(section, option):
+            raise AdminCommandError(
+                _("Option '%(option)s' doesn't exist in section '%(section)s'",
+                  option=option, section=section))
+        printout(self.config.get(section, option))
+
+    def _do_set(self, section, option, value):
+        self.config.set(section, option, value)
+        self.config.save()
+        if section == 'inherit' and option == 'file':
+            self.config.parse_if_needed(force=True) # Full reload
+
+    def _do_remove(self, section, option):
+        if not self.config.has_option(section, option):
+            raise AdminCommandError(
+                _("Option '%(option)s' doesn't exist in section '%(section)s'",
+                  option=option, section=section))
+        self.config.remove(section, option)
+        self.config.save()
+        if section == 'inherit' and option == 'file':
+            self.config.parse_if_needed(force=True) # Full reload
