@@ -31,7 +31,7 @@ has_psycopg = False
 try:
     import psycopg2 as psycopg
     import psycopg2.extensions
-    from psycopg2 import ProgrammingError as PGSchemaError
+    from psycopg2 import DataError, ProgrammingError
     from psycopg2.extensions import register_type, UNICODE, \
                                     register_adapter, AsIs, QuotedString
 
@@ -61,7 +61,7 @@ def assemble_pg_dsn(path, user=None, password=None, host=None, port=None):
 
 class PostgreSQLConnector(Component):
     """Database connector for PostgreSQL.
-    
+
     Database URLs should be of the form:
     {{{
     postgres://user[:password]@host[:port]/database[?schema=my_schema]
@@ -79,7 +79,7 @@ class PostgreSQLConnector(Component):
     def get_supported_schemes(self):
         if not has_psycopg:
             self.error = _("Cannot load Python bindings for PostgreSQL")
-        yield ('postgres', self.error and -1 or 1)
+        yield ('postgres', -1 if self.error else 1)
 
     def get_connection(self, path, log=None, user=None, password=None,
                        host=None, port=None, params={}):
@@ -92,15 +92,19 @@ class PostgreSQLConnector(Component):
             self.required = True
         return cnx
 
-    def init_db(self, path, log=None, user=None, password=None, host=None,
-                port=None, params={}):
+    def get_exceptions(self):
+        return psycopg
+
+    def init_db(self, path, schema=None, log=None, user=None, password=None,
+                host=None, port=None, params={}):
         cnx = self.get_connection(path, log, user, password, host, port,
                                   params)
         cursor = cnx.cursor()
         if cnx.schema:
             cursor.execute('CREATE SCHEMA "%s"' % cnx.schema)
             cursor.execute('SET search_path TO %s', (cnx.schema,))
-        from trac.db_default import schema
+        if schema is None:
+            from trac.db_default import schema
         for table in schema:
             for stmt in self.to_sql(table):
                 cursor.execute(stmt)
@@ -123,16 +127,16 @@ class PostgreSQLConnector(Component):
         sql.append(',\n'.join(coldefs) + '\n)')
         yield '\n'.join(sql)
         for index in table.indices:
-            unique = index.unique and 'UNIQUE' or ''
+            unique = 'UNIQUE' if index.unique else ''
             yield 'CREATE %s INDEX "%s_%s_idx" ON "%s" ("%s")' % \
-                    (unique, table.name, 
+                    (unique, table.name,
                      '_'.join(index.columns), table.name,
                      '","'.join(index.columns))
 
     def alter_column_types(self, table, columns):
         """Yield SQL statements altering the type of one or more columns of
         a table.
-        
+
         Type changes are specified as a `columns` dict mapping column names
         to `(from, to)` SQL type tuples.
         """
@@ -166,7 +170,19 @@ class PostgreSQLConnector(Component):
                 args.extend(['-p', str(db_prop.get('port', '5432'))])
 
         if 'schema' in db_params:
-            args.extend(['-n', '"%s"' % db_params['schema']])
+            try:
+                p = Popen([self.pg_dump_path, '--version'], stdout=PIPE,
+                          close_fds=close_fds)
+            except OSError, e:
+                raise TracError(_("Unable to run %(path)s: %(msg)s",
+                                  path=self.pg_dump_path,
+                                  msg=exception_to_unicode(e)))
+            # Need quote for -n (--schema) option in PostgreSQL 8.2+
+            version = p.communicate()[0]
+            if re.search(r' 8\.[01]\.', version):
+                args.extend(['-n', db_params['schema']])
+            else:
+                args.extend(['-n', '"%s"' % db_params['schema']])
 
         dest_file += ".gz"
         args.extend(['-f', dest_file, db_name])
@@ -200,7 +216,7 @@ class PostgreSQLConnection(ConnectionWrapper):
             path = path[1:]
         if 'host' in params:
             host = params['host']
-        
+
         cnx = psycopg.connect(assemble_pg_dsn(path, user, password, host,
                                               port))
 
@@ -211,7 +227,7 @@ class PostgreSQLConnection(ConnectionWrapper):
                 self.schema = params['schema']
                 cnx.cursor().execute('SET search_path TO %s', (self.schema,))
                 cnx.commit()
-        except PGSchemaError:
+        except (DataError, ProgrammingError):
             cnx.rollback()
         ConnectionWrapper.__init__(self, cnx, log)
 
@@ -231,15 +247,15 @@ class PostgreSQLConnection(ConnectionWrapper):
 
     def quote(self, identifier):
         """Return the quoted identifier."""
-        return '"%s"' % identifier
+        return '"%s"' % identifier.replace('"', '""')
 
     def get_last_id(self, cursor, table, column='id'):
-        cursor.execute("SELECT CURRVAL('%s_%s_seq')" % (table, column))
+        cursor.execute("""SELECT CURRVAL('"%s_%s_seq"')""" % (table, column))
         return cursor.fetchone()[0]
 
     def update_sequence(self, cursor, table, column='id'):
         cursor.execute("""
-            SELECT setval('%s_%s_seq', (SELECT MAX(id) FROM %s))
+            SELECT setval('"%s_%s_seq"', (SELECT MAX(id) FROM %s))
             """ % (table, column, table))
 
     def cursor(self):
