@@ -1,5 +1,7 @@
 # -*- coding: utf-8 -*-
 
+from __future__ import with_statement
+
 import os
 import unittest
 
@@ -10,13 +12,13 @@ from trac.util.concurrency import ThreadLocal
 
 
 class Connection(object):
-    
+
     committed = False
     rolledback = False
-    
+
     def commit(self):
         self.committed = True
-    
+
     def rollback(self):
         self.rolledback = True
 
@@ -25,33 +27,25 @@ class Error(Exception):
     pass
 
 
-class MockDatabaseManager(object):
-    def __init__(self):
-        self._transaction_local = ThreadLocal(db=None)
-        
-class MinimalEnv(object):
-    def __init__(self, db=None):
-        self.db = db
-        self.components = {DatabaseManager: MockDatabaseManager()}
-    def get_db_cnx(self):
-        if self.db is Connection:
-            return Connection()
-        return self.db
+def make_env(get_cnx):
+    return Mock(components={DatabaseManager:
+             Mock(get_connection=get_cnx,
+                  _transaction_local=ThreadLocal(wdb=None, rdb=None))})
 
 
 class WithTransactionTest(unittest.TestCase):
 
     def test_successful_transaction(self):
         db = Connection()
-        env = MinimalEnv(db)
+        env = make_env(lambda: db)
         @with_transaction(env)
         def do_transaction(db):
             self.assertTrue(not db.committed and not db.rolledback)
         self.assertTrue(db.committed and not db.rolledback)
-        
+
     def test_failed_transaction(self):
         db = Connection()
-        env = MinimalEnv(db)
+        env = make_env(lambda: db)
         try:
             @with_transaction(env)
             def do_transaction(db):
@@ -61,9 +55,9 @@ class WithTransactionTest(unittest.TestCase):
         except Error:
             pass
         self.assertTrue(not db.committed and db.rolledback)
-        
+
     def test_implicit_nesting_success(self):
-        env = MinimalEnv(Connection)
+        env = make_env(Connection)
         dbs = [None, None]
         @with_transaction(env)
         def level0(db):
@@ -78,7 +72,7 @@ class WithTransactionTest(unittest.TestCase):
         self.assertTrue(dbs[0].committed and not dbs[0].rolledback)
 
     def test_implicit_nesting_failure(self):
-        env = MinimalEnv(Connection)
+        env = make_env(Connection)
         dbs = [None, None]
         try:
             @with_transaction(env)
@@ -103,7 +97,7 @@ class WithTransactionTest(unittest.TestCase):
 
     def test_explicit_success(self):
         db = Connection()
-        env = MinimalEnv(None)
+        env = make_env(lambda: None)
         @with_transaction(env, db)
         def do_transaction(idb):
             self.assertTrue(idb is db)
@@ -112,7 +106,7 @@ class WithTransactionTest(unittest.TestCase):
 
     def test_explicit_failure(self):
         db = Connection()
-        env = MinimalEnv(None)
+        env = make_env(lambda: None)
         try:
             @with_transaction(env, db)
             def do_transaction(idb):
@@ -126,7 +120,7 @@ class WithTransactionTest(unittest.TestCase):
 
     def test_implicit_in_explicit_success(self):
         db = Connection()
-        env = MinimalEnv(Connection)
+        env = make_env(lambda: db)
         dbs = [None, None]
         @with_transaction(env, db)
         def level0(db):
@@ -142,7 +136,7 @@ class WithTransactionTest(unittest.TestCase):
 
     def test_implicit_in_explicit_failure(self):
         db = Connection()
-        env = MinimalEnv(Connection)
+        env = make_env(lambda: db)
         dbs = [None, None]
         try:
             @with_transaction(env, db)
@@ -163,7 +157,7 @@ class WithTransactionTest(unittest.TestCase):
 
     def test_explicit_in_implicit_success(self):
         db = Connection()
-        env = MinimalEnv(Connection)
+        env = make_env(lambda: db)
         dbs = [None, None]
         @with_transaction(env)
         def level0(db):
@@ -179,7 +173,7 @@ class WithTransactionTest(unittest.TestCase):
 
     def test_explicit_in_implicit_failure(self):
         db = Connection()
-        env = MinimalEnv(Connection)
+        env = make_env(lambda: db)
         dbs = [None, None]
         try:
             @with_transaction(env)
@@ -199,7 +193,7 @@ class WithTransactionTest(unittest.TestCase):
         self.assertTrue(not dbs[0].committed and dbs[0].rolledback)
 
     def test_invalid_nesting(self):
-        env = MinimalEnv(Connection)
+        env = make_env(Connection)
         try:
             @with_transaction(env)
             def level0(db):
@@ -282,36 +276,27 @@ class StringsTestCase(unittest.TestCase):
         self.env = EnvironmentStub()
 
     def test_insert_unicode(self):
-        db = self.env.get_db_cnx()
-        cursor = db.cursor()
-        cursor.execute('INSERT INTO system (name,value) VALUES (%s,%s)',
-                       ('test-unicode', u'ünicöde'))
-        db.commit()
-        cursor = db.cursor()
-        cursor.execute("SELECT value FROM system WHERE name='test-unicode'")
-        self.assertEqual([(u'ünicöde',)], cursor.fetchall())
+        self.env.db_transaction(
+                "INSERT INTO system (name,value) VALUES (%s,%s)",
+                ('test-unicode', u'ünicöde'))
+        self.assertEqual([(u'ünicöde',)], self.env.db_query(
+            "SELECT value FROM system WHERE name='test-unicode'"))
 
     def test_insert_empty(self):
         from trac.util.text import empty
-        db = self.env.get_db_cnx()
-        cursor = db.cursor()
-        cursor.execute('INSERT INTO system (name,value) VALUES (%s,%s)',
-                       ('test-empty', empty))
-        db.commit()
-        cursor = db.cursor()
-        cursor.execute("SELECT value FROM system WHERE name='test-empty'")
-        self.assertEqual([(u'',)], cursor.fetchall())
+        self.env.db_transaction(
+                "INSERT INTO system (name,value) VALUES (%s,%s)",
+                ('test-empty', empty))
+        self.assertEqual([(u'',)], self.env.db_query(
+            "SELECT value FROM system WHERE name='test-empty'"))
 
     def test_insert_markup(self):
         from genshi.core import Markup
-        db = self.env.get_db_cnx()
-        cursor = db.cursor()
-        cursor.execute('INSERT INTO system (name,value) VALUES (%s,%s)',
-                       ('test-markup', Markup(u'<em>märkup</em>')))
-        db.commit()
-        cursor = db.cursor()
-        cursor.execute("SELECT value FROM system WHERE name='test-markup'")
-        self.assertEqual([(u'<em>märkup</em>',)], cursor.fetchall())
+        self.env.db_transaction(
+                "INSERT INTO system (name,value) VALUES (%s,%s)",
+                ('test-markup', Markup(u'<em>märkup</em>')))
+        self.assertEqual([(u'<em>märkup</em>',)], self.env.db_query(
+            "SELECT value FROM system WHERE name='test-markup'"))
 
     def test_quote(self):
         db = self.env.get_db_cnx()
@@ -325,37 +310,36 @@ class StringsTestCase(unittest.TestCase):
 class ConnectionTestCase(unittest.TestCase):
     def setUp(self):
         self.env = EnvironmentStub()
-        self.db = self.env.get_db_cnx()
 
     def tearDown(self):
         self.env.reset_db()
 
     def test_get_last_id(self):
-        c = self.db.cursor()
+        id1 = id2 = None
         q = "INSERT INTO report (author) VALUES ('anonymous')"
-        c.execute(q)
-        # Row ID correct before...
-        id1 = self.db.get_last_id(c, 'report')
-        self.assertNotEqual(0, id1)
-        self.db.commit()
-        c.execute(q)
-        self.db.commit()
-        # ... and after commit()
-        id2 = self.db.get_last_id(c, 'report')
-        self.assertEqual(id1 + 1, id2)
+        with self.env.db_transaction as db:
+            cursor = db.cursor()
+            cursor.execute(q)
+            # Row ID correct before...
+            id1 = db.get_last_id(cursor, 'report')
+            self.assertNotEqual(0, id1)
+            db.commit()
+            cursor.execute(q)
+            # ... and after commit()
+            db.commit()
+            id2 = db.get_last_id(cursor, 'report')
+            self.assertEqual(id1 + 1, id2)
 
     def test_update_sequence(self):
-        cursor = self.db.cursor()
-        cursor.execute("""
-            INSERT INTO report (id, author) VALUES (42, 'anonymous')
-            """)
-        self.db.commit()
-        self.db.update_sequence(cursor, 'report', 'id')
-        self.db.commit()
-        cursor.execute("INSERT INTO report (author) VALUES ('next-id')")
-        self.db.commit()
-        cursor.execute("SELECT id FROM report WHERE author='next-id'")
-        self.assertEqual(43, cursor.fetchall()[0][0])
+        self.env.db_transaction(
+            "INSERT INTO report (id, author) VALUES (42, 'anonymous')")
+        with self.env.db_transaction as db:
+            cursor = db.cursor()
+            db.update_sequence(cursor, 'report', 'id')
+        self.env.db_transaction(
+            "INSERT INTO report (author) VALUES ('next-id')")
+        self.assertEqual(43, self.env.db_query(
+                "SELECT id FROM report WHERE author='next-id'")[0][0])
 
 
 def suite():
