@@ -1,7 +1,6 @@
 #!/usr/bin/python
 
 import os
-from trac.core import TracError
 from trac.tests.functional import *
 
 
@@ -11,8 +10,10 @@ class DatabaseBackupTestCase(FunctionalTestCaseSetup):
         env = self._testenv.get_trac_environment()
         # raises TracError if backup fails
         backup_file = env.backup()
-        self.assert_(os.path.exists(backup_file), 'Backup file was not created.')
-        self.assertNotEqual(os.path.getsize(backup_file), 0, 'Backup file is zero length.')
+        self.assertTrue(os.path.exists(backup_file),
+                        'Backup file was not created.')
+        self.assertNotEqual(os.path.getsize(backup_file), 0,
+                            'Backup file is zero length.')
 
 
 def functionalSuite(suite=None):
