@@ -31,11 +31,11 @@ class GetMimeTypeTestCase(unittest.TestCase):
     def test_from_suffix_using_MIME_MAP(self):
         self.assertEqual('text/plain', get_mimetype('README', None))
         self.assertEqual('text/plain', get_mimetype('README.txt', None))
-        
+
     def test_from_suffix_using_mimetypes(self):
         accepted = ('image/png', 'image/x-png')
         self.assertTrue(get_mimetype('doc/trac_logo.png', None) in accepted)
-        
+
     def test_from_content_using_CONTENT_RE(self):
         self.assertEqual('text/x-python',
                          get_mimetype('xxx', """
@@ -124,7 +124,7 @@ class GroupLinesTestCase(unittest.TestCase):
         self.assertEquals(lines[0].events, [(TEXT, "test", (None, -1, -1))])
 
     def test_simplespan(self):
-        input = HTMLParser(StringIO("<span>test</span>"))
+        input = HTMLParser(StringIO(u"<span>test</span>"), encoding=None)
         lines = list(_group_lines(input))
         self.assertEquals(len(lines), 1)
         self.assertTrue(isinstance(lines[0], Stream))
@@ -160,13 +160,14 @@ class GroupLinesTestCase(unittest.TestCase):
                 ]
         lines = list(_group_lines(input))
         self.assertEqual(len(lines), 0)
-                 
+
     def test_newline(self):
         """
         If the text element does not end with a newline, it's not properly
         closed.
         """
-        input = HTMLParser(StringIO('<span class="c">a\nb</span>'))
+        input = HTMLParser(StringIO(u'<span class="c">a\nb</span>'),
+            encoding=None)
         expected = ['<span class="c">a</span>',
                     '<span class="c">b</span>',
                    ]
@@ -180,7 +181,8 @@ class GroupLinesTestCase(unittest.TestCase):
         Same as test_newline above, but make sure it behaves properly wrt
         the trailing \\n, especially given it's inside an element.
         """
-        input = HTMLParser(StringIO('<span class="c">a\nb\n</span>'))
+        input = HTMLParser(StringIO(u'<span class="c">a\nb\n</span>'),
+            encoding=None)
         expected = ['<span class="c">a</span>',
                     '<span class="c">b</span>',
                    ]
@@ -193,7 +195,8 @@ class GroupLinesTestCase(unittest.TestCase):
         """
         ditto.
         """
-        input = HTMLParser(StringIO('<span class="c">\n\n\na</span>'))
+        input = HTMLParser(StringIO(u'<span class="c">\n\n\na</span>'),
+            encoding=None)
         expected = ['<span class="c"></span>',
                     '<span class="c"></span>',
                     '<span class="c"></span>',
