@@ -12,11 +12,10 @@
 # history and logs, available at http://trac.edgewall.org/log/.
 
 import os
-import re
 import unittest
 
 from genshi.core import Stream, TEXT
-from genshi.input import HTMLParser, XML
+from genshi.input import HTMLParser
 
 try:
     pygments = __import__('pygments', {}, {}, [])
@@ -24,33 +23,30 @@ try:
 except ImportError:
     have_pygments = False
 
-from trac.mimeview.api import Mimeview, Context
+from trac.mimeview.api import Mimeview, RenderingContext
 if have_pygments:
     from trac.mimeview.pygments import PygmentsRenderer
 from trac.test import EnvironmentStub, Mock
-from trac.web.chrome import Chrome
+from trac.web.chrome import Chrome, web_context
 from trac.web.href import Href
 
 
 class PygmentsRendererTestCase(unittest.TestCase):
 
-    if not hasattr(unittest.TestCase, "assertTrue"):
-        assertTrue = unittest.TestCase.failUnless   # Python 2.3 compatibility
-
     def setUp(self):
         self.env = EnvironmentStub(enable=[Chrome, PygmentsRenderer])
         self.pygments = Mimeview(self.env).renderers[0]
-        self.req = Mock(base_path='',chrome={}, args={},
+        self.req = Mock(base_path='', chrome={}, args={},
                         abs_href=Href('/'), href=Href('/'),
                         session={}, perm=None, authname=None, tz=None)
-        self.context = Context.from_request(self.req)
+        self.context = web_context(self.req)
         pygments_html = open(os.path.join(os.path.split(__file__)[0],
                                        'pygments.html'))
-        self.pygments_html = Stream(list(HTMLParser(pygments_html)))
+        self.pygments_html = Stream(list(HTMLParser(pygments_html, encoding='utf-8')))
 
     def _expected(self, expected_id):
         return self.pygments_html.select(
-            '//div[@id="%s"]/*|//div[@id="%s"]/text())' % 
+            '//div[@id="%s"]/*|//div[@id="%s"]/text())' %
             (expected_id, expected_id))
 
     def _test(self, expected_id, result):
@@ -78,22 +74,13 @@ def hello():
         """
         Simple Python highlighting with Pygments (through Mimeview.render)
         """
-        result = mimeview = Mimeview(self.env).render(self.context,
-                                                      'text/x-python', """
+        result = Mimeview(self.env).render(self.context, 'text/x-python', """
 def hello():
         return "Hello World!"
 """)
         self.assertTrue(result)
         self._test('python_hello_mimeview', result)
 
-    def test_empty_content(self):
-        """
-        Simple test for direct rendering of empty content.
-        """
-        result = self.pygments.render(self.context, 'text/x-python', '')
-        self.assertTrue(result)
-        self._test('empty_content', result)
-
     def test_newline_content(self):
         """
         The behavior of Pygments changed post-Pygments 0.11.1, and now
@@ -123,6 +110,19 @@ def hello():
         result = self.pygments.render(self.context, 'text/x-python', '')
         self.assertEqual(None, result)
 
+    def test_extra_mimetypes(self):
+        """
+        The text/x-ini mimetype is normally not known by Trac, but
+        Pygments supports it.
+        """
+        mimeview = Mimeview(self.env)
+        self.assertEqual('text/x-ini; charset=utf-8',
+                         mimeview.get_mimetype('file.ini'))
+        self.assertEqual('text/x-ini; charset=utf-8',
+                         mimeview.get_mimetype('file.cfg'))
+        self.assertEqual('text/x-ini; charset=utf-8',
+                         mimeview.get_mimetype('file.text/x-ini'))
+
 def suite():
     suite = unittest.TestSuite()
     if have_pygments:
