@@ -13,6 +13,22 @@ from trac.resource import Resource, resource_exists
 from trac.test import EnvironmentStub
 
 
+hashes = {
+    '42': '92cfceb39d57d914ed8b14d0e37643de0797ae56',
+    'Foo.Mp3': '95797b6eb253337ff2c54e0881e2b747ec394f51',
+    'SomePage': 'd7e80bae461ca8568e794792f5520b603f540e06',
+    'Teh bar.jpg': 'ed9102c4aa099e92baf1073f824d21c5e4be5944',
+    'Teh foo.txt': 'ab97ba98d98fcf72b92e33a66b07077010171f70',
+    'bar.7z': '6c9600ad4d59ac864e6f0d2030c1fc76b4b406cb',
+    'bar.jpg': 'ae0faa593abf2b6f8871f6f32fe5b28d1c6572be',
+    'foo.$$$': 'eefc6aa745dbe129e8067a4a57637883edd83a8a',
+    'foo.2.txt': 'a8fcfcc2ef4e400ee09ae53c1aabd7f5a5fda0c7',
+    'foo.txt': '9206ac42b532ef8e983470c251f4e1a365fd636c',
+    u'bar.aäc': '70d0e3b813fdc756602d82748719a3ceb85cbf29',
+    u'ÜberSicht': 'a16c6837f6d3d2cc3addd68976db1c55deb694c8',
+}
+
+
 class TicketOnlyViewsTicket(Component):
     implements(IPermissionPolicy)
 
@@ -29,7 +45,8 @@ class AttachmentTestCase(unittest.TestCase):
         self.env = EnvironmentStub()
         self.env.path = os.path.join(tempfile.gettempdir(), 'trac-tempenv')
         os.mkdir(self.env.path)
-        self.attachments_dir = os.path.join(self.env.path, 'attachments')
+        self.attachments_dir = os.path.join(self.env.path, 'files',
+                                            'attachments')
         self.env.config.set('trac', 'permission_policies',
                             'TicketOnlyViewsTicket, LegacyAttachmentPolicy')
         self.env.config.set('attachment', 'max_size', 512)
@@ -43,25 +60,59 @@ class AttachmentTestCase(unittest.TestCase):
     def test_get_path(self):
         attachment = Attachment(self.env, 'ticket', 42)
         attachment.filename = 'foo.txt'
-        self.assertEqual(os.path.join(self.attachments_dir, 'ticket', '42',
-                                      'foo.txt'),
+        self.assertEqual(os.path.join(self.attachments_dir, 'ticket',
+                                      hashes['42'][0:3], hashes['42'],
+                                      hashes['foo.txt'] + '.txt'),
                          attachment.path)
         attachment = Attachment(self.env, 'wiki', 'SomePage')
         attachment.filename = 'bar.jpg'
-        self.assertEqual(os.path.join(self.attachments_dir, 'wiki', 'SomePage',
-                                      'bar.jpg'),
+        self.assertEqual(os.path.join(self.attachments_dir, 'wiki',
+                                      hashes['SomePage'][0:3],
+                                      hashes['SomePage'],
+                                      hashes['bar.jpg'] + '.jpg'),
+                         attachment.path)
+
+    def test_path_extension(self):
+        attachment = Attachment(self.env, 'ticket', 42)
+        attachment.filename = 'Foo.Mp3'
+        self.assertEqual(os.path.join(self.attachments_dir, 'ticket',
+                                      hashes['42'][0:3], hashes['42'],
+                                      hashes['Foo.Mp3'] + '.Mp3'),
+                         attachment.path)
+        attachment = Attachment(self.env, 'wiki', 'SomePage')
+        attachment.filename = 'bar.7z'
+        self.assertEqual(os.path.join(self.attachments_dir, 'wiki',
+                                      hashes['SomePage'][0:3],
+                                      hashes['SomePage'],
+                                      hashes['bar.7z'] + '.7z'),
+                         attachment.path)
+        attachment = Attachment(self.env, 'ticket', 42)
+        attachment.filename = 'foo.$$$'
+        self.assertEqual(os.path.join(self.attachments_dir, 'ticket',
+                                      hashes['42'][0:3], hashes['42'],
+                                      hashes['foo.$$$']),
+                         attachment.path)
+        attachment = Attachment(self.env, 'wiki', 'SomePage')
+        attachment.filename = u'bar.aäc'
+        self.assertEqual(os.path.join(self.attachments_dir, 'wiki',
+                                      hashes['SomePage'][0:3],
+                                      hashes['SomePage'],
+                                      hashes[u'bar.aäc']),
                          attachment.path)
 
     def test_get_path_encoded(self):
         attachment = Attachment(self.env, 'ticket', 42)
         attachment.filename = 'Teh foo.txt'
-        self.assertEqual(os.path.join(self.attachments_dir, 'ticket', '42',
-                                      'Teh%20foo.txt'),
+        self.assertEqual(os.path.join(self.attachments_dir, 'ticket',
+                                      hashes['42'][0:3], hashes['42'],
+                                      hashes['Teh foo.txt'] + '.txt'),
                          attachment.path)
         attachment = Attachment(self.env, 'wiki', u'ÜberSicht')
         attachment.filename = 'Teh bar.jpg'
         self.assertEqual(os.path.join(self.attachments_dir, 'wiki',
-                                      '%C3%9CberSicht', 'Teh%20bar.jpg'),
+                                      hashes[u'ÜberSicht'][0:3],
+                                      hashes[u'ÜberSicht'],
+                                      hashes['Teh bar.jpg'] + '.jpg'),
                          attachment.path)
 
     def test_select_empty(self):
@@ -88,6 +139,11 @@ class AttachmentTestCase(unittest.TestCase):
         attachment = Attachment(self.env, 'ticket', 42)
         attachment.insert('foo.txt', StringIO(''), 0)
         self.assertEqual('foo.2.txt', attachment.filename)
+        self.assertEqual(os.path.join(self.attachments_dir, 'ticket',
+                                      hashes['42'][0:3], hashes['42'],
+                                      hashes['foo.2.txt'] + '.txt'),
+                         attachment.path)
+        self.assert_(os.path.exists(attachment.path))
 
     def test_insert_outside_attachments_dir(self):
         attachment = Attachment(self.env, '../../../../../sth/private', 42)
@@ -141,7 +197,7 @@ class AttachmentTestCase(unittest.TestCase):
         self.assertEqual('ticket', attachment1.resource.parent.realm)
         self.assertEqual('123', attachment1.parent_id)
         self.assertEqual('123', attachment1.resource.parent.id)
-        
+
         attachments = Attachment.select(self.env, 'wiki', 'SomePage')
         self.assertEqual(1, len(list(attachments)))
         attachments = Attachment.select(self.env, 'ticket', 123)
