@@ -12,6 +12,8 @@
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at http://trac.edgewall.org/log/.
 
+from __future__ import with_statement
+
 import os
 import tempfile
 import time
@@ -39,11 +41,8 @@ class ConfigurationTestCase(unittest.TestCase):
         return Configuration(self.filename)
 
     def _write(self, lines):
-        fileobj = open(self.filename, 'w')
-        try:
+        with open(self.filename, 'w') as fileobj:
             fileobj.write(('\n'.join(lines + [''])).encode('utf-8'))
-        finally:
-            fileobj.close()
 
     def test_default(self):
         config = self._read()
@@ -186,18 +185,28 @@ class ConfigurationTestCase(unittest.TestCase):
         self.assertEquals(['', 'bar', 'baz'],
                           config.getlist('a', 'option', keep_empty=True))
 
+    def test_read_and_getlist_false_values(self):
+        config = self._read()
+        values = [None, False, '', 'foo', u'', u'bar',
+                  0, 0L, 0.0, 0j, 42, 43.0]
+        self.assertEquals([False, 'foo', u'bar', 0, 0L, 0.0, 0j, 42, 43.0],
+                          config.getlist('a', 'false', values))
+        self.assertEquals(values, config.getlist('a', 'false', values,
+                                                 keep_empty=True))
+
     def test_read_and_choice(self):
         self._write(['[a]', 'option = 2', 'invalid = d'])
         config = self._read()
 
         class Foo(object):
-            option = ChoiceOption('a', 'option', ['Item1', 2, '3'])
-            other = ChoiceOption('a', 'other', [1, 2, 3])
-            invalid = ChoiceOption('a', 'invalid', ['a', 'b', 'c'])
-        
+            # enclose in parentheses to avoid messages extraction
+            option = (ChoiceOption)('a', 'option', ['Item1', 2, '3'])
+            other = (ChoiceOption)('a', 'other', [1, 2, 3])
+            invalid = (ChoiceOption)('a', 'invalid', ['a', 'b', 'c'])
+
             def __init__(self):
                 self.config = config
-        
+
         foo = Foo()
         self.assertEquals('2', foo.option)
         self.assertEquals('1', foo.other)
@@ -233,13 +242,13 @@ class ConfigurationTestCase(unittest.TestCase):
         self.assertEquals(['# -*- coding: utf-8 -*-\n',
                            '\n',
                            '[aä]\n',
-                           "option1 = Voilà l'été\n", 
-                           "option2 = Voilà l'été\n", 
-                           'öption0 = x\n', 
-                           # "option3 = VoilÃ  l'Ã©tÃ©\n", 
+                           "option1 = Voilà l'été\n",
+                           "option2 = Voilà l'été\n",
+                           'öption0 = x\n',
+                           # "option3 = VoilÃ  l'Ã©tÃ©\n",
                            '\n',
                            '[b]\n',
-                           'öption0 = y\n', 
+                           'öption0 = y\n',
                            '\n'],
                           configfile.readlines())
         configfile.close()
@@ -263,11 +272,11 @@ class ConfigurationTestCase(unittest.TestCase):
             self.assertEquals(['# -*- coding: utf-8 -*-\n',
                                '\n',
                                '[a]\n',
-                               "option1 = Voilà l'été\n", 
-                               "option2 = Voilà l'été\n", 
+                               "option1 = Voilà l'été\n",
+                               "option2 = Voilà l'été\n",
                                '\n',
                                '[inherit]\n',
-                               "file = trac-site.ini\n", 
+                               "file = trac-site.ini\n",
                                '\n'],
                               configfile.readlines())
             configfile.close()
@@ -293,20 +302,26 @@ class ConfigurationTestCase(unittest.TestCase):
         self._write(['[a]', 'option = x', '[b]', 'option = y'])
         config = self._read()
         self.assertEquals(['a', 'b'], config.sections())
-        
+
         class Foo(object):
+            # enclose in parentheses to avoid messages extraction
+            section_c = (ConfigSection)('c', 'Doc for c')
             option_c = Option('c', 'option', 'value')
-        
+
         self.assertEquals(['a', 'b', 'c'], config.sections())
+        foo = Foo()
+        foo.config = config
+        self.assert_(foo.section_c is config['c'])
+        self.assertEquals('value', foo.section_c.get('option'))
 
     def test_sections_unicode(self):
         self._write([u'[aä]', u'öption = x', '[b]', 'option = y'])
         config = self._read()
         self.assertEquals([u'aä', 'b'], config.sections())
-        
+
         class Foo(object):
             option_c = Option(u'cä', 'option', 'value')
-        
+
         self.assertEquals([u'aä', 'b', u'cä'], config.sections())
 
     def test_options(self):
@@ -318,10 +333,10 @@ class ConfigurationTestCase(unittest.TestCase):
         self.assertEquals('option', iter(config['a']).next())
         self.assertEquals('option', iter(config['b']).next())
         self.assertRaises(StopIteration, iter(config['c']).next)
-        
+
         class Foo(object):
             option_a = Option('a', 'b', 'c')
-        
+
         self.assertEquals([('option', 'x'), ('b', 'c')],
                                 list(config.options('a')))
 
@@ -332,10 +347,10 @@ class ConfigurationTestCase(unittest.TestCase):
         self.assertEquals(('option', 'y'), iter(config.options('b')).next())
         self.assertRaises(StopIteration, iter(config.options('c')).next)
         self.assertEquals(u'öption', iter(config['ä']).next())
-        
+
         class Foo(object):
             option_a = Option(u'ä', u'öption2', 'c')
-        
+
         self.assertEquals([(u'öption', 'x'), (u'öption2', 'c')],
                                 list(config.options(u'ä')))
 
@@ -350,7 +365,7 @@ class ConfigurationTestCase(unittest.TestCase):
 
         class Foo(object):
             option_a = Option('a', 'option2', 'x2')
-        
+
         self.assertEquals(True, config.has_option('a', 'option2'))
 
     def test_has_option_unicode(self):
@@ -364,7 +379,7 @@ class ConfigurationTestCase(unittest.TestCase):
 
         class Foo(object):
             option_a = Option(u'ä', u'öption2', 'x2')
-        
+
         self.assertEquals(True, config.has_option(u'ä', u'öption2'))
 
     def test_reparse(self):
@@ -428,12 +443,9 @@ class ConfigurationTestCase(unittest.TestCase):
 
     def _test_with_inherit(self, testcb):
         sitename = os.path.join(tempfile.gettempdir(), 'trac-site.ini')
-        sitefile = open(sitename, 'w')
         try:
-            try:
+            with open(sitename, 'w') as sitefile:
                 sitefile.write('[a]\noption = x\n')
-            finally:
-                sitefile.close()
 
             self._write(['[inherit]', 'file = trac-site.ini'])
             testcb()
