@@ -1,9 +1,7 @@
+# -*- encoding: utf-8 -*-
 #!/usr/bin/python
 import os
-from subprocess import call
-from tempfile import mkdtemp
 from trac.tests.functional import *
-from trac.util.datefmt import format_date, utc
 
 
 class RegressionTestRev6017(FunctionalTwillTestCaseSetup):
@@ -154,6 +152,50 @@ class RegressionTestTicket7209(FunctionalTwillTestCaseSetup):
         tc.notfind('Second Attachment')
 
 
+class RegressionTestTicket9880(FunctionalTwillTestCaseSetup):
+    def runTest(self):
+        """Test for regression of http://trac.edgewall.org/ticket/9880
+
+        Upload of a file which the browsers associates a Content-Type
+        of multipart/related (e.g. an .mht file) should succeed.
+        """
+        summary = random_sentence(5)
+        ticketid = self._tester.create_ticket(summary)
+        self._tester.create_ticket()
+        self._tester.attach_file_to_ticket(ticketid, tempfilename='hello.mht',
+                                           content_type='multipart/related',
+                                           data="""
+Well, the actual content of the file doesn't matter, the problem is
+related to the "multipart/..." content_type associated to the file.
+See also http://bugs.python.org/issue15564.
+""")
+
+
+class ErrorPageValidation(FunctionalTwillTestCaseSetup):
+    def runTest(self):
+        """Validate the error page"""
+        url = self._tester.url + '/wiki/WikiStart'
+        tc.go(url + '?version=bug')
+        tc.url(url)
+        tc.find(internal_error)
+
+
+class RegressionTestTicket3663(FunctionalTwillTestCaseSetup):
+    def runTest(self):
+        """Regression test for non-UTF-8 PATH_INFO (#3663)
+
+        Verify that URLs not encoded with UTF-8 are reported as invalid.
+        """
+        # invalid PATH_INFO
+        self._tester.go_to_wiki(u'été'.encode('latin1'))
+        tc.code(404)
+        tc.find('Invalid URL encoding')
+        # invalid SCRIPT_NAME
+        tc.go(u'été'.encode('latin1'))
+        tc.code(404)
+        tc.find('Invalid URL encoding')
+
+
 def functionalSuite():
     suite = FunctionalTestSuite()
     return suite
@@ -168,6 +210,9 @@ def suite():
     suite.addTest(RegressionTestTicket3833c())
     suite.addTest(RegressionTestTicket5572())
     suite.addTest(RegressionTestTicket7209())
+    suite.addTest(RegressionTestTicket9880())
+    suite.addTest(ErrorPageValidation())
+    suite.addTest(RegressionTestTicket3663())
 
     import trac.versioncontrol.tests
     trac.versioncontrol.tests.functionalSuite(suite)
