@@ -3,17 +3,10 @@
 working with a Trac environment to make test cases more succinct.
 """
 
-import os
-import re
-from datetime import datetime, timedelta
-from subprocess import call, Popen, PIPE
-from tempfile import mkdtemp
-
-from trac.tests.functional import internal_error, logfile, close_fds, rmtree
+from trac.tests.functional import internal_error
 from trac.tests.functional.better_twill import tc, b
 from trac.tests.contentgen import random_page, random_sentence, random_word, \
     random_unique_camel
-from trac.util.datefmt import format_date, utc
 from trac.util.text import unicode_quote
 
 try:
@@ -26,7 +19,7 @@ class FunctionalTester(object):
     test environment.
 
     It makes assumptions such as knowing what ticket number is next, so
-    avoid doing things manually in :class:`FunctionalTestCase`s when you can.
+    avoid doing things manually in a :class:`FunctionalTestCase` when you can.
     """
 
     def __init__(self, url):
@@ -61,9 +54,9 @@ class FunctionalTester(object):
         """Create a new (random) ticket in the test environment.  Returns
         the new ticket number.
 
-        :summary:
+        :param summary:
             may optionally be set to the desired summary
-        :info:
+        :param info:
             may optionally be set to a dictionary of field value pairs for
             populating the ticket.  ``info['summary']`` overrides summary.
 
@@ -154,11 +147,14 @@ class FunctionalTester(object):
         tc.formvalue('propertyform', 'comment', comment)
         tc.submit("submit")
         # Verify we're where we're supposed to be.
-        tc.url(self.url + '/ticket/%s#comment:.*' % ticketid)
+        # The fragment is stripped since Python 2.7.1, see:
+        # http://trac.edgewall.org/ticket/9990#comment:18
+        tc.url(self.url + '/ticket/%s(?:#comment:.*)?$' % ticketid)
         return comment
 
     def attach_file_to_ticket(self, ticketid, data=None, tempfilename=None,
-                              description=None, replace=False):
+                              description=None, replace=False,
+                              content_type=None):
         """Attaches a file to the given ticket id, with random data if none is
         provided.  Assumes the ticket exists.
         """
@@ -177,12 +173,14 @@ class FunctionalTester(object):
         tc.url(self.url + "/attachment/ticket/" \
                "%s/\\?action=new&attachfilebutton=Attach\\+file" % ticketid)
         fp = StringIO(data)
-        tc.formfile('attachment', 'attachment', tempfilename, fp=fp)
+        tc.formfile('attachment', 'attachment', tempfilename,
+                    content_type=content_type, fp=fp)
         tc.formvalue('attachment', 'description', description)
         if replace:
             tc.formvalue('attachment', 'replace', True)
         tc.submit()
         tc.url(self.url + '/attachment/ticket/%s/$' % ticketid)
+        return tempfilename
 
     def clone_ticket(self, ticketid):
         """Create a clone of the given ticket id using the clone button."""
@@ -205,7 +203,7 @@ class FunctionalTester(object):
         page_url = self.url + "/wiki/" + page
         tc.go(page_url)
         tc.url(page_url)
-        tc.find("Describe %s here." % page)
+        tc.find("The page %s does not exist." % page)
         tc.formvalue('modifypage', 'action', 'edit')
         tc.submit()
         tc.url(page_url + '\\?action=edit')
@@ -220,12 +218,14 @@ class FunctionalTester(object):
         tc.submit()
         tc.find(page + ".*created")
 
-    def attach_file_to_wiki(self, name, data=None):
+    def attach_file_to_wiki(self, name, data=None, tempfilename=None):
         """Attaches a file to the given wiki page, with random content if none
         is provided.  Assumes the wiki page exists.
         """
         if data == None:
             data = random_page()
+        if tempfilename is None:
+            tempfilename = random_word()
         self.go_to_wiki(name)
         # set the value to what it already is, so that twill will know we
         # want this form.
@@ -233,21 +233,19 @@ class FunctionalTester(object):
         tc.submit()
         tc.url(self.url + "/attachment/wiki/" \
                "%s/\\?action=new&attachfilebutton=Attach\\+file" % name)
-        tempfilename = random_word()
         fp = StringIO(data)
         tc.formfile('attachment', 'attachment', tempfilename, fp=fp)
         tc.formvalue('attachment', 'description', random_sentence())
         tc.submit()
         tc.url(self.url + '/attachment/wiki/%s/$' % name)
+        return tempfilename
 
     def create_milestone(self, name=None, due=None):
         """Creates the specified milestone, with a random name if none is
         provided.  Returns the name of the milestone.
         """
-        find = False
         if name == None:
             name = random_unique_camel()
-            find = True
         milestone_url = self.url + "/admin/ticket/milestones"
         tc.go(milestone_url)
         tc.url(milestone_url)
