@@ -24,7 +24,7 @@ from trac.config import *
 from trac.core import *
 from trac.perm import IPermissionRequestor, PermissionCache, PermissionSystem
 from trac.resource import IResourceManager
-from trac.util import Ranges
+from trac.util import Ranges, as_int
 from trac.util.text import shorten_line
 from trac.util.translation import _, N_, gettext
 from trac.wiki import IWikiSyntaxProvider, WikiParser
@@ -47,7 +47,7 @@ class ITicketActionController(Interface):
         `action` is a key used to identify that particular action.
         (note that 'history' and 'diff' are reserved and should not be used
         by plugins)
-        
+
         The actions will be presented on the page in descending order of the
         integer weight. The first action in the list is used as the default
         action.
@@ -69,7 +69,7 @@ class ITicketActionController(Interface):
         `label` is a short text that will be used when listing the action,
         `control` is the markup for the action control and `hint` should
         explain what will happen if this action is taken.
-        
+
         This method will only be called if the controller claimed to handle
         the given `action` in the call to `get_ticket_actions`.
 
@@ -114,7 +114,7 @@ class ITicketChangeListener(Interface):
 
     def ticket_changed(ticket, comment, author, old_values):
         """Called when a ticket is modified.
-        
+
         `old_values` is a dictionary containing the previous values of the
         fields that have changed.
         """
@@ -132,7 +132,7 @@ class ITicketManipulator(Interface):
 
     def validate_ticket(req, ticket):
         """Validate a ticket after it's been populated from user input.
-        
+
         Must return a list of `(field, message)` tuples, one for each problem
         detected. `field` can be `None` to indicate an overall problem with the
         ticket. Therefore, a return value of `[]` means everything is OK."""
@@ -162,7 +162,11 @@ class TicketSystem(Component):
 
     change_listeners = ExtensionPoint(ITicketChangeListener)
     milestone_change_listeners = ExtensionPoint(IMilestoneChangeListener)
-    
+
+    ticket_custom_section = ConfigSection('ticket-custom',
+        """In this section, you can define additional fields for tickets. See
+        TracTicketsCustomFields for more details.""")
+
     action_controllers = OrderedExtensionsOption('ticket', 'workflow',
         ITicketActionController, default='ConfigurableTicketWorkflow',
         include_missing=False,
@@ -174,7 +178,7 @@ class TicketSystem(Component):
         Be sure to understand the performance implications before activating
         this option. See
         [TracTickets#Assign-toasDrop-DownList Assign-to as Drop-Down List].
-        
+
         Please note that e-mail addresses are '''not''' obfuscated in the
         resulting drop-down menu, so this option should not be used if
         e-mail addresses must remain protected.
@@ -207,7 +211,7 @@ class TicketSystem(Component):
     default_keywords = Option('ticket', 'default_keywords', '',
         """Default keywords for newly created tickets.""")
 
-    default_owner = Option('ticket', 'default_owner', '',
+    default_owner = Option('ticket', 'default_owner', '< default >',
         """Default owner for newly created tickets.""")
 
     default_cc = Option('ticket', 'default_cc', '',
@@ -218,7 +222,7 @@ class TicketSystem(Component):
         (''since 0.11'').""")
 
     def __init__(self):
-        self.log.debug('action controllers for ticket workflow: %r' % 
+        self.log.debug('action controllers for ticket workflow: %r' %
                 [c.__class__.__name__ for c in self.action_controllers])
 
     # Public API
@@ -248,8 +252,10 @@ class TicketSystem(Component):
 
     def get_ticket_field_labels(self):
         """Produce a (name,label) mapping from `get_ticket_fields`."""
-        return dict((f['name'], f['label']) for f in
-                    TicketSystem(self.env).get_ticket_fields())
+        labels = dict((f['name'], f['label'])
+                      for f in self.get_ticket_fields())
+        labels['attachment'] = _("Attachment")
+        return labels
 
     def get_ticket_fields(self):
         """Returns list of fields available for tickets.
@@ -282,7 +288,7 @@ class TicketSystem(Component):
         fields.append({'name': 'reporter', 'type': 'text',
                        'label': N_('Reporter')})
 
-        # Owner field, by default text but can be changed dynamically 
+        # Owner field, by default text but can be changed dynamically
         # into a drop-down depending on configuration (restrict_owner=true)
         field = {'name': 'owner', 'label': N_('Owner')}
         field['type'] = 'text'
@@ -318,9 +324,10 @@ class TicketSystem(Component):
             fields.append(field)
 
         # Advanced text fields
-        fields.append({'name': 'keywords', 'type': 'text',
+        fields.append({'name': 'keywords', 'type': 'text', 'format': 'list',
                        'label': N_('Keywords')})
-        fields.append({'name': 'cc', 'type': 'text', 'label': N_('Cc')})
+        fields.append({'name': 'cc', 'type': 'text',  'format': 'list',
+                       'label': N_('Cc')})
 
         # Date/time fields
         fields.append({'name': 'time', 'type': 'time',
@@ -357,7 +364,7 @@ class TicketSystem(Component):
     def custom_fields(self, db):
         """Return the list of custom ticket fields available for tickets."""
         fields = []
-        config = self.config['ticket-custom']
+        config = self.ticket_custom_section
         for name in [option for option, value in config.options()
                      if '.' not in option]:
             field = {
@@ -404,6 +411,7 @@ class TicketSystem(Component):
                                                            ticket.resource):
                     possible_owners.append(user)
             possible_owners.sort()
+            possible_owners.insert(0, '< default >')
             field['options'] = possible_owners
             field['optional'] = True
 
@@ -412,12 +420,13 @@ class TicketSystem(Component):
     def get_permission_actions(self):
         return ['TICKET_APPEND', 'TICKET_CREATE', 'TICKET_CHGPROP',
                 'TICKET_VIEW', 'TICKET_EDIT_CC', 'TICKET_EDIT_DESCRIPTION',
-                'TICKET_EDIT_COMMENT',
+                'TICKET_EDIT_COMMENT', 'TICKET_BATCH_MODIFY',
                 ('TICKET_MODIFY', ['TICKET_APPEND', 'TICKET_CHGPROP']),
                 ('TICKET_ADMIN', ['TICKET_CREATE', 'TICKET_MODIFY',
                                   'TICKET_VIEW', 'TICKET_EDIT_CC',
                                   'TICKET_EDIT_DESCRIPTION',
-                                  'TICKET_EDIT_COMMENT'])]
+                                  'TICKET_EDIT_COMMENT',
+                                  'TICKET_BATCH_MODIFY'])]
 
     # IWikiSyntaxProvider methods
 
@@ -449,22 +458,26 @@ class TicketSystem(Component):
                 from trac.ticket.model import Ticket
                 if Ticket.id_is_valid(num) and \
                         'TICKET_VIEW' in formatter.perm(ticket):
-                    # TODO: watch #6436 and when done, attempt to retrieve 
-                    #       ticket directly (try: Ticket(self.env, num) ...)
-                    cursor = formatter.db.cursor() 
-                    cursor.execute("SELECT type,summary,status,resolution "
-                                   "FROM ticket WHERE id=%s", (str(num),)) 
-                    for type, summary, status, resolution in cursor:
+                    # TODO: attempt to retrieve ticket view directly,
+                    #       something like: t = Ticket.view(num)
+                    for type, summary, status, resolution in \
+                            self.env.db_query("""
+                            SELECT type, summary, status, resolution
+                            FROM ticket WHERE id=%s
+                            """, (str(num),)):
                         title = self.format_summary(summary, status,
                                                     resolution, type)
                         href = formatter.href.ticket(num) + params + fragment
-                        return tag.a(label, class_='%s ticket' % status, 
-                                     title=title, href=href)
+                        return tag.a(label, title=title, href=href,
+                                     class_='%s ticket' % status)
             else:
                 ranges = str(r)
                 if params:
                     params = '&' + params[1:]
-                return tag.a(label, title='Tickets '+ranges,
+                label_wrap = label.replace(',', u',\u200b')
+                ranges_wrap = ranges.replace(',', u', ')
+                return tag.a(label_wrap,
+                             title=_("Tickets %(ranges)s", ranges=ranges_wrap),
                              href=formatter.href.query(id=ranges) + params)
         except ValueError:
             pass
@@ -483,14 +496,21 @@ class TicketSystem(Component):
             resource = formatter.resource
             cnum = target
 
-        if resource:
-            href = "%s#comment:%s" % (formatter.href.ticket(resource.id), cnum)
-            title = _("Comment %(cnum)s for Ticket #%(id)s", cnum=cnum,
-                      id=resource.id)
-            return tag.a(label, href=href, title=title)
-        else:
-            return label
- 
+        if resource and resource.realm == 'ticket':
+            id = as_int(resource.id, None)
+            if id is not None:
+                href = "%s#comment:%s" % (formatter.href.ticket(resource.id),
+                                          cnum)
+                title = _("Comment %(cnum)s for Ticket #%(id)s", cnum=cnum,
+                          id=resource.id)
+                if 'TICKET_VIEW' in formatter.perm(resource):
+                    for status, in self.env.db_query(
+                            "SELECT status FROM ticket WHERE id=%s", (id,)):
+                        return tag.a(label, href=href, title=title,
+                                     class_=status)
+                return tag.a(label, href=href, title=title)
+        return label
+
     # IResourceManager methods
 
     def get_resource_realms(self):
@@ -535,16 +555,13 @@ class TicketSystem(Component):
         >>> resource_exists(env, t.resource)
         True
         """
-        db = self.env.get_read_db()
-        cursor = db.cursor()
-        cursor.execute("SELECT id FROM ticket WHERE id=%s", (resource.id,))
-        latest_exists = bool(cursor.fetchall())
-        if latest_exists:
+        if self.env.db_query("SELECT id FROM ticket WHERE id=%s",
+                             (resource.id,)):
             if resource.version is None:
                 return True
-            cursor.execute("""
-                SELECT count(distinct time) FROM ticket_change WHERE ticket=%s
+            revcount = self.env.db_query("""
+                SELECT count(DISTINCT time) FROM ticket_change WHERE ticket=%s
                 """, (resource.id,))
-            return cursor.fetchone()[0] >= resource.version
+            return revcount[0][0] >= resource.version
         else:
             return False
