@@ -3,7 +3,7 @@ import unittest
 
 import trac.ticket
 from trac.ticket.tests import api, model, query, wikisyntax, notification, \
-                              conversion, report, roadmap
+                              conversion, report, roadmap, batch
 from trac.ticket.tests.functional import functionalSuite
 
 def suite():
@@ -16,7 +16,10 @@ def suite():
     suite.addTest(conversion.suite())
     suite.addTest(report.suite())
     suite.addTest(roadmap.suite())
+    suite.addTest(batch.suite())
+    suite.addTest(doctest.DocTestSuite(trac.ticket.api))
     suite.addTest(doctest.DocTestSuite(trac.ticket.report))
+    suite.addTest(doctest.DocTestSuite(trac.ticket.roadmap))
     return suite
 
 if __name__ == '__main__':
