@@ -1,3 +1,5 @@
+from __future__ import with_statement
+
 from datetime import datetime, timedelta
 import os.path
 from StringIO import StringIO
@@ -9,10 +11,12 @@ from trac import core
 from trac.attachment import Attachment
 from trac.core import TracError, implements
 from trac.resource import ResourceNotFound
-from trac.ticket.model import Ticket, Component, Milestone, Priority, Type, \
-                              Version
-from trac.ticket.api import IMilestoneChangeListener, ITicketChangeListener, \
-                            TicketSystem
+from trac.ticket.model import (
+    Ticket, Component, Milestone, Priority, Type, Version
+)
+from trac.ticket.api import (
+    IMilestoneChangeListener, ITicketChangeListener, TicketSystem
+)
 from trac.test import EnvironmentStub
 from trac.util.datefmt import from_utimestamp, to_utimestamp, utc
 
@@ -31,7 +35,7 @@ class TestTicketChangeListener(core.Component):
         self.comment = comment
         self.author = author
         self.old_values = old_values
-        
+
     def ticket_deleted(self, ticket):
         self.action = 'deleted'
         self.ticket = ticket
@@ -135,10 +139,10 @@ class TicketTestCase(unittest.TestCase):
     def test_can_save_ticket_without_explicit_comment(self):
         ticket = Ticket(self.env)
         ticket.insert()
-        
+
         ticket['summary'] = 'another summary'
         ticket.save_changes('foo')
-        
+
         changes = ticket.get_changelog()
         comment_change = [c for c in changes if c[2] == 'comment'][0]
         self.assertEqual('1', comment_change[3])
@@ -147,13 +151,29 @@ class TicketTestCase(unittest.TestCase):
     def test_can_save_ticket_without_explicit_username(self):
         ticket = Ticket(self.env)
         ticket.insert()
-        
+
         ticket['summary'] = 'another summary'
         ticket.save_changes()
-        
+
         for change in ticket.get_changelog():
             self.assertEqual(None, change[1])
 
+    def test_comment_with_whitespace_only_is_not_saved(self):
+        ticket = Ticket(self.env)
+        ticket.insert()
+
+        ticket.save_changes(comment='\n \n ')
+        self.assertEqual(0, len(ticket.get_changelog()))
+
+    def test_prop_whitespace_change_is_not_saved(self):
+        ticket = Ticket(self.env)
+        ticket.populate({'summary': 'ticket summary'})
+        ticket.insert()
+
+        ticket['summary'] = ' ticket summary '
+        ticket.save_changes()
+        self.assertEqual(0, len(ticket.get_changelog()))
+
     def test_ticket_default_values(self):
         """
         Verify that a ticket uses default values specified in the configuration
@@ -319,14 +339,11 @@ class TicketTestCase(unittest.TestCase):
         t1 = datetime(2001, 1, 1, 1, 1, 1, 0, utc)
         ticket.save_changes('jane', 'Testing', t1)
         t2 = datetime(2001, 1, 1, 1, 1, 2, 0, utc)
-        db = self.env.get_db_cnx()
-        cursor = db.cursor()
-        cursor.execute("INSERT INTO attachment (type,id,filename,size,time,"
-                       "                        description,author,ipnr) "
-                       "VALUES ('ticket',%s,'file.txt',1234,%s,"
-                       "        'My file','mark','')",
-                       (str(tkt_id), to_utimestamp(t2)))
-        db.commit()
+        self.env.db_transaction("""
+            INSERT INTO attachment (type, id, filename, size, time,
+                                    description, author, ipnr)
+            VALUES ('ticket',%s,'file.txt',1234,%s, 'My file','mark','')
+            """, (str(tkt_id), to_utimestamp(t2)))
         t3 = datetime(2001, 1, 1, 1, 1, 3, 0, utc)
         ticket.save_changes('jim', 'Other', t3)
         log = ticket.get_changelog()
@@ -396,7 +413,7 @@ class TicketTestCase(unittest.TestCase):
 
 
 class TicketCommentTestCase(unittest.TestCase):
-    
+
     def _insert_ticket(self, summary, when, **kwargs):
         ticket = Ticket(self.env)
         for k, v in kwargs.iteritems():
@@ -408,21 +425,20 @@ class TicketCommentTestCase(unittest.TestCase):
         for k, v in kwargs.iteritems():
             ticket[k] = v
         ticket.save_changes(author, comment, when, cnum=cnum)
-    
+
     def _find_change(self, ticket, cnum):
-        (ts, author, comment) = ticket._find_change(cnum, self.db)
+        (ts, author, comment) = ticket._find_change(cnum)
         return from_utimestamp(ts)
-    
+
     def assertChange(self, ticket, cnum, date, author, **fields):
-        change = ticket.get_change(cnum)
+        change = ticket.get_change(cnum=cnum)
         self.assertEqual(dict(date=date, author=author, fields=fields), change)
-    
+
 
 class TicketCommentEditTestCase(TicketCommentTestCase):
 
     def setUp(self):
         self.env = EnvironmentStub(default_data=True)
-        self.db = self.env.get_db_cnx()
         self.created = datetime(2001, 1, 1, 1, 0, 0, 0, utc)
         self._insert_ticket('Test ticket', self.created,
                             owner='john', keywords='a, b, c')
@@ -449,7 +465,7 @@ class TicketCommentEditTestCase(TicketCommentTestCase):
         self.assertChange(ticket, 3, self.t3, 'jim',
             keywords=dict(author='jim', old='a, b, c', new='a, b'),
             comment=dict(author='jim', old='3', new='Comment 3'))
-        
+
         t = self.created + timedelta(seconds=10)
         ticket.modify_comment(self._find_change(ticket, 1),
                               'joe', 'New comment 1', t)
@@ -470,14 +486,11 @@ class TicketCommentEditTestCase(TicketCommentTestCase):
             comment=dict(author='john', old='1.2', new='New comment 2'),
             _comment0=dict(author='joe', old='Comment 2',
                            new=str(to_utimestamp(t))))
-        
+
     def test_modify_missing_cnum(self):
         """Editing a comment with no cnum in oldvalue"""
-        cursor = self.db.cursor()
-        cursor.execute("UPDATE ticket_change SET oldvalue='' "
-                       "WHERE oldvalue='3'")
-        self.db.commit()
-
+        self.env.db_transaction(
+            "UPDATE ticket_change SET oldvalue='' WHERE oldvalue='3'")
         ticket = Ticket(self.env, self.id)
         t = self.created + timedelta(seconds=30)
         ticket.modify_comment(self._find_change(ticket, 3),
@@ -487,14 +500,12 @@ class TicketCommentEditTestCase(TicketCommentTestCase):
             comment=dict(author='jim', old='', new='New comment 3'),
             _comment0=dict(author='joe', old='Comment 3',
                            new=str(to_utimestamp(t))))
-        
+
     def test_modify_missing_comment(self):
         """Editing a comment where the comment field is missing"""
-        cursor = self.db.cursor()
-        cursor.execute("DELETE FROM ticket_change "
-                       "WHERE field='comment' AND oldvalue='1.2'")
-        self.db.commit()
-
+        self.env.db_transaction("""
+            DELETE FROM ticket_change WHERE field='comment' AND oldvalue='1.2'
+            """)
         ticket = Ticket(self.env, self.id)
         t = self.created + timedelta(seconds=40)
         ticket.modify_comment(self._find_change(ticket, 2),
@@ -504,19 +515,16 @@ class TicketCommentEditTestCase(TicketCommentTestCase):
             comment=dict(author='john', old='', new='New comment 2'),
             _comment0=dict(author='joe', old='',
                            new=str(to_utimestamp(t))))
-        
+
     def test_modify_missing_cnums_and_comment(self):
         """Editing a comment when all cnums are missing and one comment
         field is missing
         """
-        cursor = self.db.cursor()
-        cursor.execute("UPDATE ticket_change SET oldvalue='' "
-                       "WHERE oldvalue='1'")
-        cursor.execute("DELETE FROM ticket_change "
-                       "WHERE field='comment' AND oldvalue='1.2'")
-        cursor.execute("UPDATE ticket_change SET oldvalue='' "
-                       "WHERE oldvalue='3'")
-        self.db.commit()
+        with self.env.db_transaction as db:
+            db("UPDATE ticket_change SET oldvalue='' WHERE oldvalue='1'")
+            db("""DELETE FROM ticket_change
+                  WHERE field='comment' AND oldvalue='1.2'""")
+            db("UPDATE ticket_change SET oldvalue='' WHERE oldvalue='3'")
 
         # Modify after missing comment
         ticket = Ticket(self.env, self.id)
@@ -555,16 +563,14 @@ class TicketCommentEditTestCase(TicketCommentTestCase):
                            new=str(to_utimestamp(t1))),
             _comment1=dict(author='joe', old='New comment 1',
                            new=str(to_utimestamp(t2))))
-        
-        cursor = self.db.cursor()
-        cursor.execute("DELETE FROM ticket_change "
-                       "WHERE field='_comment0'")
-        self.db.commit()
+
+        self.env.db_transaction(
+            "DELETE FROM ticket_change WHERE field='_comment0'")
 
         t3 = self.created + timedelta(seconds=90)
         ticket.modify_comment(self._find_change(ticket, 1),
                               'joe', 'Newest comment 1', t3)
-        
+
         self.assertChange(ticket, 1, self.t1, 'jack',
             comment=dict(author='jack', old='1', new='Newest comment 1'),
             _comment1=dict(author='joe', old='New comment 1',
@@ -581,7 +587,12 @@ class TicketCommentEditTestCase(TicketCommentTestCase):
             ticket.modify_comment(self._find_change(ticket, 1),
                                   'joe (%d)' % i,
                                   'Comment 1 (%d)' % i, t[-1])
-        history = ticket.get_comment_history(1)
+        history = ticket.get_comment_history(cnum=1)
+        self.assertEqual((0, t[0], 'jack', 'Comment 1'), history[0])
+        for i in range(1, len(history)):
+            self.assertEqual((i, t[i], 'joe (%d)' % i,
+                             'Comment 1 (%d)' % i), history[i])
+        history = ticket.get_comment_history(cdate=self.t1)
         self.assertEqual((0, t[0], 'jack', 'Comment 1'), history[0])
         for i in range(1, len(history)):
             self.assertEqual((i, t[i], 'joe (%d)' % i,
@@ -593,7 +604,6 @@ class TicketCommentDeleteTestCase(TicketCommentTestCase):
     def setUp(self):
         self.env = EnvironmentStub(default_data=True)
         self.env.config.set('ticket-custom', 'foo', 'text')
-        self.db = self.env.get_db_cnx()
         self.created = datetime(2001, 1, 1, 1, 0, 0, 0, utc)
         self._insert_ticket('Test ticket', self.created,
                             owner='john', keywords='a, b, c', foo='initial')
@@ -617,62 +627,90 @@ class TicketCommentDeleteTestCase(TicketCommentTestCase):
         ticket = Ticket(self.env, self.id)
         self.assertEqual('a', ticket['keywords'])
         self.assertEqual('change4', ticket['foo'])
-        ticket.delete_change(4)
+        t = datetime.now(utc)
+        ticket.delete_change(cnum=4, when=t)
         self.assertEqual('a, b', ticket['keywords'])
         self.assertEqual('change3', ticket['foo'])
-        self.assertEqual(None, ticket.get_change(4))
-        self.assertNotEqual(None, ticket.get_change(3))
-        self.assertEqual(self.t3, ticket.time_changed)
+        self.assertEqual(None, ticket.get_change(cnum=4))
+        self.assertNotEqual(None, ticket.get_change(cnum=3))
+        self.assertEqual(t, ticket.time_changed)
 
-    def test_delete_last_comment_when_custom_field_gone(self): 
-        """Regression test for http://trac.edgewall.org/ticket/10858""" 
-        ticket = Ticket(self.env, self.id) 
-        self.assertEqual('a', ticket['keywords']) 
-        self.assertEqual('change4', ticket['foo']) 
+    def test_delete_last_comment_when_custom_field_gone(self):
+        """Regression test for http://trac.edgewall.org/ticket/10858"""
+        ticket = Ticket(self.env, self.id)
+        self.assertEqual('a', ticket['keywords'])
+        self.assertEqual('change4', ticket['foo'])
         # we simulate the removal of the definition of the 'foo' custom field
-        self.env.config.remove('ticket-custom', 'foo') 
-        del TicketSystem(self.env).fields 
-        del TicketSystem(self.env).custom_fields 
-        ticket = Ticket(self.env, self.id) 
+        self.env.config.remove('ticket-custom', 'foo')
+        del TicketSystem(self.env).fields
+        del TicketSystem(self.env).custom_fields
+        ticket = Ticket(self.env, self.id)
         #
-        ticket.delete_change(cnum=4) 
+        t = datetime.now(utc)
+        ticket.delete_change(cnum=4, when=t)
         self.assertEqual('a, b', ticket['keywords'])
-        # 'foo' is no longer defined for the ticket 
-        self.assertEqual(None, ticket['foo']) 
-        # however, 'foo=change3' is still in the database 
-        cursor = self.db.cursor()
-        cursor.execute("""
+        # 'foo' is no longer defined for the ticket
+        self.assertEqual(None, ticket['foo'])
+        # however, 'foo=change3' is still in the database
+        self.assertEqual([('change3',)], self.env.db_query("""
             SELECT value FROM ticket_custom WHERE ticket=%s AND name='foo'
-            """, (self.id,))
-        self.assertEqual([('change3',)], cursor.fetchall())
-        self.assertEqual(None, ticket.get_change(cnum=4)) 
-        self.assertNotEqual(None, ticket.get_change(cnum=3)) 
-        self.assertEqual(self.t3, ticket.time_changed) 
-    
+            """, (self.id,)))
+        self.assertEqual(None, ticket.get_change(cnum=4))
+        self.assertNotEqual(None, ticket.get_change(cnum=3))
+        self.assertEqual(t, ticket.time_changed)
+
+    def test_delete_last_comment_by_date(self):
+        ticket = Ticket(self.env, self.id)
+        self.assertEqual('a', ticket['keywords'])
+        self.assertEqual('change4', ticket['foo'])
+        t = datetime.now(utc)
+        ticket.delete_change(cdate=self.t4, when=t)
+        self.assertEqual('a, b', ticket['keywords'])
+        self.assertEqual('change3', ticket['foo'])
+        self.assertEqual(None, ticket.get_change(cdate=self.t4))
+        self.assertNotEqual(None, ticket.get_change(cdate=self.t3))
+        self.assertEqual(t, ticket.time_changed)
+
     def test_delete_mid_comment(self):
         ticket = Ticket(self.env, self.id)
         self.assertChange(ticket, 4, self.t4, 'joe',
             comment=dict(author='joe', old='4', new='Comment 4'),
             keywords=dict(author='joe', old='a, b', new='a'),
             foo=dict(author='joe', old='change3', new='change4'))
-        ticket.delete_change(3)
-        self.assertEqual(None, ticket.get_change(3))
+        t = datetime.now(utc)
+        ticket.delete_change(cnum=3, when=t)
+        self.assertEqual(None, ticket.get_change(cnum=3))
         self.assertEqual('a', ticket['keywords'])
         self.assertChange(ticket, 4, self.t4, 'joe',
             comment=dict(author='joe', old='4', new='Comment 4'),
             keywords=dict(author='joe', old='a, b, c', new='a'),
             foo=dict(author='joe', old='change2', new='change4'))
-        self.assertEqual(self.t4, ticket.time_changed)
-        
+        self.assertEqual(t, ticket.time_changed)
+
+    def test_delete_mid_comment_by_date(self):
+        ticket = Ticket(self.env, self.id)
+        self.assertChange(ticket, 4, self.t4, 'joe',
+            comment=dict(author='joe', old='4', new='Comment 4'),
+            keywords=dict(author='joe', old='a, b', new='a'),
+            foo=dict(author='joe', old='change3', new='change4'))
+        t = datetime.now(utc)
+        ticket.delete_change(cdate=self.t3, when=t)
+        self.assertEqual(None, ticket.get_change(cdate=self.t3))
+        self.assertEqual('a', ticket['keywords'])
+        self.assertChange(ticket, 4, self.t4, 'joe',
+            comment=dict(author='joe', old='4', new='Comment 4'),
+            keywords=dict(author='joe', old='a, b, c', new='a'),
+            foo=dict(author='joe', old='change2', new='change4'))
+        self.assertEqual(t, ticket.time_changed)
+
     def test_delete_mid_comment_inconsistent(self):
         # Make oldvalue on keywords for change 4 inconsistent. This should
         # result in no change in oldvalue when deleting change 3. The
         # oldvalue of foo should change normally.
-        cursor = self.db.cursor()
-        cursor.execute("UPDATE ticket_change SET oldvalue='1, 2' "
-                       "WHERE field='keywords' AND oldvalue='a, b'")
-        self.db.commit()
-
+        self.env.db_transaction("""
+            UPDATE ticket_change SET oldvalue='1, 2'
+            WHERE field='keywords' AND oldvalue='a, b'
+            """)
         ticket = Ticket(self.env, self.id)
         self.assertChange(ticket, 4, self.t4, 'joe',
             comment=dict(author='joe', old='4', new='Comment 4'),
@@ -685,15 +723,15 @@ class TicketCommentDeleteTestCase(TicketCommentTestCase):
             comment=dict(author='joe', old='4', new='Comment 4'),
             keywords=dict(author='joe', old='1, 2', new='a'),
             foo=dict(author='joe', old='change2', new='change4'))
-        
+
     def test_delete_all_comments(self):
-        # See ticket:10338
         ticket = Ticket(self.env, self.id)
         ticket.delete_change(4)
         ticket.delete_change(3)
         ticket.delete_change(2)
-        ticket.delete_change(1)
-        self.assertEquals(ticket['time'], ticket['changetime'])
+        t = datetime.now(utc)
+        ticket.delete_change(1, when=t)
+        self.assertEqual(t, ticket.time_changed)
 
 
 class EnumTestCase(unittest.TestCase):
@@ -758,7 +796,7 @@ class TestMilestoneChangeListener(core.Component):
         self.action = 'changed'
         self.milestone = milestone
         self.old_values = old_values
-        
+
     def milestone_deleted(self, milestone):
         self.action = 'deleted'
         self.milestone = milestone
@@ -770,7 +808,6 @@ class MilestoneTestCase(unittest.TestCase):
         self.env = EnvironmentStub(default_data=True)
         self.env.path = os.path.join(tempfile.gettempdir(), 'trac-tempenv')
         os.mkdir(self.env.path)
-        self.db = self.env.get_db_cnx()
 
     def tearDown(self):
         shutil.rmtree(self.env.path)
@@ -803,9 +840,7 @@ class MilestoneTestCase(unittest.TestCase):
         self.assertEqual('', milestone.description)
 
     def test_existing_milestone(self):
-        cursor = self.db.cursor()
-        cursor.execute("INSERT INTO milestone (name) VALUES ('Test')")
-        cursor.close()
+        self.env.db_transaction("INSERT INTO milestone (name) VALUES ('Test')")
 
         milestone = Milestone(self.env, 'Test')
         self.assertEqual(True, milestone.exists)
@@ -819,39 +854,34 @@ class MilestoneTestCase(unittest.TestCase):
         milestone.name = 'Test'
         milestone.insert()
 
-        cursor = self.db.cursor()
-        cursor.execute("SELECT name,due,completed,description FROM milestone "
-                       "WHERE name='Test'")
-        self.assertEqual(('Test', 0, 0, ''), cursor.fetchone())
-        
+        self.assertEqual([('Test', 0, 0, '')], self.env.db_query("""
+            SELECT name, due, completed, description FROM milestone
+            WHERE name='Test'
+            """))
+
         # Use the same model object to update the milestone
         milestone.description = 'Some text'
         milestone.update()
-        cursor.execute("SELECT name,due,completed,description FROM milestone "
-                       "WHERE name='Test'")
-        self.assertEqual(('Test', 0, 0, 'Some text'), cursor.fetchone())
+        self.assertEqual([('Test', 0, 0, 'Some text')], self.env.db_query("""
+            SELECT name, due, completed, description FROM milestone
+            WHERE name='Test'
+            """))
 
     def test_create_milestone_without_name(self):
         milestone = Milestone(self.env)
         self.assertRaises(TracError, milestone.insert)
 
     def test_delete_milestone(self):
-        cursor = self.db.cursor()
-        cursor.execute("INSERT INTO milestone (name) VALUES ('Test')")
-        cursor.close()
+        self.env.db_transaction("INSERT INTO milestone (name) VALUES ('Test')")
 
         milestone = Milestone(self.env, 'Test')
         milestone.delete()
         self.assertEqual(False, milestone.exists)
-
-        cursor = self.db.cursor()
-        cursor.execute("SELECT * FROM milestone WHERE name='Test'")
-        self.assertEqual(None, cursor.fetchone())
+        self.assertEqual([],
+            self.env.db_query("SELECT * FROM milestone WHERE name='Test'"))
 
     def test_delete_milestone_retarget_tickets(self):
-        cursor = self.db.cursor()
-        cursor.execute("INSERT INTO milestone (name) VALUES ('Test')")
-        cursor.close()
+        self.env.db_transaction("INSERT INTO milestone (name) VALUES ('Test')")
 
         tkt1 = Ticket(self.env)
         tkt1.populate({'summary': 'Foo', 'milestone': 'Test'})
@@ -868,9 +898,7 @@ class MilestoneTestCase(unittest.TestCase):
         self.assertEqual('Other', Ticket(self.env, tkt2.id)['milestone'])
 
     def test_update_milestone(self):
-        cursor = self.db.cursor()
-        cursor.execute("INSERT INTO milestone (name) VALUES ('Test')")
-        cursor.close()
+        self.env.db_transaction("INSERT INTO milestone (name) VALUES ('Test')")
 
         milestone = Milestone(self.env, 'Test')
         t1 = datetime(2001, 01, 01, tzinfo=utc)
@@ -880,25 +908,19 @@ class MilestoneTestCase(unittest.TestCase):
         milestone.description = 'Foo bar'
         milestone.update()
 
-        cursor = self.db.cursor()
-        cursor.execute("SELECT * FROM milestone WHERE name='Test'")
-        self.assertEqual(('Test', to_utimestamp(t1), to_utimestamp(t2),
-                          'Foo bar'),
-                         cursor.fetchone())
+        self.assertEqual(
+            [('Test', to_utimestamp(t1), to_utimestamp(t2), 'Foo bar')],
+            self.env.db_query("SELECT * FROM milestone WHERE name='Test'"))
 
     def test_update_milestone_without_name(self):
-        cursor = self.db.cursor()
-        cursor.execute("INSERT INTO milestone (name) VALUES ('Test')")
-        cursor.close()
+        self.env.db_transaction("INSERT INTO milestone (name) VALUES ('Test')")
 
         milestone = Milestone(self.env, 'Test')
         milestone.name = None
         self.assertRaises(TracError, milestone.update)
 
     def test_update_milestone_update_tickets(self):
-        cursor = self.db.cursor()
-        cursor.execute("INSERT INTO milestone (name) VALUES ('Test')")
-        cursor.close()
+        self.env.db_transaction("INSERT INTO milestone (name) VALUES ('Test')")
 
         tkt1 = Ticket(self.env)
         tkt1.populate({'summary': 'Foo', 'milestone': 'Test'})
@@ -918,14 +940,14 @@ class MilestoneTestCase(unittest.TestCase):
         milestone = Milestone(self.env)
         milestone.name = 'OldName'
         milestone.insert()
-        
+
         attachment = Attachment(self.env, 'milestone', 'OldName')
         attachment.insert('foo.txt', StringIO(), 0, 1)
-        
+
         milestone = Milestone(self.env, 'OldName')
         milestone.name = 'NewName'
         milestone.update()
-        
+
         self.assertRaises(ResourceNotFound, Milestone, self.env, 'OldName')
         self.assertEqual('NewName', Milestone(self.env, 'NewName').name)
 
@@ -934,12 +956,11 @@ class MilestoneTestCase(unittest.TestCase):
         attachments = Attachment.select(self.env, 'milestone', 'NewName')
         self.assertEqual('foo.txt', attachments.next().filename)
         self.assertRaises(StopIteration, attachments.next)
-        
+
     def test_select_milestones(self):
-        cursor = self.db.cursor()
-        cursor.executemany("INSERT INTO milestone (name) VALUES (%s)",
-                           [('1.0',), ('2.0',)])
-        cursor.close()
+        self.env.db_transaction.executemany(
+            "INSERT INTO milestone (name) VALUES (%s)",
+            [('1.0',), ('2.0',)])
 
         milestones = list(Milestone.select(self.env))
         self.assertEqual('1.0', milestones[0].name)
@@ -961,12 +982,12 @@ class MilestoneTestCase(unittest.TestCase):
             due=datetime(2001, 01, 01, tzinfo=utc),
             description='The milestone description')
         milestone.insert()
-        
+
         milestone.name = 'Milestone 2'
         milestone.completed = datetime(2001, 02, 03, tzinfo=utc)
         milestone.description = 'The changed description'
         milestone.update()
-        
+
         self.assertEqual('changed', listener.action)
         self.assertEqual(milestone, listener.milestone)
         self.assertEqual({'name': 'Milestone 1', 'completed': None,
@@ -989,7 +1010,6 @@ class ComponentTestCase(unittest.TestCase):
 
     def setUp(self):
         self.env = EnvironmentStub(default_data=True)
-        self.db = self.env.get_db_cnx()
 
     def tearDown(self):
         self.env.reset_db()
@@ -1010,25 +1030,21 @@ class ComponentTestCase(unittest.TestCase):
         component = Component(self.env)
         component.name = 'Test'
         component.insert()
-        
-        cursor = self.db.cursor()
-        cursor.execute("SELECT name,owner,description FROM component "
-                       "WHERE name='Test'")
-        self.assertEqual(('Test', None, None), cursor.fetchone())
-        
+
+        self.assertEqual([('Test', None, None)], self.env.db_query("""
+            SELECT name, owner, description FROM component
+            WHERE name='Test'"""))
+
         # Use the same model object to update the component
         component.owner = 'joe'
         component.update()
-        cursor.execute("SELECT name,owner,description FROM component "
-                       "WHERE name='Test'")
-        self.assertEqual(('Test', 'joe', None), cursor.fetchone())
-
+        self.assertEqual([('Test', 'joe', None)], self.env.db_query(
+            "SELECT name, owner, description FROM component WHERE name='Test'"))
 
 class VersionTestCase(unittest.TestCase):
 
     def setUp(self):
         self.env = EnvironmentStub(default_data=True)
-        self.db = self.env.get_db_cnx()
 
     def tearDown(self):
         self.env.reset_db()
@@ -1049,18 +1065,15 @@ class VersionTestCase(unittest.TestCase):
         version = Version(self.env)
         version.name = 'Test'
         version.insert()
-        
-        cursor = self.db.cursor()
-        cursor.execute("SELECT name,time,description FROM version "
-                       "WHERE name='Test'")
-        self.assertEqual(('Test', 0, None), cursor.fetchone())
-        
+
+        self.assertEqual([('Test', 0, None)], self.env.db_query(
+            "SELECT name, time, description FROM version WHERE name='Test'"))
+
         # Use the same model object to update the version
         version.description = 'Some text'
         version.update()
-        cursor.execute("SELECT name,time,description FROM version "
-                       "WHERE name='Test'")
-        self.assertEqual(('Test', 0, 'Some text'), cursor.fetchone())
+        self.assertEqual([('Test', 0, 'Some text')], self.env.db_query(
+            "SELECT name, time, description FROM version WHERE name='Test'"))
 
 
 def suite():
