@@ -1,7 +1,7 @@
-from trac.mimeview import Context
-from trac.test import Mock, EnvironmentStub, MockPerm
+from trac.test import Mock, EnvironmentStub, MockPerm, locale_en
 from trac.ticket.query import Query, QueryModule, TicketQueryMacro
 from trac.util.datefmt import utc
+from trac.web.chrome import web_context
 from trac.web.href import Href
 from trac.wiki.formatter import LinkFormatter
 
@@ -9,7 +9,7 @@ import unittest
 import difflib
 
 # Note: we don't want to replicate 1:1 all the SQL dialect abstraction
-#       methods from the trac.db layer here. 
+#       methods from the trac.db layer here.
 
 class QueryTestCase(unittest.TestCase):
 
@@ -32,9 +32,9 @@ class QueryTestCase(unittest.TestCase):
 
     def setUp(self):
         self.env = EnvironmentStub(default_data=True)
-        self.db = self.env.get_db_cnx()
-        self.req = Mock(href=self.env.href, authname='anonymous', tz=utc)
-        
+        self.req = Mock(href=self.env.href, authname='anonymous', tz=utc,
+                        locale=locale_en, lc_time=locale_en)
+
     def tearDown(self):
         self.env.reset_db()
 
@@ -90,7 +90,7 @@ ORDER BY COALESCE(t.id,0)=0,t.id""")
 FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
 ORDER BY COALESCE(priority.value,'')='',%(cast_priority)s,t.id""" % {
-          'cast_priority': self.env.get_db_cnx().cast('priority.value', 'int')})
+          'cast_priority': self.env.get_read_db().cast('priority.value', 'int')})
         self.assertEqual([], args)
         tickets = query.execute(self.req)
 
@@ -102,7 +102,7 @@ ORDER BY COALESCE(priority.value,'')='',%(cast_priority)s,t.id""" % {
 FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
 ORDER BY COALESCE(priority.value,'')='' DESC,%(cast_priority)s DESC,t.id""" % {
-          'cast_priority': self.env.get_db_cnx().cast('priority.value', 'int')})
+          'cast_priority': self.env.get_read_db().cast('priority.value', 'int')})
         self.assertEqual([], args)
         tickets = query.execute(self.req)
 
@@ -174,7 +174,7 @@ ORDER BY COALESCE(t.milestone,'')='' DESC,COALESCE(milestone.completed,0)=0 DESC
 FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
 ORDER BY COALESCE(priority.value,'')='',%(cast_priority)s,t.id""" % {
-          'cast_priority': self.env.get_db_cnx().cast('priority.value', 'int')})
+          'cast_priority': self.env.get_read_db().cast('priority.value', 'int')})
         self.assertEqual([], args)
         tickets = query.execute(self.req)
 
@@ -211,7 +211,7 @@ ORDER BY COALESCE(t.id,0)=0,t.id""")
 FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
 WHERE ((COALESCE(t.owner,'') %(like)s))
-ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_db_cnx().like()})
+ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_read_db().like()})
         self.assertEqual(['%someone%'], args)
         tickets = query.execute(self.req)
 
@@ -223,7 +223,7 @@ ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_db_cnx().like()})
 FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
 WHERE ((COALESCE(t.owner,'') NOT %(like)s))
-ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_db_cnx().like()})
+ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_read_db().like()})
         self.assertEqual(['%someone%'], args)
         tickets = query.execute(self.req)
 
@@ -235,7 +235,7 @@ ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_db_cnx().like()})
 FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
 WHERE ((COALESCE(t.owner,'') %(like)s))
-ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_db_cnx().like()})
+ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_read_db().like()})
         self.assertEqual(['someone%'], args)
         tickets = query.execute(self.req)
 
@@ -247,7 +247,7 @@ ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_db_cnx().like()})
 FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
 WHERE ((COALESCE(t.owner,'') %(like)s))
-ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_db_cnx().like()})
+ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_read_db().like()})
         self.assertEqual(['%someone'], args)
         tickets = query.execute(self.req)
 
@@ -255,7 +255,7 @@ ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_db_cnx().like()})
         self.env.config.set('ticket-custom', 'foo', 'text')
         query = Query.from_string(self.env, 'foo=something', order='id')
         sql, args = query.get_sql()
-        foo = self.db.quote('foo')
+        foo = self.env.get_read_db().quote('foo')
         self.assertEqualSQL(sql,
 """SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.milestone AS milestone,t.time AS time,t.changetime AS changetime,priority.value AS priority_value,%s.value AS %s
 FROM ticket AS t
@@ -270,7 +270,7 @@ ORDER BY COALESCE(t.id,0)=0,t.id""" % ((foo,) * 6))
         self.env.config.set('ticket-custom', 'foo', 'text')
         query = Query(self.env, group='foo', order='id')
         sql, args = query.get_sql()
-        foo = self.db.quote('foo')
+        foo = self.env.get_read_db().quote('foo')
         self.assertEqualSQL(sql,
 """SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.milestone AS milestone,t.time AS time,t.changetime AS changetime,priority.value AS priority_value,%s.value AS %s
 FROM ticket AS t
@@ -317,7 +317,7 @@ ORDER BY COALESCE(t.id,0)=0,t.id""")
 FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
 WHERE ((COALESCE(t.owner,'') %(like)s OR COALESCE(t.owner,'') %(like)s))
-ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_db_cnx().like()})
+ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_read_db().like()})
         tickets = query.execute(self.req)
 
     def test_constrained_by_empty_value_contains(self):
@@ -362,7 +362,7 @@ FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
 WHERE (((%(cast_time)s>=%%s AND %(cast_time)s<%%s)))
 ORDER BY COALESCE(t.id,0)=0,t.id""" % {
-          'cast_time': self.env.get_db_cnx().cast('t.time', 'int64')})
+          'cast_time': self.env.get_read_db().cast('t.time', 'int64')})
         self.assertEqual([1217548800000000L, 1220227200000000L], args)
         tickets = query.execute(self.req)
 
@@ -375,7 +375,7 @@ FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
 WHERE ((NOT (%(cast_time)s>=%%s AND %(cast_time)s<%%s)))
 ORDER BY COALESCE(t.id,0)=0,t.id""" % {
-          'cast_time': self.env.get_db_cnx().cast('t.time', 'int64')})
+          'cast_time': self.env.get_read_db().cast('t.time', 'int64')})
         self.assertEqual([1217548800000000L, 1220227200000000L], args)
         tickets = query.execute(self.req)
 
@@ -388,7 +388,7 @@ FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
 WHERE ((%(cast_time)s>=%%s))
 ORDER BY COALESCE(t.id,0)=0,t.id""" % {
-          'cast_time': self.env.get_db_cnx().cast('t.time', 'int64')})
+          'cast_time': self.env.get_read_db().cast('t.time', 'int64')})
         self.assertEqual([1217548800000000L], args)
         tickets = query.execute(self.req)
 
@@ -401,7 +401,7 @@ FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
 WHERE ((%(cast_time)s<%%s))
 ORDER BY COALESCE(t.id,0)=0,t.id""" % {
-          'cast_time': self.env.get_db_cnx().cast('t.time', 'int64')})
+          'cast_time': self.env.get_read_db().cast('t.time', 'int64')})
         self.assertEqual([1220227200000000L], args)
         tickets = query.execute(self.req)
 
@@ -414,7 +414,7 @@ FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
 WHERE (((%(cast_changetime)s>=%%s AND %(cast_changetime)s<%%s)))
 ORDER BY COALESCE(t.id,0)=0,t.id""" % {
-          'cast_changetime': self.env.get_db_cnx().cast('t.changetime', 'int64')})
+          'cast_changetime': self.env.get_read_db().cast('t.changetime', 'int64')})
         self.assertEqual([1217548800000000L, 1220227200000000L], args)
         tickets = query.execute(self.req)
 
@@ -427,7 +427,7 @@ ORDER BY COALESCE(t.id,0)=0,t.id""" % {
 FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
 WHERE (((COALESCE(t.keywords,'') %(like)s AND COALESCE(t.keywords,'') NOT %(like)s AND COALESCE(t.keywords,'') %(like)s)))
-ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_db_cnx().like()})
+ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_read_db().like()})
         self.assertEqual(['%foo%', '%bar%', '%baz%'], args)
         tickets = query.execute(self.req)
 
@@ -480,17 +480,44 @@ ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual(args, like_args)
         tickets = query.execute(self.req)
 
+    def test_user_var(self):
+        query = Query.from_string(self.env, 'owner=$USER&order=id')
+        sql, args = query.get_sql(req=self.req)
+        self.assertEqualSQL(sql,
+"""SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.milestone AS milestone,t.time AS time,t.changetime AS changetime,priority.value AS priority_value
+FROM ticket AS t
+  LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
+WHERE ((COALESCE(t.owner,'')=%s))
+ORDER BY COALESCE(t.id,0)=0,t.id""")
+        self.assertEqual(['anonymous'], args)
+        tickets = query.execute(self.req)
+
     def test_csv_escape(self):
         query = Mock(get_columns=lambda: ['col1'],
-                     execute=lambda r,c: [{'id': 1,
-                                           'col1': 'value, needs escaped'}],
+                     execute=lambda r: [{'id': 1,
+                                         'col1': 'value, needs escaped'}],
                      time_fields=['time', 'changetime'])
         content, mimetype = QueryModule(self.env).export_csv(
                                 Mock(href=self.env.href, perm=MockPerm()),
                                 query)
-        self.assertEqual('col1\r\n"value, needs escaped"\r\n',
+        self.assertEqual('\xef\xbb\xbfcol1\r\n"value, needs escaped"\r\n',
                          content)
 
+    def test_template_data(self):
+        req = Mock(href=self.env.href, perm=MockPerm(), authname='anonymous',
+                   tz=None, locale=None)
+        context = web_context(req, 'query')
+
+        query = Query.from_string(self.env, 'owner=$USER&order=id')
+        tickets = query.execute(req)
+        data = query.template_data(context, tickets, req=req)
+        self.assertEqual(['anonymous'], data['clauses'][0]['owner']['values'])
+
+        query = Query.from_string(self.env, 'owner=$USER&order=id')
+        tickets = query.execute(req)
+        data = query.template_data(context, tickets)
+        self.assertEqual(['$USER'], data['clauses'][0]['owner']['values'])
+
 
 class QueryLinksTestCase(unittest.TestCase):
 
@@ -498,7 +525,7 @@ class QueryLinksTestCase(unittest.TestCase):
         self.env = EnvironmentStub(default_data=True)
         self.query_module = QueryModule(self.env)
         req = Mock(perm=MockPerm(), args={}, href=Href('/'))
-        self.formatter = LinkFormatter(self.env, Context.from_request(req))
+        self.formatter = LinkFormatter(self.env, web_context(req))
 
     def tearDown(self):
         self.env.reset_db()
@@ -520,7 +547,7 @@ class TicketQueryMacroTestCase(unittest.TestCase):
         self.assertEqual(query, qs)
         self.assertEqual(kwargs, kw)
         self.assertEqual(format, f)
-    
+
     def test_owner_and_milestone(self):
         self.assertQueryIs('owner=joe, milestone=milestone1',
                            'owner=joe&milestone=milestone1',
@@ -532,7 +559,7 @@ class TicketQueryMacroTestCase(unittest.TestCase):
                            'owner=joe&or&milestone=milestone1',
                            dict(col='status|summary', max='0', order='id'),
                            'list')
-    
+
     def test_format_arguments(self):
         self.assertQueryIs('owner=joe, milestone=milestone1, col=component|severity, max=15, order=component, format=compact',
                            'owner=joe&milestone=milestone1',
@@ -548,13 +575,13 @@ class TicketQueryMacroTestCase(unittest.TestCase):
                            r'owner=joe|jack&milestone=this\&that\|here,now',
                            dict(col='status|summary', max='0', order='id'),
                            'list')
-        
+
 
 def suite():
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(QueryTestCase, 'test'))
     suite.addTest(unittest.makeSuite(QueryLinksTestCase, 'test'))
-    suite.addTest(unittest.makeSuite(TicketQueryMacroTestCase, 'text'))
+    suite.addTest(unittest.makeSuite(TicketQueryMacroTestCase, 'test'))
     return suite
 
 if __name__ == '__main__':
