@@ -11,6 +11,8 @@
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at http://trac.edgewall.org/log/.
 
+from __future__ import with_statement
+
 import atexit
 import errno
 import os
@@ -20,20 +22,17 @@ import sys
 def daemonize(pidfile=None, progname=None, stdin='/dev/null',
               stdout='/dev/null', stderr='/dev/null', umask=022):
     """Fork a daemon process."""
-
     if pidfile:
         # Check whether the pid file already exists and refers to a still
         # process running
         pidfile = os.path.abspath(pidfile)
         if os.path.exists(pidfile):
-            fileobj = open(pidfile)
-            try:
+            with open(pidfile) as fileobj:
                 try:
                     pid = int(fileobj.read())
                 except ValueError:
-                    sys.exit('Invalid PID in file %s' % pidfile)
-            finally:
-                fileobj.close()
+                    sys.exit('Invalid pid in file %s\nPlease remove it to '
+                             'proceed' % pidfile)
 
             try: # signal the process to see if it is still running
                 os.kill(pid, 0)
@@ -46,7 +45,7 @@ def daemonize(pidfile=None, progname=None, stdin='/dev/null',
 
         # The pid file must be writable
         try:
-            fileobj = open(pidfile, 'r+')
+            fileobj = open(pidfile, 'a+')
             fileobj.close()
         except IOError, e:
             from trac.util.text import exception_to_unicode
@@ -81,18 +80,15 @@ def daemonize(pidfile=None, progname=None, stdin='/dev/null',
         # Register signal handlers to ensure atexit hooks are called on exit
         for signum in [signal.SIGTERM, signal.SIGHUP]:
             signal.signal(signum, handle_signal)
-        
+
         # Create/update the pid file, and register a hook to remove it when the
         # process exits
         def remove_pidfile():
             if os.path.exists(pidfile):
                 os.remove(pidfile)
         atexit.register(remove_pidfile)
-        fileobj = open(pidfile, 'w')
-        try:
+        with open(pidfile, 'w') as fileobj:
             fileobj.write(str(os.getpid()))
-        finally:
-            fileobj.close()
 
 
 def handle_signal(signum, frame):
