@@ -9,7 +9,7 @@ from trac.util.text import empty, expandtabs, fix_eol, javascript_quote, \
                            unicode_quote_plus, unicode_unquote, \
                            unicode_urlencode, wrap, quote_query_string, \
                            unicode_to_base64, unicode_from_base64, \
-                           levenshtein_distance
+                           strip_line_ws, stripws, levenshtein_distance
 
 
 class ToUnicodeTestCase(unittest.TestCase):
@@ -158,15 +158,15 @@ class PrintTableTestCase(unittest.TestCase):
         headers = ('Version', 'Date', 'Name')
         expected = """\
 
-Version     Date         Name     
+Version     Date         Name
 ----------------------------------
-Trac 0.12 | 2010-06-13 | Babel    
-Trac 0.11 | 2008-06-22 | Genshi   
-Trac 0.10 | 2006-09-28 | Zengia   
-Trac 0.9  | 2005-10-31 | Vodun    
-Trac 0.8  | 2004-11-15 | Qualia   
-Trac 0.7  | 2004-05-18 | Fulci    
-Trac 0.6  | 2004-03-23 | Solanum  
+Trac 0.12 | 2010-06-13 | Babel
+Trac 0.11 | 2008-06-22 | Genshi
+Trac 0.10 | 2006-09-28 | Zengia
+Trac 0.9  | 2005-10-31 | Vodun
+Trac 0.8  | 2004-11-15 | Qualia
+Trac 0.7  | 2004-05-18 | Fulci
+Trac 0.6  | 2004-03-23 | Solanum
 Trac 0.5  | 2004-02-23 | Incognito
 
 """
@@ -183,11 +183,11 @@ Trac 0.5  | 2004-02-23 | Incognito
         )
         expected = u"""\
 
-NoneType | None  |      
-bool     | True  | True 
+NoneType | None  |
+bool     | True  | True
 bool     | False | False
-int      | 0     | 0    
-float    | 0.0   | 0.0  
+int      | 0     | 0
+float    | 0.0   | 0.0
 
 """
         self._validate_print_table(expected, data, sep=' | ', ambiwidth=1)
@@ -200,9 +200,9 @@ float    | 0.0   | 0.0
         headers = ('Obfuscated', 'Email')
         expected = u"""\
 
-Obfuscated      Email          
+Obfuscated      Email
 -------------------------------
-foo@localhost | foo@localhost  
+foo@localhost | foo@localhost
 bar@….com     | bar@example.com
 
 """
@@ -217,9 +217,9 @@ bar@….com     | bar@example.com
         headers = ('Obfuscated', 'Email')
         expected = u"""\
 
-Obfuscated      Email          
+Obfuscated      Email
 -------------------------------
-foo@localhost | foo@localhost  
+foo@localhost | foo@localhost
 bar@….com    | bar@example.com
 
 """
@@ -230,7 +230,8 @@ bar@….com    | bar@example.com
         out = StringIO()
         kwargs['out'] = out
         print_table(data, **kwargs)
-        self.assertEqual(expected.encode('utf-8'), out.getvalue())
+        self.assertEqual(expected.encode('utf-8'),
+                         strip_line_ws(out.getvalue(), leading=False))
 
 
 class WrapTestCase(unittest.TestCase):
@@ -307,6 +308,21 @@ class UnicodeBase64TestCase(unittest.TestCase):
         self.assertEqual(text, unicode_from_base64(text_base64_no_strip))
 
 
+class StripwsTestCase(unittest.TestCase):
+    def test_stripws(self):
+        self.assertEquals(u'stripws',
+                          stripws(u' \u200b\t\u3000stripws \u200b\t\u2008'))
+        self.assertEquals(u'stripws \u3000\t',
+                          stripws(u'\u200b\t\u2008 stripws \u3000\t',
+                                  trailing=False))
+        self.assertEquals(u' \t\u3000stripws',
+                          stripws(u' \t\u3000stripws \u200b\t\u2008',
+                                  leading=False))
+        self.assertEquals(u' \t\u3000stripws \u200b\t\u2008',
+                          stripws(u' \t\u3000stripws \u200b\t\u2008',
+                                  leading=False, trailing=False))
+
+
 
 class LevenshteinDistanceTestCase(unittest.TestCase):
     def test_distance(self):
@@ -331,6 +347,7 @@ def suite():
     suite.addTest(unittest.makeSuite(WrapTestCase, 'test'))
     suite.addTest(unittest.makeSuite(FixEolTestCase, 'test'))
     suite.addTest(unittest.makeSuite(UnicodeBase64TestCase, 'test'))
+    suite.addTest(unittest.makeSuite(StripwsTestCase, 'test'))
     suite.addTest(unittest.makeSuite(LevenshteinDistanceTestCase, 'test'))
     return suite
 
