@@ -19,6 +19,7 @@ from Cookie import CookieError, BaseCookie, SimpleCookie
 import cgi
 from datetime import datetime
 import errno
+from hashlib import md5
 import new
 import mimetypes
 import os
@@ -28,13 +29,99 @@ import sys
 import urlparse
 
 from trac.core import Interface, TracError
-from trac.util import get_last_traceback, md5, unquote
+from trac.util import get_last_traceback, unquote
 from trac.util.datefmt import http_date, localtz
 from trac.util.text import empty, to_unicode
 from trac.util.translation import _
 from trac.web.href import Href
 from trac.web.wsgi import _FileWrapper
 
+class IAuthenticator(Interface):
+    """Extension point interface for components that can provide the name
+    of the remote user."""
+
+    def authenticate(req):
+        """Return the name of the remote user, or `None` if the identity of the
+        user is unknown."""
+
+
+class IRequestHandler(Interface):
+    """Decide which `trac.core.Component` handles which `Request`, and how."""
+
+    def match_request(req):
+        """Return whether the handler wants to process the given request."""
+
+    def process_request(req):
+        """Process the request.
+
+        Return a `(template_name, data, content_type)` tuple,
+        where `data` is a dictionary of substitutions for the Genshi template.
+
+        "text/html" is assumed if `content_type` is `None`.
+
+        Note that if template processing should not occur, this method can
+        simply send the response itself and not return anything.
+
+        :Since 1.0: Clearsilver templates are no longer supported.
+        """
+
+
+class IRequestFilter(Interface):
+    """Enable components to interfere with the processing done by the
+    main handler, either before and/or after it enters in action.
+    """
+
+    def pre_process_request(req, handler):
+        """Called after initial handler selection, and can be used to change
+        the selected handler or redirect request.
+
+        Always returns the request handler, even if unchanged.
+        """
+
+    def post_process_request(req, template, data, content_type):
+        """Do any post-processing the request might need; typically adding
+        values to the template `data` dictionary, or changing the Genshi
+        template or mime type.
+
+        `data` may be updated in place.
+
+        Always returns a tuple of (template, data, content_type), even if
+        unchanged.
+
+        Note that `template`, `data`, `content_type` will be `None` if:
+         - called when processing an error page
+         - the default request handler did not return any result
+
+        :Since 0.11: there's a `data` argument for supporting Genshi templates;
+           this introduced a difference in arity which made it possible to
+           distinguish between the IRequestFilter components still targeted
+           at ClearSilver templates and the newer ones targeted at Genshi
+           templates.
+
+        :Since 1.0: Clearsilver templates are no longer supported.
+        """
+
+
+class ITemplateStreamFilter(Interface):
+    """Transform the generated content by filtering the Genshi event stream
+    generated by the template, prior to its serialization.
+    """
+
+    def filter_stream(req, method, filename, stream, data):
+        """Return a filtered Genshi event stream, or the original unfiltered
+        stream if no match.
+
+        `req` is the current request object, `method` is the Genshi render
+        method (xml, xhtml or text), `filename` is the filename of the template
+        to be rendered, `stream` is the event stream and `data` is the data for
+        the current template.
+
+        See the Genshi_ documentation for more information.
+
+        .. _Genshi: http://genshi.edgewall.org/wiki/Documentation/filters.html
+        """
+
+
 HTTP_STATUS = dict([(code, reason.title()) for code, (reason, description)
                     in BaseHTTPRequestHandler.responses.items()])
 
@@ -63,7 +150,7 @@ class HTTPException(Exception):
         new_class.reason = reason
         return new_class
 
-
+_HTTPException_subclass_names = []
 for code in [code for code in HTTP_STATUS if code >= 400]:
     exc_name = HTTP_STATUS[code].replace(' ', '').replace('-', '')
     # 2.5 compatibility hack:
@@ -71,9 +158,10 @@ for code in [code for code in HTTP_STATUS if code >= 400]:
         exc_name = 'InternalError'
     if exc_name.lower().startswith('http'):
         exc_name = exc_name[4:]
-    exc_name = 'HTTP' + exc_name        
+    exc_name = 'HTTP' + exc_name
     setattr(sys.modules[__name__], exc_name,
             HTTPException.subclass(exc_name, code))
+    _HTTPException_subclass_names.append(exc_name)
 del code, exc_name
 
 
@@ -184,16 +272,16 @@ class Cookie(SimpleCookie):
 
 class Request(object):
     """Represents a HTTP request/response pair.
-    
+
     This class provides a convenience API over WSGI.
     """
 
     def __init__(self, environ, start_response):
         """Create the request wrapper.
-        
-        @param environ: The WSGI environment dict
-        @param start_response: The WSGI callback for starting the response
-        @param callbacks: A dictionary of functions that are used to lazily
+
+        :param environ: The WSGI environment dict
+        :param start_response: The WSGI callback for starting the response
+        :param callbacks: A dictionary of functions that are used to lazily
             evaluate attribute lookups
         """
         self.environ = environ
@@ -265,14 +353,14 @@ class Request(object):
     @property
     def remote_user(self):
         """ Name of the remote user.
-        
+
         Will be `None` if the user has not logged in using HTTP authentication.
         """
         return self.environ.get('REMOTE_USER')
 
     @property
     def scheme(self):
-        """The scheme of the request URL"""        
+        """The scheme of the request URL"""
         return self.environ['wsgi.url_scheme']
 
     @property
@@ -292,7 +380,7 @@ class Request(object):
 
     def add_redirect_listener(self, listener):
         """Add a callable to be called prior to executing a redirect.
-        
+
         The callable is passed the arguments to the `redirect()` call.
         """
         self.redirect_listeners.append(listener)
@@ -364,14 +452,12 @@ class Request(object):
 
     def redirect(self, url, permanent=False):
         """Send a redirect to the client, forwarding to the specified URL.
-        
+
         The `url` may be relative or absolute, relative URLs will be translated
         appropriately.
         """
         for listener in self.redirect_listeners:
             listener(self, url, permanent)
-        
-        self.session.save() # has to be done before the redirect is sent
 
         if permanent:
             status = 301 # 'Moved Permanently'
@@ -381,7 +467,7 @@ class Request(object):
             status = 302 # 'Found' -- normal temporary redirect
 
         self.send_response(status)
-        if not url.startswith('http://') and not url.startswith('https://'):
+        if not url.startswith(('http://', 'https://')):
             # Make sure the URL is absolute
             scheme, host = urlparse.urlparse(self.base_url)[:2]
             url = urlparse.urlunparse((scheme, host, url, None, None, None))
@@ -400,28 +486,6 @@ class Request(object):
         self.end_headers()
         raise RequestDone
 
-    def display(self, template, content_type='text/html', status=200):
-        """Render the response using the ClearSilver template given by the
-        `template` parameter, which can be either the name of the template
-        file, or an already parsed `neo_cs.CS` object.
-        """
-        assert self.hdf, \
-               'HDF dataset not available. Check your clearsilver installation'
-        if self.args.has_key('hdfdump'):
-            # FIXME: the administrator should probably be able to disable HDF
-            #        dumps
-            self.perm.require('TRAC_ADMIN')
-            content_type = 'text/plain'
-            data = str(self.hdf)
-        else:
-            try:
-                form_token = self.form_token
-            except AttributeError:
-                form_token = None
-            data = self.hdf.render(template, form_token)
-
-        self.send(data, content_type, status)
-
     def send(self, content, content_type='text/html', status=200):
         self.send_response(status)
         self.send_header('Cache-Control', 'must-revalidate')
@@ -437,14 +501,6 @@ class Request(object):
     def send_error(self, exc_info, template='error.html',
                    content_type='text/html', status=500, env=None, data={}):
         try:
-            if template.endswith('.cs') and self.hdf: # FIXME: remove this
-                if self.args.has_key('hdfdump'):
-                    self.perm.require('TRAC_ADMIN')
-                    content_type = 'text/plain'
-                    data = str(self.hdf)
-                else:
-                    data = self.hdf.render(template)
-
             if template.endswith('.html'):
                 if env:
                     from trac.web.chrome import Chrome
@@ -461,7 +517,7 @@ class Request(object):
                     data = '%s\n\n%s: %s' % (data.get('title'),
                                              data.get('type'),
                                              data.get('message'))
-        except: # failed to render
+        except Exception: # failed to render
             data = get_last_traceback()
             content_type = 'text/plain'
 
@@ -483,9 +539,16 @@ class Request(object):
             self.write(data)
         raise RequestDone
 
+    def send_no_content(self):
+        self.send_response(204)
+        self.send_header('Content-Length', 0)
+        self.send_header('Content-Type', 'text/plain')
+        self.end_headers()
+        raise RequestDone
+
     def send_file(self, path, mimetype=None):
         """Send a local file to the browser.
-        
+
         This method includes the "Last-Modified", "Content-Type" and
         "Content-Length" headers in the response, corresponding to the file
         attributes. It also checks the last modification time of the local file
@@ -512,9 +575,12 @@ class Request(object):
         self.send_header('Content-Type', mimetype)
         self.send_header('Content-Length', stat.st_size)
         self.send_header('Last-Modified', last_modified)
+        use_xsendfile = getattr(self, 'use_xsendfile', False)
+        if use_xsendfile:
+            self.send_header('X-Sendfile', os.path.abspath(path))
         self.end_headers()
 
-        if self.method != 'HEAD':
+        if not use_xsendfile and self.method != 'HEAD':
             fileobj = file(path, 'rb')
             file_wrapper = self.environ.get('wsgi.file_wrapper', _FileWrapper)
             self._response = file_wrapper(fileobj, 4096)
@@ -539,8 +605,8 @@ class Request(object):
         which has been specified in the ''Content-Type'' header
         or 'utf-8' otherwise.
 
-        Note that the ''Content-Length'' header must have been specified. 
-        Its value either corresponds to the length of `data`, or, if there 
+        Note that the ''Content-Length'' header must have been specified.
+        Its value either corresponds to the length of `data`, or, if there
         are multiple calls to `write`, to the cumulated length of the `data`
         arguments.
         """
@@ -573,7 +639,7 @@ class Request(object):
         if ctype not in ('application/x-www-form-urlencoded',
                          'multipart/form-data'):
             fp = StringIO('')
-        
+
         # Python 2.6 introduced a backwards incompatible change for
         # FieldStorage where QUERY_STRING is no longer ignored for POST
         # requests. We'll keep the pre 2.6 behaviour for now...
@@ -582,7 +648,7 @@ class Request(object):
         fs = _FieldStorage(fp, environ=self.environ, keep_blank_values=True)
         if self.method == 'POST':
             self.environ['QUERY_STRING'] = qs_on_post
-        
+
         args = []
         for value in fs.list or ():
             name = value.name
@@ -654,97 +720,4 @@ class Request(object):
         for cookie in cookies.splitlines():
             self._outheaders.append(('Set-Cookie', cookie.strip()))
 
-
-class IAuthenticator(Interface):
-    """Extension point interface for components that can provide the name
-    of the remote user."""
-
-    def authenticate(req):
-        """Return the name of the remote user, or `None` if the identity of the
-        user is unknown."""
-
-
-class IRequestHandler(Interface):
-    """Extension point interface for request handlers."""
-
-    def match_request(req):
-        """Return whether the handler wants to process the given request."""
-
-    def process_request(req):
-        """Process the request.
-
-        For ClearSilver, return a `(template_name, content_type)` tuple,
-        where `template` is the ClearSilver template to use (either a
-        `neo_cs.CS` object, or the file name of the template), and
-        `content_type` is the MIME type of the content.
-
-        For Genshi, return a `(template_name, data, content_type)` tuple,
-        where `data` is a dictionary of substitutions for the template.
-
-        For either templating systems, "text/html" is assumed if `content_type`
-        is `None`.
-
-        Note that if template processing should not occur, this method can
-        simply send the response itself and not return anything.
-        """
-
-
-class IRequestFilter(Interface):
-    """Extension point interface for components that want to filter HTTP
-    requests, before and/or after they are processed by the main handler."""
-
-    def pre_process_request(req, handler):
-        """Called after initial handler selection, and can be used to change
-        the selected handler or redirect request.
-        
-        Always returns the request handler, even if unchanged.
-        """
-
-    # for ClearSilver templates
-    def post_process_request(req, template, content_type):
-        """Do any post-processing the request might need; typically adding
-        values to req.hdf, or changing template or mime type.
-        
-        Always returns a tuple of (template, content_type), even if
-        unchanged.
-
-        Note that `template`, `content_type` will be `None` if:
-         - called when processing an error page
-         - the default request handler did not return any result
-
-        (for 0.10 compatibility; only used together with ClearSilver templates)
-        """
-
-    # for Genshi templates
-    def post_process_request(req, template, data, content_type):
-        """Do any post-processing the request might need; typically adding
-        values to the template `data` dictionary, or changing template or
-        mime type.
-        
-        `data` may be update in place.
-
-        Always returns a tuple of (template, data, content_type), even if
-        unchanged.
-
-        Note that `template`, `data`, `content_type` will be `None` if:
-         - called when processing an error page
-         - the default request handler did not return any result
-
-        (Since 0.11)
-        """
-
-
-class ITemplateStreamFilter(Interface):
-    """Filter a Genshi event stream prior to rendering."""
-
-    def filter_stream(req, method, filename, stream, data):
-        """Return a filtered Genshi event stream, or the original unfiltered
-        stream if no match.
-
-        `req` is the current request object, `method` is the Genshi render
-        method (xml, xhtml or text), `filename` is the filename of the template
-        to be rendered, `stream` is the event stream and `data` is the data for
-        the current template.
-
-        See the Genshi documentation for more information.
-        """
+__no_apidoc__ = _HTTPException_subclass_names
