@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2009 Edgewall Software
+# Copyright (C) 2005-2010 Edgewall Software
 # Copyright (C) 2005-2006 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
@@ -14,15 +14,25 @@
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
+"""Content presentation for the web layer.
+
+The Chrome module deals with delivering and shaping content to the end user,
+mostly targeting (X)HTML generation but not exclusively, RSS or other forms of
+web content are also using facilities provided here.
+"""
+
+from __future__ import with_statement
+
 import datetime
+from functools import partial
 import itertools
 import os.path
 import pkg_resources
 import pprint
 import re
-try: 
+try:
     from cStringIO import StringIO
-except ImportError: 
+except ImportError:
     from StringIO import StringIO
 
 from genshi import Markup
@@ -36,17 +46,20 @@ from trac import __version__ as VERSION
 from trac.config import *
 from trac.core import *
 from trac.env import IEnvironmentSetupParticipant, ISystemInfoProvider
-from trac.mimeview import get_mimetype, Context
+from trac.mimeview.api import RenderingContext, get_mimetype
 from trac.resource import *
 from trac.util import compat, get_reporter_id, presentation, get_pkginfo, \
                       pathjoin, translation
-from trac.util.compat import any, partial
 from trac.util.html import escape, plaintext
 from trac.util.text import pretty_size, obfuscate_email_address, \
                            shorten_line, unicode_quote_plus, to_unicode, \
                            javascript_quote, exception_to_unicode
-from trac.util.datefmt import pretty_timedelta, format_datetime, format_date, \
-                              format_time, from_utimestamp, http_date, utc
+from trac.util.datefmt import (
+    pretty_timedelta, format_datetime, format_date, format_time,
+    from_utimestamp, http_date, utc, get_date_format_jquery_ui, is_24_hours,
+    get_time_format_jquery_ui, user_time, get_month_names_jquery_ui,
+    get_day_names_jquery_ui, get_timezone_list_jquery_ui,
+    get_first_week_day_jquery_ui)
 from trac.util.translation import _, get_available_locales
 from trac.web.api import IRequestHandler, ITemplateStreamFilter, HTTPNotFound
 from trac.web.href import Href
@@ -54,6 +67,48 @@ from trac.wiki import IWikiSyntaxProvider
 from trac.wiki.formatter import format_to, format_to_html, format_to_oneliner
 
 
+class INavigationContributor(Interface):
+    """Extension point interface for components that contribute items to the
+    navigation.
+    """
+
+    def get_active_navigation_item(req):
+        """This method is only called for the `IRequestHandler` processing the
+        request.
+
+        It should return the name of the navigation item that should be
+        highlighted as active/current.
+        """
+
+    def get_navigation_items(req):
+        """Should return an iterable object over the list of navigation items
+        to add, each being a tuple in the form (category, name, text).
+        """
+
+
+class ITemplateProvider(Interface):
+    """Extension point interface for components that provide their own
+    Genshi templates and accompanying static resources.
+    """
+
+    def get_htdocs_dirs():
+        """Return a list of directories with static resources (such as style
+        sheets, images, etc.)
+
+        Each item in the list must be a `(prefix, abspath)` tuple. The
+        `prefix` part defines the path in the URL that requests to these
+        resources are prefixed with.
+
+        The `abspath` is the absolute path to the directory containing the
+        resources on the local file system.
+        """
+
+    def get_templates_dirs():
+        """Return a list of directories containing the provided template
+        files.
+        """
+
+
 def add_meta(req, content, http_equiv=None, name=None, scheme=None, lang=None):
     """Add a `<meta>` tag into the `<head>` of the generated HTML."""
     meta = {'content': content, 'http-equiv': http_equiv, 'name': name,
@@ -79,23 +134,26 @@ def add_link(req, rel, href, title=None, mimetype=None, classname=None,
 def add_stylesheet(req, filename, mimetype='text/css', media=None):
     """Add a link to a style sheet to the chrome info so that it gets included
     in the generated HTML page.
-    
+
     If the filename is absolute (i.e. starts with a slash), the generated link
     will be based off the application root path. If it is relative, the link
     will be based off the `/chrome/` path.
     """
-    if filename.startswith('common/') and 'htdocs_location' in req.chrome:
-        href = Href(req.chrome['htdocs_location'])
-        filename = filename[7:]
+    if filename.startswith(('http://', 'https://')):
+        href = filename
+    elif filename.startswith('common/') and 'htdocs_location' in req.chrome:
+        href = Href(req.chrome['htdocs_location'])(filename[7:])
     else:
         href = req.href
         if not filename.startswith('/'):
             href = href.chrome
-    add_link(req, 'stylesheet', href(filename), mimetype=mimetype, media=media)
+        href = href(filename)
+    add_link(req, 'stylesheet', href, mimetype=mimetype, media=media)
 
-def add_script(req, filename, mimetype='text/javascript'):
+def add_script(req, filename, mimetype='text/javascript', charset='utf-8',
+               ie_if=None):
     """Add a reference to an external javascript file to the template.
-    
+
     If the filename is absolute (i.e. starts with a slash), the generated link
     will be based off the application root path. If it is relative, the link
     will be based off the `/chrome/` path.
@@ -104,44 +162,62 @@ def add_script(req, filename, mimetype='text/javascript'):
     if filename in scriptset:
         return False # Already added that script
 
-    if filename.startswith('common/') and 'htdocs_location' in req.chrome:
-        href = Href(req.chrome['htdocs_location'])
-        path = filename[7:]
+    if filename.startswith(('http://', 'https://')):
+        href = filename
+    elif filename.startswith('common/') and 'htdocs_location' in req.chrome:
+        href = Href(req.chrome['htdocs_location'])(filename[7:])
     else:
         href = req.href
         if not filename.startswith('/'):
             href = href.chrome
-        path = filename
-    script = {'href': href(path), 'type': mimetype}
+        href = href(filename)
+    script = {'href': href, 'type': mimetype, 'charset': charset,
+              'prefix': Markup('<!--[if %s]>' % ie_if) if ie_if else None,
+              'suffix': Markup('<![endif]-->') if ie_if else None}
 
     req.chrome.setdefault('scripts', []).append(script)
     scriptset.add(filename)
 
-def add_script_data(req, data):
+def add_script_data(req, data={}, **kwargs):
     """Add data to be made available in javascript scripts as global variables.
-    
-    The keys in `data` provide the names of the global variables. The values
-    are converted to JSON and assigned to the corresponding variables.
+
+    The keys in `data` and the keyword argument names provide the names of the
+    global variables. The values are converted to JSON and assigned to the
+    corresponding variables.
     """
-    req.chrome.setdefault('script_data', {}).update(data)
+    script_data = req.chrome.setdefault('script_data', {})
+    script_data.update(data)
+    script_data.update(kwargs)
 
 def add_javascript(req, filename):
-    """Deprecated: use `add_script()` instead."""
+    """:deprecated: use `add_script` instead."""
     add_script(req, filename, mimetype='text/javascript')
 
 def add_warning(req, msg, *args):
     """Add a non-fatal warning to the request object.
-    When rendering pages, any warnings will be rendered to the user."""
+
+    When rendering pages, all warnings will be rendered to the user. Note that
+    the message is escaped (and therefore converted to `Markup`) before it is
+    stored in the request object.
+    """
     if args:
         msg %= args
-    req.chrome['warnings'].append(msg)
+    msg = escape(msg, False)
+    if msg not in req.chrome['warnings']:
+        req.chrome['warnings'].append(msg)
 
 def add_notice(req, msg, *args):
     """Add an informational notice to the request object.
-    When rendering pages, any notice will be rendered to the user."""
+
+    When rendering pages, all notices will be rendered to the user. Note that
+    the message is escaped (and therefore converted to `Markup`) before it is
+    stored in the request object.
+    """
     if args:
         msg %= args
-    req.chrome['notices'].append(msg)
+    msg = escape(msg, False)
+    if msg not in req.chrome['notices']:
+        req.chrome['notices'].append(msg)
 
 def add_ctxtnav(req, elm_or_label, href=None, title=None):
     """Add an entry to the current page's ctxtnav bar."""
@@ -154,24 +230,24 @@ def add_ctxtnav(req, elm_or_label, href=None, title=None):
 def prevnext_nav(req, prev_label, next_label, up_label=None):
     """Add Previous/Up/Next navigation links.
 
-       @param req        a `Request` object
-       @param prev_label the label to use for left (previous) link
-       @param up_label   the label to use for the middle (up) link
-       @param next_label the label to use for right (next) link
+       :param        req: a `Request` object
+       :param prev_label: the label to use for left (previous) link
+       :param   up_label: the label to use for the middle (up) link
+       :param next_label: the label to use for right (next) link
     """
     links = req.chrome['links']
     prev_link = next_link = None
-    
+
     if not any(lnk in links for lnk in ('prev', 'up', 'next')): # Short circuit
         return
-    
+
     if 'prev' in links:
         prev = links['prev'][0]
         prev_link = tag.a(prev_label, href=prev['href'], title=prev['title'],
                           class_='prev')
-        
+
     add_ctxtnav(req, tag.span(Markup('&larr; '), prev_link or prev_label,
-                              class_=not prev_link and 'missing' or None))
+                              class_='missing' if not prev_link else None))
 
     if up_label and 'up' in links:
         up = links['up'][0]
@@ -183,7 +259,56 @@ def prevnext_nav(req, prev_label, next_label, up_label=None):
                           class_='next')
 
     add_ctxtnav(req, tag.span(next_link or next_label, Markup(' &rarr;'),
-                              class_=not next_link and 'missing' or None))
+                              class_='missing' if not next_link else None))
+
+
+def web_context(req, resource=None, id=False, version=False, parent=False,
+                absurls=False):
+    """Create a rendering context from a request.
+
+    The `perm` and `href` properties of the context will be initialized
+    from the corresponding properties of the request object.
+
+    >>> from trac.test import Mock, MockPerm
+    >>> req = Mock(href=Mock(), perm=MockPerm())
+    >>> context = web_context(req)
+    >>> context.href is req.href
+    True
+    >>> context.perm is req.perm
+    True
+
+    :param      req: the HTTP request object
+    :param resource: the `Resource` object or realm
+    :param       id: the resource identifier
+    :param  version: the resource version
+    :param  absurls: whether URLs generated by the ``href`` object should
+                     be absolute (including the protocol scheme and host
+                     name)
+    :return: a new rendering context
+    :rtype: `RenderingContext`
+    """
+    if req:
+        href = req.abs_href if absurls else req.href
+        perm = req.perm
+    else:
+        href = None
+        perm = None
+    self = RenderingContext(Resource(resource, id=id, version=version,
+                                     parent=parent), href=href, perm=perm)
+    self.req = req
+    return self
+
+
+def auth_link(req, link):
+    """Return an "authenticated" link to `link` for authenticated users.
+
+    If the user is anonymous, returns `link` unchanged. For authenticated
+    users, returns a link to `/login` that redirects to `link` after
+    authentication.
+    """
+    if req.authname != 'anonymous':
+        return req.href.login(referer=link)
+    return link
 
 
 def _save_messages(req, url, permanent):
@@ -191,49 +316,7 @@ def _save_messages(req, url, permanent):
     be displayed after the redirect."""
     for type_ in ['warnings', 'notices']:
         for (i, message) in enumerate(req.chrome[type_]):
-            req.session['chrome.%s.%d' % (type_, i)] = escape(message)
-
-
-class INavigationContributor(Interface):
-    """Extension point interface for components that contribute items to the
-    navigation.
-    """
-
-    def get_active_navigation_item(req):
-        """This method is only called for the `IRequestHandler` processing the
-        request.
-        
-        It should return the name of the navigation item that should be
-        highlighted as active/current.
-        """
-
-    def get_navigation_items(req):
-        """Should return an iterable object over the list of navigation items to
-        add, each being a tuple in the form (category, name, text).
-        """
-
-
-class ITemplateProvider(Interface):
-    """Extension point interface for components that provide their own
-    ClearSilver templates and accompanying static resources.
-    """
-
-    def get_htdocs_dirs():
-        """Return a list of directories with static resources (such as style
-        sheets, images, etc.)
-
-        Each item in the list must be a `(prefix, abspath)` tuple. The
-        `prefix` part defines the path in the URL that requests to these
-        resources are prefixed with.
-        
-        The `abspath` is the absolute path to the directory containing the
-        resources on the local file system.
-        """
-
-    def get_templates_dirs():
-        """Return a list of directories containing the provided template
-        files.
-        """
+            req.session['chrome.%s.%d' % (type_, i)] = escape(message, False)
 
 
 # Mappings for removal of control characters
@@ -241,14 +324,14 @@ _translate_nop = "".join([chr(i) for i in range(256)])
 _invalid_control_chars = "".join([chr(i) for i in range(32)
                                   if i not in [0x09, 0x0a, 0x0d]])
 
-    
+
 class Chrome(Component):
     """Web site chrome assembly manager.
-    
+
     Chrome is everything that is not actual page content.
     """
     required = True
-    
+
     implements(ISystemInfoProvider, IEnvironmentSetupParticipant,
                IRequestHandler, ITemplateProvider, IWikiSyntaxProvider)
 
@@ -258,15 +341,26 @@ class Chrome(Component):
 
     shared_templates_dir = PathOption('inherit', 'templates_dir', '',
         """Path to the //shared templates directory//.
-        
+
         Templates in that directory are loaded in addition to those in the
         environments `templates` directory, but the latter take precedence.
-        
+
         (''since 0.11'')""")
 
+    shared_htdocs_dir = PathOption('inherit', 'htdocs_dir', '',
+        """Path to the //shared htdocs directory//.
+
+        Static resources in that directory are mapped to /chrome/shared
+        under the environment URL, in addition to common and site locations.
+
+        This can be useful in site.html for common interface customization
+        of multiple Trac environments.
+
+        (''since 1.0'')""")
+
     auto_reload = BoolOption('trac', 'auto_reload', False,
         """Automatically reload template files after modification.""")
-    
+
     genshi_cache_size = IntOption('trac', 'genshi_cache_size', 128,
         """The maximum number of templates that the template loader will cache
         in memory. The default value is 128. You may want to choose a higher
@@ -275,7 +369,7 @@ class Chrome(Component):
         memory.""")
 
     htdocs_location = Option('trac', 'htdocs_location', '',
-        """Base URL for serving the core static resources below 
+        """Base URL for serving the core static resources below
         `/chrome/common/`.
 
         It can be left empty, and Trac will simply serve those resources
@@ -285,19 +379,58 @@ class Chrome(Component):
         [TracAdmin trac-admin ... deploy <deploydir>] to allow serving the
         static resources for Trac directly from the web server.
         Note however that this only applies to the `<deploydir>/htdocs/common`
-        directory, the other deployed resources (i.e. those from plugins) 
-        will not be made available this way and additional rewrite 
+        directory, the other deployed resources (i.e. those from plugins)
+        will not be made available this way and additional rewrite
         rules will be needed in the web server.""")
 
+    jquery_location = Option('trac', 'jquery_location', '',
+        """Location of the jQuery !JavaScript library (version 1.7.2).
+
+        An empty value loads jQuery from the copy bundled with Trac.
+
+        Alternatively, jQuery could be loaded from a CDN, for example:
+        http://code.jquery.com/jquery-1.7.2.min.js,
+        http://ajax.aspnetcdn.com/ajax/jQuery/jquery-1.7.2.min.js or
+        https://ajax.googleapis.com/ajax/libs/jquery/1.7.2/jquery.min.js.
+
+        (''since 1.0'')""")
+
+    jquery_ui_location = Option('trac', 'jquery_ui_location', '',
+        """Location of the jQuery UI !JavaScript library (version 1.8.21).
+
+        An empty value loads jQuery UI from the copy bundled with Trac.
+
+        Alternatively, jQuery UI could be loaded from a CDN, for example:
+        https://ajax.googleapis.com/ajax/libs/jqueryui/1.8.21/jquery-ui.min.js
+        or
+        http://ajax.aspnetcdn.com/ajax/jquery.ui/1.8.21/jquery-ui.min.js.
+
+        (''since 1.0'')""")
+
+    jquery_ui_theme_location = Option('trac', 'jquery_ui_theme_location', '',
+        """Location of the theme to be used with the jQuery UI !JavaScript
+        library (version 1.8.21).
+
+        An empty value loads the custom Trac jQuery UI theme from the copy
+        bundled with Trac.
+
+        Alternatively, a jQuery UI theme could be loaded from a CDN, for
+        example:
+        https://ajax.googleapis.com/ajax/libs/jqueryui/1.8.21/themes/start/jquery-ui.css
+        or
+        http://ajax.aspnetcdn.com/ajax/jquery.ui/1.8.21/themes/start/jquery-ui.css.
+
+        (''since 1.0'')""")
+
     metanav_order = ListOption('trac', 'metanav',
-                               'login,logout,prefs,help,about', doc=
+                               'login, logout, prefs, help, about', doc=
         """Order of the items to display in the `metanav` navigation bar,
            listed by IDs. See also TracNavigation.""")
 
     mainnav_order = ListOption('trac', 'mainnav',
-                               'wiki,timeline,roadmap,browser,tickets,'
-                               'newticket,search', doc=
-        """Order of the items to display in the `mainnav` navigation bar, 
+                               'wiki, timeline, roadmap, browser, tickets, '
+                               'newticket, search', doc=
+        """Order of the items to display in the `mainnav` navigation bar,
            listed by IDs. See also TracNavigation.""")
 
     logo_link = Option('header_logo', 'link', '',
@@ -311,10 +444,10 @@ class Chrome(Component):
         `site/your-logo.png` if `your-logo.png` is located in the `htdocs`
         folder within your TracEnvironment;
         `common/your-logo.png` if `your-logo.png` is located in the
-        folder mapped to the [#trac-section htdocs_location] URL. 
+        folder mapped to the [#trac-section htdocs_location] URL.
         Only specifying `your-logo.png` is equivalent to the latter.""")
 
-    logo_alt = Option('header_logo', 'alt', 
+    logo_alt = Option('header_logo', 'alt',
         "(please configure the [header_logo] section in trac.ini)",
         """Alternative text for the header logo.""")
 
@@ -328,10 +461,10 @@ class Chrome(Component):
         """Show email addresses instead of usernames. If false, we obfuscate
         email addresses. (''since 0.11'')""")
 
-    never_obfuscate_mailto = BoolOption('trac', 'never_obfuscate_mailto', 
+    never_obfuscate_mailto = BoolOption('trac', 'never_obfuscate_mailto',
         'false',
-        """Never obfuscate `mailto:` links explicitly written in the wiki, 
-        even if `show_email_addresses` is false or the user has not the 
+        """Never obfuscate `mailto:` links explicitly written in the wiki,
+        even if `show_email_addresses` is false or the user has not the
         EMAIL_VIEW permission (''since 0.11.6'').""")
 
     show_ip_addresses = BoolOption('trac', 'show_ip_addresses', 'false',
@@ -349,13 +482,20 @@ class Chrome(Component):
         this to 0 to disable automatic preview. The default is 2.0 seconds.
         (''since 0.12'')""")
 
+    default_dateinfo_format = Option('trac', 'default_dateinfo_format',
+        'relative',
+        """The date information format. Valid options are 'relative' for
+        displaying relative format and 'absolute' for displaying absolute
+        format. (''since 1.0'')
+        """)
+
     templates = None
 
     # A dictionary of default context data for templates
     _default_context_data = {
         '_': translation.gettext,
-        'all': compat.all,
-        'any': compat.any,
+        'all': all,
+        'any': any,
         'classes': presentation.classes,
         'date': datetime.date,
         'datetime': datetime.datetime,
@@ -390,7 +530,7 @@ class Chrome(Component):
     }
 
     # ISystemInfoProvider methods
-    
+
     def get_system_info(self):
         import genshi
         info = get_pkginfo(genshi).get('version')
@@ -420,8 +560,7 @@ class Chrome(Component):
                 os.mkdir(templates_dir)
 
             site_path = os.path.join(templates_dir, 'site.html.sample')
-            fileobj = open(site_path, 'w')
-            try:
+            with open(site_path, 'w') as fileobj:
                 fileobj.write("""\
 <html xmlns="http://www.w3.org/1999/xhtml"
       xmlns:xi="http://www.w3.org/2001/XInclude"
@@ -432,15 +571,13 @@ class Chrome(Component):
     Add your customizations here and rename the file to site.html. Note that
     it will take precedence over a global site.html placed in the directory
     specified by [inherit] templates_dir.
-    
+
     More information about site appearance customization can be found here:
-    
+
       http://trac.edgewall.org/wiki/TracInterfaceCustomization#SiteAppearance
   -->
 </html>
 """)
-            finally:
-                fileobj.close()
 
     def environment_needs_upgrade(self, db):
         return False
@@ -480,6 +617,7 @@ class Chrome(Component):
 
     def get_htdocs_dirs(self):
         return [('common', pkg_resources.resource_filename('trac', 'htdocs')),
+                ('shared', self.shared_htdocs_dir),
                 ('site', self.env.get_htdocs_dir())]
 
     def get_templates_dirs(self):
@@ -490,10 +628,10 @@ class Chrome(Component):
         ])
 
     # IWikiSyntaxProvider methods
-    
+
     def get_wiki_syntax(self):
         return []
-    
+
     def get_link_resolvers(self):
         yield ('htdocs', self._format_link)
 
@@ -513,16 +651,16 @@ class Chrome(Component):
 
     def prepare_request(self, req, handler=None):
         """Prepare the basic chrome data for the request.
-        
-        @param req: the request object
-        @param handler: the `IRequestHandler` instance that is processing the
-            request
+
+        :param     req: the request object
+        :param handler: the `IRequestHandler` instance that is processing the
+                        request
         """
         self.log.debug('Prepare chrome data for request')
 
         chrome = {'metas': [], 'links': {}, 'scripts': [], 'script_data': {},
                   'ctxtnav': [], 'warnings': [], 'notices': []}
-        setattr(req, 'chrome', chrome)
+        req.chrome = chrome
 
         htdocs_location = self.htdocs_location or req.href.chrome('common')
         chrome['htdocs_location'] = htdocs_location.rstrip('/') + '/'
@@ -532,7 +670,7 @@ class Chrome(Component):
         add_link(req, 'search', req.href.search())
         add_link(req, 'help', req.href.wiki('TracGuide'))
         add_stylesheet(req, 'common/css/trac.css')
-        add_script(req, 'common/js/jquery.js')
+        add_script(req, self.jquery_location or 'common/js/jquery.js')
         # Only activate noConflict mode if requested to by the handler
         if handler is not None and \
            getattr(handler.__class__, 'jquery_noconflict', False):
@@ -618,7 +756,7 @@ class Chrome(Component):
                 })
 
         chrome['nav'] = nav
-        
+
         # Default theme file
         chrome['theme'] = 'theme.html'
 
@@ -649,9 +787,7 @@ class Chrome(Component):
         logo_src = self.logo_src
         if logo_src:
             abs_href = abs_href or href
-            if logo_src.startswith('http://') or \
-                    logo_src.startswith('https://') or \
-                    logo_src.startswith('/'):
+            if logo_src.startswith(('http://', 'https://', '/')):
                 # Nothing further can be calculated
                 logo_src_abs = logo_src
             elif '/' in logo_src:
@@ -662,8 +798,8 @@ class Chrome(Component):
                 # Like 'trac_banner.png'
                 logo_src_abs = abs_href.chrome('common', logo_src)
                 logo_src = href.chrome('common', logo_src)
-            width = self.logo_width > -1 and self.logo_width or None
-            height = self.logo_height > -1 and self.logo_height or None
+            width = self.logo_width if self.logo_width > -1 else None
+            height = self.logo_height if self.logo_height > -1 else None
             logo = {
                 'link': self.logo_link, 'src': logo_src,
                 'src_abs': logo_src_abs, 'alt': self.logo_alt,
@@ -673,34 +809,21 @@ class Chrome(Component):
             logo = {'link': self.logo_link, 'alt': self.logo_alt}
         return logo
 
-    def populate_hdf(self, req):
-        """Add chrome-related data to the HDF (deprecated)."""
-        req.hdf['HTTP.PathInfo'] = req.path_info
-        req.hdf['htdocs_location'] = req.chrome['htdocs_location']
-
-        req.hdf['chrome.href'] = req.href.chrome()
-        req.hdf['chrome.links'] = req.chrome['links']
-        req.hdf['chrome.scripts'] = req.chrome['scripts']
-        req.hdf['chrome.logo'] = req.chrome['logo']
-
-        for category, items in req.chrome['nav'].items():
-            req.hdf['chrome.nav.%s' % category] = items
-
     def populate_data(self, req, data):
         d = self._default_context_data.copy()
         d['trac'] = {
             'version': VERSION,
             'homepage': 'http://trac.edgewall.org/', # FIXME: use setup data
         }
-        
+
         href = req and req.href
-        abs_href = req and req.abs_href or self.env.abs_href
+        abs_href = req.abs_href if req else self.env.abs_href
         admin_href = None
         if self.env.project_admin_trac_url == '.':
             admin_href = href
         elif self.env.project_admin_trac_url:
             admin_href = Href(self.env.project_admin_trac_url)
-            
+
         d['project'] = {
             'name': self.env.project_name,
             'descr': self.env.project_description,
@@ -709,8 +832,9 @@ class Chrome(Component):
             'admin_href': admin_href,
             'admin_trac_url': self.env.project_admin_trac_url,
         }
+        footer = self.env.project_footer
         d['chrome'] = {
-            'footer': Markup(translation.gettext(self.env.project_footer))
+            'footer': Markup(footer and translation.gettext(footer))
         }
         if req:
             d['chrome'].update(req.chrome)
@@ -726,16 +850,27 @@ class Chrome(Component):
         except Exception, e:
             # simply log the exception here, as we might already be rendering
             # the error page
-            self.log.error("Error during check of EMAIL_VIEW: %s", 
+            self.log.error("Error during check of EMAIL_VIEW: %s",
                            exception_to_unicode(e))
             show_email_addresses = False
-        tzinfo = None
-        if req:
-            tzinfo = req.tz
+
+        def pretty_dateinfo(date, format=None, dateonly=False):
+            absolute = user_time(req, format_datetime, date)
+            relative = pretty_timedelta(date)
+            if not format:
+                format = req.session.get('dateinfo',
+                                         self.default_dateinfo_format)
+            if format == 'absolute':
+                label = absolute
+                title = _("%(relativetime)s ago", relativetime=relative)
+            else:
+                label = _("%(relativetime)s ago", relativetime=relative) \
+                        if not dateonly else relative
+                title = absolute
+            return tag.span(label, title=title)
 
         def dateinfo(date):
-            return tag.span(pretty_timedelta(date),
-                            title=format_datetime(date))
+            return pretty_dateinfo(date, format='relative', dateonly=True)
 
         def get_rel_url(resource, **kwargs):
             return get_resource_url(self.env, resource, href, **kwargs)
@@ -744,7 +879,7 @@ class Chrome(Component):
             return get_resource_url(self.env, resource, abs_href, **kwargs)
 
         d.update({
-            'context': req and Context.from_request(req) or None,
+            'context': web_context(req) if req else None,
             'Resource': Resource,
             'url_of': get_rel_url,
             'abs_url_of': get_abs_url,
@@ -755,7 +890,7 @@ class Chrome(Component):
             'abs_href': abs_href,
             'href': href,
             'perm': req and req.perm,
-            'authname': req and req.authname or '<trac>',
+            'authname': req.authname if req else '<trac>',
             'locale': req and req.locale,
             'show_email_addresses': show_email_addresses,
             'show_ip_addresses': self.show_ip_addresses,
@@ -764,14 +899,16 @@ class Chrome(Component):
             'format_author': partial(self.format_author, req),
             'format_emails': self.format_emails,
             'get_systeminfo': self.env.get_systeminfo,
+            'captioned_button': partial(presentation.captioned_button, req),
 
             # Date/time formatting
             'dateinfo': dateinfo,
-            'format_datetime': partial(format_datetime, tzinfo=tzinfo),
-            'format_date': partial(format_date, tzinfo=tzinfo),
-            'format_time': partial(format_time, tzinfo=tzinfo),
+            'pretty_dateinfo': pretty_dateinfo,
+            'format_datetime': partial(user_time, req, format_datetime),
+            'format_date': partial(user_time, req, format_date),
+            'format_time': partial(user_time, req, format_time),
             'fromtimestamp': partial(datetime.datetime.fromtimestamp,
-                                     tz=tzinfo),
+                                     tz=req and req.tz),
             'from_utimestamp': from_utimestamp,
 
             # Wiki-formatting functions
@@ -795,6 +932,7 @@ class Chrome(Component):
             self.templates = TemplateLoader(
                 self.get_all_templates_dirs(), auto_reload=self.auto_reload,
                 max_cache_size=self.genshi_cache_size,
+                default_encoding="utf-8",
                 variable_lookup='lenient', callback=lambda template:
                 Translator(translation.get_translations()).setup(template))
 
@@ -827,8 +965,10 @@ class Chrome(Component):
             for type_ in ['warnings', 'notices']:
                 try:
                     for i in itertools.count():
-                        message = req.session.pop('chrome.%s.%d' % (type_, i))
-                        req.chrome[type_].append(Markup(message))
+                        message = Markup(req.session.pop('chrome.%s.%d'
+                                                         % (type_, i)))
+                        if message not in req.chrome[type_]:
+                            req.chrome[type_].append(message)
                 except KeyError:
                     pass
 
@@ -888,8 +1028,8 @@ class Chrome(Component):
                 else:
                     location = _("(unknown template location)")
                 raise TracError(_("Genshi %(error)s error while rendering "
-                                  "template %(location)s", 
-                                  error=e.__class__.__name__, 
+                                  "template %(location)s",
+                                  error=e.__class__.__name__,
                                   location=location))
             raise
 
@@ -909,16 +1049,16 @@ class Chrome(Component):
 
         :param context: the context in which the check for obfuscation should
                         be done
-        :param value: a string containing a comma-separated list of e-mails
-        :param sep: the separator to use when rendering the list again
+        :param   value: a string containing a comma-separated list of e-mails
+        :param     sep: the separator to use when rendering the list again
         """
         all_cc = self.cc_list(value)
         if not (self.show_email_addresses or 'EMAIL_VIEW' in context.perm):
             all_cc = [obfuscate_email_address(cc) for cc in all_cc]
         return sep.join(all_cc)
-    
+
     def authorinfo(self, req, author, email_map=None):
-        return self.format_author(req, 
+        return self.format_author(req,
                                   email_map and '@' not in author and
                                   email_map.get(author) or author)
 
@@ -930,9 +1070,9 @@ class Chrome(Component):
                 if email:
                     email_map[username] = email
         return email_map
-        
+
     _long_author_re = re.compile(r'.*<([^@]+)@[^@]+>\s*|([^@]+)@[^@]+')
-    
+
     def authorinfo_short(self, author):
         if not author or author == 'anonymous':
             return _("anonymous")
@@ -964,9 +1104,32 @@ class Chrome(Component):
     def add_auto_preview(self, req):
         """Setup auto-preview for `<textarea>` fields."""
         add_script(req, 'common/js/auto_preview.js')
-        add_script_data(req, {
-            'auto_preview_timeout': self.auto_preview_timeout,
-            'form_token': req.form_token})
+        add_script_data(req, auto_preview_timeout=self.auto_preview_timeout,
+                        form_token=req.form_token)
+
+    def add_jquery_ui(self, req):
+        """Add a reference to the jQuery UI script and link the stylesheet."""
+        add_script(req, self.jquery_ui_location
+                        or 'common/js/jquery-ui.js')
+        add_stylesheet(req, self.jquery_ui_theme_location
+                            or 'common/css/jquery-ui/jquery-ui.css')
+        add_script(req, 'common/js/jquery-ui-addons.js')
+        add_stylesheet(req, 'common/css/jquery-ui-addons.css')
+        is_iso8601 = req.lc_time == 'iso8601'
+        add_script_data(req, jquery_ui={
+            'month_names': get_month_names_jquery_ui(req),
+            'day_names': get_day_names_jquery_ui(req),
+            'date_format': get_date_format_jquery_ui(req.lc_time),
+            'time_format': get_time_format_jquery_ui(req.lc_time),
+            'ampm': not is_24_hours(req.lc_time),
+            'first_week_day': get_first_week_day_jquery_ui(req),
+            'timepicker_separator': 'T' if is_iso8601 else ' ',
+            'show_timezone': is_iso8601,
+            'timezone_list': get_timezone_list_jquery_ui() \
+                             if is_iso8601 else [],
+            'timezone_iso8601': is_iso8601,
+        })
+        add_script(req, 'common/js/jquery-ui-i18n.js')
 
     # Template filters
 
