@@ -18,6 +18,9 @@
 # Author: Daniel Lundin <daniel@edgewall.com>
 #         Christopher Lenz <cmlenz@gmx.de>
 
+from __future__ import with_statement
+
+import sys
 import time
 
 from trac.admin.api import console_date_format
@@ -66,35 +69,31 @@ class DetachedSession(dict):
                 self.pop(key, None)
                 return
         dict.__setitem__(self, key, value)
-        
+
     def get_session(self, sid, authenticated=False):
-        self.env.log.debug('Retrieving session for ID %r', sid)
+        self.env.log.debug("Retrieving session for ID %r", sid)
 
-        db = self.env.get_db_cnx()
-        cursor = db.cursor()
+        with self.env.db_query as db:
+            self.sid = sid
+            self.authenticated = authenticated
+            self.clear()
 
-        self.sid = sid
-        self.authenticated = authenticated
-        self.clear()
-
-        cursor.execute("""
-            SELECT last_visit FROM session WHERE sid=%s AND authenticated=%s
-            """, (sid, int(authenticated)))
-        row = cursor.fetchone()
-        if not row:
-            self.last_visit = 0
-            self._new = True
-            self._old = {}
-            return
-        self._new = False
-        self.last_visit = int(row[0] or 0)
-
-        cursor.execute("""
-            SELECT name,value FROM session_attribute
-            WHERE sid=%s and authenticated=%s
-            """, (sid, int(authenticated)))
-        self.update(cursor)
-        self._old = self.copy()
+            for last_visit, in db("""
+                    SELECT last_visit FROM session
+                    WHERE sid=%s AND authenticated=%s
+                    """, (sid, int(authenticated))):
+                self._new = False
+                self.last_visit = int(last_visit or 0)
+                self.update(db("""
+                    SELECT name, value FROM session_attribute
+                    WHERE sid=%s and authenticated=%s
+                    """, (sid, int(authenticated))))
+                self._old = self.copy()
+                break
+            else:
+                self.last_visit = 0
+                self._new = True
+                self._old = {}
 
     def save(self):
         items = self.items()
@@ -111,13 +110,10 @@ class DetachedSession(dict):
         # session_attribute tables are prone to deadlocks (#9705).
         # Therefore we first we save the current session, then we
         # eventually purge the tables.
-        
-        session_saved = [False]
 
-        @self.env.with_transaction()
-        def save_session(db):
-            cursor = db.cursor()
+        session_saved = False
 
+        with self.env.db_transaction as db:
             # Try to save the session if it's a new one. A failure to
             # do so is not critical but we nevertheless skip the
             # following steps.
@@ -128,11 +124,10 @@ class DetachedSession(dict):
                 # The session might already exist even if _new is True since
                 # it could have been created by a concurrent request (#3563).
                 try:
-                    cursor.execute("""
-                        INSERT INTO session (sid,last_visit,authenticated)
-                        VALUES (%s,%s,%s)
-                        """, (self.sid, self.last_visit, authenticated))
-                except Exception:
+                    db("""INSERT INTO session (sid, last_visit, authenticated)
+                          VALUES (%s,%s,%s)
+                          """, (self.sid, self.last_visit, authenticated))
+                except self.env.db_exc.IntegrityError:
                     self.env.log.warning('Session %s already exists', self.sid)
                     db.rollback()
                     return
@@ -143,62 +138,54 @@ class DetachedSession(dict):
             if self._old != self:
                 if not items and not authenticated:
                     # No need to keep around empty unauthenticated sessions
-                    cursor.execute("""
-                        DELETE FROM session WHERE sid=%s AND authenticated=0
-                        """, (self.sid,))
-                cursor.execute("""
-                    DELETE FROM session_attribute
-                    WHERE sid=%s AND authenticated=%s
-                    """, (self.sid, authenticated))
+                    db("DELETE FROM session WHERE sid=%s AND authenticated=0",
+                       (self.sid,))
+                db("""DELETE FROM session_attribute
+                      WHERE sid=%s AND authenticated=%s
+                      """, (self.sid, authenticated))
                 self._old = dict(self.items())
                 # The session variables might already have been updated by a
                 # concurrent request.
                 try:
-                    cursor.executemany("""
+                    db.executemany("""
                         INSERT INTO session_attribute
                           (sid,authenticated,name,value)
                         VALUES (%s,%s,%s,%s)
                         """, [(self.sid, authenticated, k, v)
                               for k, v in items])
-                except Exception:
+                except self.env.db_exc.IntegrityError:
                     self.env.log.warning('Attributes for session %s already '
                                          'updated', self.sid)
                     db.rollback()
                     return
-                session_saved[0] = True
+                session_saved = True
 
         # Purge expired sessions. We do this only when the session was
         # changed as to minimize the purging.
 
-        if session_saved[0] and now - self.last_visit > UPDATE_INTERVAL:
+        if session_saved and now - self.last_visit > UPDATE_INTERVAL:
             self.last_visit = now
             mintime = now - PURGE_AGE
 
-            @self.env.with_transaction()
-            def purge_session_attributes(db):
-                cursor = db.cursor()
+            with self.env.db_transaction as db:
                 # Update the session last visit time if it is over an
                 # hour old, so that session doesn't get purged
                 self.env.log.info("Refreshing session %s", self.sid)
-                cursor.execute("""
-                    UPDATE session SET last_visit=%s
-                    WHERE sid=%s AND authenticated=%s
-                    """, (self.last_visit, self.sid, authenticated))
+                db("""UPDATE session SET last_visit=%s
+                      WHERE sid=%s AND authenticated=%s
+                      """, (self.last_visit, self.sid, authenticated))
                 self.env.log.debug('Purging old, expired, sessions.')
-                cursor.execute("""
-                    DELETE FROM session_attribute
-                    WHERE authenticated=0 AND sid IN (
-                      SELECT sid FROM session 
-                      WHERE authenticated=0 AND last_visit < %s
-                    )
-                    """, (mintime,))
+                db("""DELETE FROM session_attribute
+                      WHERE authenticated=0 AND sid IN (
+                          SELECT sid FROM session
+                          WHERE authenticated=0 AND last_visit < %s
+                      )
+                      """, (mintime,))
 
             # Avoid holding locks on lot of rows on both session_attribute
             # and session tables
-            @self.env.with_transaction()
-            def purge_sessions(db):
-                cursor = db.cursor()
-                cursor.execute("""
+            with self.env.db_transaction as db:
+                db("""
                     DELETE FROM session
                     WHERE authenticated=0 AND last_visit < %s
                     """, (mintime,))
@@ -230,6 +217,8 @@ class Session(DetachedSession):
         self.req.outcookie[COOKIE_KEY]['expires'] = expires
         if self.env.secure_cookies:
             self.req.outcookie[COOKIE_KEY]['secure'] = True
+        if sys.version_info >= (2, 6):
+            self.req.outcookie[COOKIE_KEY]['httponly'] = True
 
     def get_session(self, sid, authenticated=False):
         refresh_cookie = False
@@ -251,24 +240,19 @@ class Session(DetachedSession):
         assert new_sid, 'Session ID cannot be empty'
         if new_sid == self.sid:
             return
-        @self.env.with_transaction()
-        def update_session_id(db):
-            cursor = db.cursor()
-            cursor.execute("SELECT sid FROM session WHERE sid=%s", (new_sid,))
-            if cursor.fetchone():
+        with self.env.db_transaction as db:
+            if db("SELECT sid FROM session WHERE sid=%s", (new_sid,)):
                 raise TracError(_("Session '%(id)s' already exists. "
                                   "Please choose a different session ID.",
                                   id=new_sid),
                                 _("Error renaming session"))
-            self.env.log.debug('Changing session ID %s to %s', self.sid,
+            self.env.log.debug("Changing session ID %s to %s", self.sid,
                                new_sid)
-            cursor.execute("""
-                UPDATE session SET sid=%s WHERE sid=%s AND authenticated=0
+            db("UPDATE session SET sid=%s WHERE sid=%s AND authenticated=0",
+               (new_sid, self.sid))
+            db("""UPDATE session_attribute SET sid=%s
+                  WHERE sid=%s and authenticated=0
                   """, (new_sid, self.sid))
-            cursor.execute("""
-                UPDATE session_attribute SET sid=%s 
-                WHERE sid=%s and authenticated=0
-                """, (new_sid, self.sid))
         self.sid = new_sid
         self.bake_cookie()
 
@@ -277,51 +261,43 @@ class Session(DetachedSession):
         is no preexisting session data for that user name.
         """
         assert self.req.authname != 'anonymous', \
-               'Cannot promote session of anonymous user'
+               "Cannot promote session of anonymous user"
+
+        with self.env.db_transaction as db:
+            authenticated_flags = [authenticated for authenticated, in db(
+                "SELECT authenticated FROM session WHERE sid=%s OR sid=%s",
+                (sid, self.req.authname))]
 
-        @self.env.with_transaction()
-        def update_session_id(db):
-            cursor = db.cursor()
-            cursor.execute("""
-                SELECT authenticated FROM session WHERE sid=%s OR sid=%s
-                """, (sid, self.req.authname))
-            authenticated_flags = [row[0] for row in cursor.fetchall()]
-            
             if len(authenticated_flags) == 2:
                 # There's already an authenticated session for the user,
                 # we simply delete the anonymous session
-                cursor.execute("""
-                    DELETE FROM session WHERE sid=%s AND authenticated=0
-                    """, (sid,))
-                cursor.execute("""
-                    DELETE FROM session_attribute
-                    WHERE sid=%s AND authenticated=0
-                    """, (sid,))
+                db("DELETE FROM session WHERE sid=%s AND authenticated=0",
+                   (sid,))
+                db("""DELETE FROM session_attribute
+                      WHERE sid=%s AND authenticated=0
+                      """, (sid,))
             elif len(authenticated_flags) == 1:
                 if not authenticated_flags[0]:
                     # Update the anomymous session records so the session ID
                     # becomes the user name, and set the authenticated flag.
-                    self.env.log.debug('Promoting anonymous session %s to '
-                                       'authenticated session for user %s',
+                    self.env.log.debug("Promoting anonymous session %s to "
+                                       "authenticated session for user %s",
                                        sid, self.req.authname)
-                    cursor.execute("""
-                        UPDATE session SET sid=%s,authenticated=1
-                        WHERE sid=%s AND authenticated=0
-                        """, (self.req.authname, sid))
-                    cursor.execute("""
-                        UPDATE session_attribute SET sid=%s,authenticated=1
-                        WHERE sid=%s
-                        """, (self.req.authname, sid))
+                    db("""UPDATE session SET sid=%s, authenticated=1
+                          WHERE sid=%s AND authenticated=0
+                          """, (self.req.authname, sid))
+                    db("""UPDATE session_attribute SET sid=%s, authenticated=1
+                          WHERE sid=%s
+                          """, (self.req.authname, sid))
             else:
                 # We didn't have an anonymous session for this sid. The
                 # authenticated session might have been inserted between the
                 # SELECT above and here, so we catch the error.
                 try:
-                    cursor.execute("""
-                        INSERT INTO session (sid,last_visit,authenticated)
-                        VALUES (%s,%s,1)
-                        """, (self.req.authname, int(time.time())))
-                except Exception:
+                    db("""INSERT INTO session (sid, last_visit, authenticated)
+                          VALUES (%s, %s, 1)
+                          """, (self.req.authname, int(time.time())))
+                except self.env.db_exc.IntegrityError:
                     self.env.log.warning('Authenticated session for %s '
                                          'already exists', self.req.authname)
                     db.rollback()
@@ -344,7 +320,7 @@ class SessionAdmin(Component):
                sessions, and 'authenticated' all authenticated sessions.
                '*' lists all sessions, and is the default if no sids are
                given.
-               
+
                An sid suffix ':0' operates on an unauthenticated session with
                the given sid, and a suffix ':1' on an authenticated session
                (the default).""",
@@ -361,7 +337,7 @@ class SessionAdmin(Component):
 
         yield ('session set', '<name|email> <sid[:0|1]> <value>',
                """Set the name or email attribute of the given sid
-               
+
                An sid suffix ':0' operates on an unauthenticated session with
                the given sid, and a suffix ':1' on an authenticated session
                (the default).""",
@@ -392,19 +368,15 @@ class SessionAdmin(Component):
             return (sid, 1)
 
     def _get_sids(self):
-        db = self.env.get_db_cnx()
-        cursor = db.cursor()
-        cursor.execute("SELECT sid, authenticated FROM session")
-        return ['%s:%d' % (sid, auth) for sid, auth in cursor]
+        rows = self.env.db_query("SELECT sid, authenticated FROM session")
+        return ['%s:%d' % (sid, auth) for sid, auth in rows]
 
     def _get_list(self, sids):
         all_anon = 'anonymous' in sids or '*' in sids
         all_auth = 'authenticated' in sids or '*' in sids
         sids = set(self._split_sid(sid) for sid in sids
                    if sid not in ('anonymous', 'authenticated', '*'))
-        db = self.env.get_db_cnx()
-        cursor = db.cursor()
-        cursor.execute("""
+        rows = self.env.db_query("""
             SELECT DISTINCT s.sid, s.authenticated, s.last_visit,
                             n.value, e.value
             FROM session AS s
@@ -416,7 +388,7 @@ class SessionAdmin(Component):
                     AND e.name='email')
             ORDER BY s.sid, s.authenticated
             """)
-        for sid, authenticated, last_visit, name, email in cursor:
+        for sid, authenticated, last_visit, name, email in rows:
             if all_anon and not authenticated or all_auth and authenticated \
                     or (sid, authenticated) in sids:
                 yield (sid, authenticated, last_visit, name, email)
@@ -444,83 +416,67 @@ class SessionAdmin(Component):
                      for r in self._get_list(sids)],
                     [_('SID'), _('Auth'), _('Last Visit'), _('Name'),
                      _('Email')])
-        
+
     def _do_add(self, sid, name=None, email=None):
         sid, authenticated = self._split_sid(sid)
-        @self.env.with_transaction()
-        def add_session(db):
-            cursor = db.cursor()
+        with self.env.db_transaction as db:
             try:
-                cursor.execute("INSERT INTO session VALUES (%s, %s, %s)",
-                               (sid, authenticated, int(time.time())))
+                db("INSERT INTO session VALUES (%s, %s, %s)",
+                   (sid, authenticated, int(time.time())))
             except Exception:
                 raise AdminCommandError(_("Session '%(sid)s' already exists",
                                           sid=sid))
             if name is not None:
-                cursor.execute("""
-                    INSERT INTO session_attribute VALUES (%s, %s, 'name', %s)
-                    """, (sid, authenticated, name))
+                db("INSERT INTO session_attribute VALUES (%s,%s,'name',%s)",
+                    (sid, authenticated, name))
             if email is not None:
-                cursor.execute("""
-                    INSERT INTO session_attribute VALUES (%s, %s, 'email', %s)
-                    """, (sid, authenticated, email))
+                db("INSERT INTO session_attribute VALUES (%s,%s,'email',%s)",
+                    (sid, authenticated, email))
 
     def _do_set(self, attr, sid, val):
         if attr not in ('name', 'email'):
             raise AdminCommandError(_("Invalid attribute '%(attr)s'",
                                       attr=attr))
         sid, authenticated = self._split_sid(sid)
-        @self.env.with_transaction()
-        def set_attr(db):
-            cursor = db.cursor()
-            cursor.execute("""
-                SELECT sid FROM session WHERE sid=%s AND authenticated=%s
-                """, (sid, authenticated))
-            if not cursor.fetchone():
+        with self.env.db_transaction as db:
+            if not db("""SELECT sid FROM session
+                         WHERE sid=%s AND authenticated=%s""",
+                         (sid, authenticated)):
                 raise AdminCommandError(_("Session '%(sid)s' not found",
                                           sid=sid))
-            cursor.execute("""
+            db("""
                 DELETE FROM session_attribute
                 WHERE sid=%s AND authenticated=%s AND name=%s
                 """, (sid, authenticated, attr))
-            cursor.execute("""
-                INSERT INTO session_attribute VALUES (%s, %s, %s, %s)
-                """, (sid, authenticated, attr, val))
+            db("INSERT INTO session_attribute VALUES (%s, %s, %s, %s)",
+               (sid, authenticated, attr, val))
 
     def _do_delete(self, *sids):
-        @self.env.with_transaction()
-        def delete_session(db):
-            cursor = db.cursor()
+        with self.env.db_transaction as db:
             for sid in sids:
                 sid, authenticated = self._split_sid(sid)
                 if sid == 'anonymous':
-                    cursor.execute("""
-                        DELETE FROM session WHERE authenticated=0
-                        """)
-                    cursor.execute("""
-                        DELETE FROM session_attribute WHERE authenticated=0
-                        """)
+                    db("DELETE FROM session WHERE authenticated=0")
+                    db("DELETE FROM session_attribute WHERE authenticated=0")
                 else:
-                    cursor.execute("""
+                    db("""
                         DELETE FROM session
                         WHERE sid=%s AND authenticated=%s
                         """, (sid, authenticated))
-                    cursor.execute("""
+                    db("""
                         DELETE FROM session_attribute
                         WHERE sid=%s AND authenticated=%s
                         """, (sid, authenticated))
 
     def _do_purge(self, age):
         when = parse_date(age)
-        @self.env.with_transaction()
-        def purge_session(db):
-            cursor = db.cursor()
+        with self.env.db_transaction as db:
             ts = to_timestamp(when)
-            cursor.execute("""
+            db("""
                 DELETE FROM session
                 WHERE authenticated=0 AND last_visit<%s
                 """, (ts,))
-            cursor.execute("""
+            db("""
                 DELETE FROM session_attribute
                 WHERE authenticated=0
                       AND sid NOT IN (SELECT sid FROM session
