@@ -3,7 +3,7 @@
 # Copyright (C) 2003-2009 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2004-2005 Christopher Lenz <cmlenz@gmx.de>
-# Copyright (C) 2005-2007 Christian Boos <cboos@neuf.fr>
+# Copyright (C) 2005-2007 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -16,7 +16,7 @@
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 #         Christopher Lenz <cmlenz@gmx.de>
-#         Christian Boos <cboos@neuf.fr>
+#         Christian Boos <cboos@edgewall.org>
 
 import re
 import os
@@ -31,18 +31,18 @@ from genshi.util import plaintext
 from trac.core import *
 from trac.mimeview import *
 from trac.resource import get_relative_resource, get_resource_url
-from trac.wiki.api import WikiSystem, parse_args
-from trac.wiki.parser import WikiParser
 from trac.util import arity
-from trac.util.compat import all
 from trac.util.text import exception_to_unicode, shorten_line, to_unicode, \
-                           unicode_quote, unicode_quote_plus
+                           unicode_quote, unicode_quote_plus, unquote_label
 from trac.util.html import TracHTMLSanitizer
 from trac.util.translation import _
+from trac.wiki.api import WikiSystem, parse_args
+from trac.wiki.parser import WikiParser, parse_processor_args
 
 __all__ = ['wiki_to_html', 'wiki_to_oneliner', 'wiki_to_outline',
            'Formatter', 'format_to', 'format_to_html', 'format_to_oneliner',
-           'extract_link']
+           'extract_link', 'split_url_into_path_query_fragment',
+           'concat_path_query_fragment']
 
 
 def system_message(msg, text=None):
@@ -71,6 +71,31 @@ def split_url_into_path_query_fragment(target):
         target, query = target[:idx], target[idx:]
     return (target, query, fragment)
 
+def concat_path_query_fragment(path, query, fragment=None):
+    """Assemble `path`, `query` and `fragment` into a proper URL.
+
+    Can be used to re-assemble an URL decomposed using
+    `split_url_into_path_query_fragment` after modification.
+
+    >>> concat_path_query_fragment('/wiki/page', '?version=1')
+    '/wiki/page?version=1'
+    >>> concat_path_query_fragment('/wiki/page#a', '?version=1', '#b')
+    '/wiki/page?version=1#b'
+    >>> concat_path_query_fragment('/wiki/page?version=1#a', '?format=txt')
+    '/wiki/page?version=1&format=txt#a'
+    >>> concat_path_query_fragment('/wiki/page?version=1', '&format=txt')
+    '/wiki/page?version=1&format=txt'
+    >>> concat_path_query_fragment('/wiki/page?version=1', 'format=txt')
+    '/wiki/page?version=1&format=txt'
+    >>> concat_path_query_fragment('/wiki/page?version=1#a', '?format=txt', '#')
+    '/wiki/page?version=1&format=txt'
+    """
+    p, q, f = split_url_into_path_query_fragment(path)
+    if query:
+        q += ('&' if q else '?') + query.lstrip('?&')
+    if fragment:
+        f = fragment
+    return p + q + ('' if f == '#' else f)
 
 def _markup_to_unicode(markup):
     stream = None
@@ -95,9 +120,9 @@ class WikiProcessor(object):
 
     def __init__(self, formatter, name, args=None):
         """Find the processor by name
-        
-        :param formatter: the formatter embedding a call for this processor 
-        :param name: the name of the processor 
+
+        :param formatter: the formatter embedding a call for this processor
+        :param name: the name of the processor
         :param args: extra parameters for the processor
 
         (since 0.11)
@@ -109,11 +134,13 @@ class WikiProcessor(object):
         self.error = None
         self.macro_provider = None
 
+        # FIXME: move these tables outside of __init__
         builtin_processors = {'html': self._html_processor,
                               'htmlcomment': self._htmlcomment_processor,
                               'default': self._default_processor,
                               'comment': self._comment_processor,
                               'div': self._div_processor,
+                              'rtl': self._rtl_processor,
                               'span': self._span_processor,
                               'Span': self._span_processor,
                               'td': self._td_processor,
@@ -122,8 +149,13 @@ class WikiProcessor(object):
                               'table': self._table_processor,
                               }
 
+        self.inline_check = {'html': self._html_is_inline,
+                                'htmlcomment': True, 'comment': True,
+                                'span': True, 'Span': True,
+                                }.get(name)
+
         self._sanitizer = TracHTMLSanitizer(formatter.wiki.safe_schemes)
-        
+
         self.processor = builtin_processors.get(name)
         if not self.processor:
             # Find a matching wiki macro
@@ -135,6 +167,8 @@ class WikiProcessor(object):
                         else:
                             self.processor = self._legacy_macro_processor
                         self.macro_provider = macro_provider
+                        self.inline_check = getattr(macro_provider, 'is_inline',
+                                                    False)
                         break
         if not self.processor:
             # Find a matching mimeview renderer
@@ -153,6 +187,20 @@ class WikiProcessor(object):
             self.processor = self._default_processor
             self.error = "No macro or processor named '%s' found" % name
 
+    # inline checks
+
+    def _html_is_inline(self, text):
+        if text:
+            tag = text[1:].lstrip()
+            idx = tag.find(' ')
+            if idx > -1:
+                tag = tag[:idx]
+            return tag.lower() in ('a', 'span', 'bdo', 'img',
+                                   'big', 'small', 'font',
+                                   'tt', 'i', 'b', 'u', 's', 'strike',
+                                   'em', 'strong', 'dfn', 'code', 'q',
+                                   'samp', 'kbd', 'var', 'cite', 'abbr',
+                                   'acronym', 'sub', 'sup')
     # builtin processors
 
     def _comment_processor(self, text):
@@ -172,13 +220,13 @@ class WikiProcessor(object):
             line = unicode(text).splitlines()[e.lineno - 1].strip()
             return system_message(_('HTML parsing error: %(message)s',
                                     message=escape(e.msg)), line)
-        
+
     def _htmlcomment_processor(self, text):
         if "--" in text:
             return system_message(_('Error: Forbidden character sequence '
                                     '"--" in htmlcomment wiki code block'))
         return Markup('<!--\n%s-->\n' % text)
-        
+
     def _elt_processor(self, eltname, format_to, text):
         # Note: as long as _processor_param_re is not re.UNICODE, **args is OK.
         # Also, parse_args is using strict mode when processing [[span(...)]].
@@ -195,10 +243,15 @@ class WikiProcessor(object):
     def _div_processor(self, text):
         if not self.args:
             self.args = {}
-        if 'class' not in self.args:
-            self.args['class'] = 'wikipage'
+        self.args.setdefault('class', 'wikipage')
         return self._elt_processor('div', format_to_html, text)
-    
+
+    def _rtl_processor(self, text):
+        if not self.args:
+            self.args = {}
+        self.args['class'] = ('rtl ' + self.args.get('class', '')).rstrip()
+        return self._elt_processor('div', format_to_html, text)
+
     def _span_processor(self, text):
         if self.args is None:
             args, self.args = parse_args(text, strict=True)
@@ -207,10 +260,10 @@ class WikiProcessor(object):
 
     def _td_processor(self, text):
         return self._tablecell_processor('td', text)
-    
+
     def _th_processor(self, text):
         return self._tablecell_processor('th', text)
-    
+
     def _tr_processor(self, text):
         try:
             elt = self._elt_processor('tr', self._format_row, text)
@@ -218,24 +271,23 @@ class WikiProcessor(object):
             return elt
         except ProcessorError, e:
             return system_message(e)
-    
+
     def _table_processor(self, text):
         if not self.args:
             self.args = {}
-        if 'class' not in self.args:
-            self.args['class'] = 'wiki'
+        self.args.setdefault('class', 'wiki')
         try:
             return self._elt_processor('table', self._format_table, text)
         except ProcessorError, e:
             return system_message(e)
-    
+
     def _tablecell_processor(self, eltname, text):
         self.formatter.open_table_row()
         return self._elt_processor(eltname, format_to_html, text)
 
     _has_multiple_tables_re = re.compile(r"</table>.*?<table",
                                          re.MULTILINE | re.DOTALL)
-    
+
     _inner_table_re = re.compile(r"""\s*
       <table[^>]*>\s*
         ((?:<tr[^>]*>)?
@@ -243,7 +295,7 @@ class WikiProcessor(object):
         (?:</tr>)?)\s*
       </table>\s*$
       """, re.MULTILINE | re.DOTALL | re.VERBOSE)
-    
+
     # Note: the need for "parsing" that crude way the formatted content
     #       will go away as soon as we have a WikiDOM to manipulate...
 
@@ -256,7 +308,7 @@ class WikiProcessor(object):
             raise ProcessorError(_("!#%(name)s must contain at least one table"
                                    " cell (and table cells only)",
                                    name=self.name))
-        return Markup(match.group(self.name == 'table' and 1 or 2))
+        return Markup(match.group(1 if self.name == 'table' else 2))
 
     def _format_row(self, env, context, text):
         if text:
@@ -271,7 +323,7 @@ class WikiProcessor(object):
             Formatter(env, context).format(text, out)
             text = self._parse_inner_table(out.getvalue())
         return text
-    
+
     # generic processors
 
     def _legacy_macro_processor(self, text): # TODO: remove in 0.12
@@ -302,36 +354,42 @@ class WikiProcessor(object):
                                   self.error)
         else:
             text = self.processor(text)
-        if not text:
-            return ''
-        if in_paragraph:
-            content_for_span = None
-            interrupt_paragraph = False
-            if isinstance(text, Element):
-                tagname = text.tag.lower()
-                if tagname == 'div':
-                    class_ = text.attrib.get('class', '')
-                    if class_ and 'code' in class_:
-                        content_for_span = text.children
-                    else:
-                        interrupt_paragraph = True
-                elif tagname == 'table':
+        return text or ''
+
+    def is_inline(self, text):
+        if callable(self.inline_check):
+            return self.inline_check(text)
+        else:
+            return self.inline_check
+
+    def ensure_inline(self, text):
+        content_for_span = None
+        interrupt_paragraph = False
+        if isinstance(text, Element):
+            tagname = text.tag.lower()
+            if tagname == 'div':
+                class_ = text.attrib.get('class', '')
+                if class_ and 'code' in class_:
+                    content_for_span = text.children
+                else:
                     interrupt_paragraph = True
-            else:
-                # FIXME: do something smarter for Streams
-                text = _markup_to_unicode(text)
-                match = re.match(self._code_block_re, text)
-                if match:
-                    if match.group(1) and 'code' in match.group(1):
-                        content_for_span = match.group(2)
-                    else:
-                        interrupt_paragraph = True
-                elif re.match(self._block_elem_re, text):
+            elif tagname == 'table':
+                interrupt_paragraph = True
+        else:
+            # FIXME: do something smarter for Streams
+            text = _markup_to_unicode(text)
+            match = re.match(self._code_block_re, text)
+            if match:
+                if match.group(1) and 'code' in match.group(1):
+                    content_for_span = match.group(2)
+                else:
                     interrupt_paragraph = True
-            if content_for_span:
-                text = tag.span(class_='code-block')(*content_for_span)
-            elif interrupt_paragraph:
-                text = "</p>%s<p>" % _markup_to_unicode(text)
+            elif re.match(self._block_elem_re, text):
+                interrupt_paragraph = True
+        if content_for_span:
+            text = tag.span(class_='code-block')(*content_for_span)
+        elif interrupt_paragraph:
+            text = "</p>%s<p>" % _markup_to_unicode(text)
         return text
 
 
@@ -340,7 +398,7 @@ class Formatter(object):
 
     Parses and formats wiki text, in a given `Context`.
     """
-    
+
     flavor = 'default'
 
     # 0.10 compatibility
@@ -351,13 +409,12 @@ class Formatter(object):
     def __init__(self, env, context):
         """Note: `req` is still temporarily used."""
         self.env = env
-        self.context = context()
+        self.context = context.child()
         self.context.set_hints(disable_warnings=True)
         self.req = context.req
         self.href = context.href
         self.resource = context.resource
         self.perm = context.perm
-        self.db = self.env.get_db_cnx() # FIXME: remove
         self.wiki = WikiSystem(self.env)
         self.wikiparser = WikiParser(self.env)
         self._anchors = {}
@@ -365,7 +422,7 @@ class Formatter(object):
         self._safe_schemes = None
         if not self.wiki.render_unsafe_content:
             self._safe_schemes = set(self.wiki.safe_schemes)
-            
+
 
     def split_link(self, target):
         return split_url_into_path_query_fragment(target)
@@ -386,13 +443,13 @@ class Formatter(object):
     def _get_open_tag(self, tag):
         """Retrieve opening tag for direct or indirect `tag`."""
         if not isinstance(tag, tuple):
-            tag = self._indirect_tags[tag]            
+            tag = self._indirect_tags[tag]
         return tag[0]
 
     def _get_close_tag(self, tag):
         """Retrieve closing tag for direct or indirect `tag`."""
         if not isinstance(tag, tuple):
-            tag = self._indirect_tags[tag]            
+            tag = self._indirect_tags[tag]
         return tag[1]
 
     def tag_open_p(self, tag):
@@ -510,32 +567,26 @@ class Formatter(object):
     # HTML escape of &, < and >
 
     def _htmlescape_formatter(self, match, fullmatch):
-        return match == "&" and "&amp;" or match == "<" and "&lt;" or "&gt;"
+        return "&amp;" if match == "&" else "&lt;" if match == "<" else "&gt;"
 
     # Short form (shref) and long form (lhref) of TracLinks
 
-    def _unquote(self, text):
-        if text and text[0] in "'\"" and text[0] == text[-1]:
-            return text[1:-1]
-        else:
-            return text
-
     def _shrefbr_formatter(self, match, fullmatch):
         ns = fullmatch.group('snsbr')
-        target = self._unquote(fullmatch.group('stgtbr'))
+        target = unquote_label(fullmatch.group('stgtbr'))
         match = match[1:-1]
         return '&lt;%s&gt;' % \
                 self._make_link(ns, target, match, match, fullmatch)
 
     def _shref_formatter(self, match, fullmatch):
         ns = fullmatch.group('sns')
-        target = self._unquote(fullmatch.group('stgt'))
+        target = unquote_label(fullmatch.group('stgt'))
         return self._make_link(ns, target, match, match, fullmatch)
 
     def _lhref_formatter(self, match, fullmatch):
         rel = fullmatch.group('rel')
         ns = fullmatch.group('lns')
-        target = self._unquote(fullmatch.group('ltgt'))
+        target = unquote_label(fullmatch.group('ltgt'))
         label = fullmatch.group('label')
         return self._make_lhref_link(match, fullmatch, rel, ns, target, label)
 
@@ -546,10 +597,10 @@ class Formatter(object):
                     label = ns + ':' + target   #  use `http://target`
                 else:                           # for `wiki:target`
                     label = target.lstrip('/')  #  use only `target`
-            else: # e.g. `[search:]` 
+            else: # e.g. `[search:]`
                 label = ns
         else:
-            label = self._unquote(label)
+            label = unquote_label(label)
         if rel:
             if not label:
                 label = self.wiki.make_label_from_target(rel)
@@ -562,20 +613,14 @@ class Formatter(object):
                 resource = get_relative_resource(self.resource, path)
                 path = get_resource_url(self.env, resource, self.href)
                 if resource.id:
-                    idx = path.find('?')
-                    if idx >= 0:
-                        if query:
-                            query = path[idx:] + '&' + query.lstrip('?')
-                        else:
-                            query = path[idx:]
-                    target = unicode(resource.id) + query + fragment
+                    target = concat_path_query_fragment(unicode(resource.id),
+                                                        query, fragment)
                     if resource.realm == 'wiki':
                         target = '/' + target   # Avoid wiki page scoping
                     return self._make_link(resource.realm, target, match,
                                            label, fullmatch)
-                if '?' in path and query:
-                    query = '&' + query.lstrip('?')
-            return tag.a(label, href=path + query + fragment)
+            return tag.a(label,
+                         href=concat_path_query_fragment(path, query, fragment))
         else:
             return self._make_link(ns or 'wiki', target or '', match, label,
                                    fullmatch)
@@ -631,7 +676,7 @@ class Formatter(object):
                 sep = target.find(':')
                 if sep != -1:
                     url = '%s/%s/%s' % (url, target[:sep], target[sep + 1:])
-                else: 
+                else:
                     url = '%s/search?q=%s' % (url, unicode_quote_plus(target))
             else:
                 url = '%s/intertrac/%s' % (url, unicode_quote(target))
@@ -652,7 +697,7 @@ class Formatter(object):
                                                  target, label) or label
 
     def _make_interwiki_link(self, ns, target, label):
-        from trac.wiki.interwiki import InterWikiMap        
+        from trac.wiki.interwiki import InterWikiMap
         interwiki = InterWikiMap(self.env)
         if ns in interwiki:
             url, title = interwiki.url(ns, target)
@@ -672,7 +717,7 @@ class Formatter(object):
                      class_="mail-link", href=url, title=title or None)
 
     # Anchors
-    
+
     def _anchor_formatter(self, match, fullmatch):
         anchor = fullmatch.group('anchorname')
         label = fullmatch.group('anchorlabel') or ''
@@ -707,8 +752,8 @@ class Formatter(object):
                 return self._macro_formatter(match, fullmatch, macro)
         fullmatch = WikiParser._creolelink_re.match(macro_or_link)
         return self._lhref_formatter(match, fullmatch)
-    
-    def _macro_formatter(self, match, fullmatch, macro=None):
+
+    def _macro_formatter(self, match, fullmatch, macro, only_inline=False):
         name = fullmatch.group('macroname')
         if name.lower() == 'br':
             return '<br />'
@@ -717,9 +762,9 @@ class Formatter(object):
         else:
             args = fullmatch.group('macroargs')
         try:
-            return macro.process(args, in_paragraph=True)
+            return macro.ensure_inline(macro.process(args))
         except Exception, e:
-            self.env.log.error('Macro %s(%s) failed: %s' % 
+            self.env.log.error('Macro %s(%s) failed: %s' %
                     (name, args, exception_to_unicode(e, traceback=True)))
             return system_message('Error: Macro %s(%s) failed' % (name, args),
                                   e)
@@ -782,13 +827,13 @@ class Formatter(object):
         self._tabstops = tabstops
 
     # Lists
-    
+
     def _list_formatter(self, match, fullmatch):
         ldepth = len(fullmatch.group('ldepth'))
         listid = match[ldepth]
         self.in_list_item = True
         class_ = start = None
-        if listid in '-*':
+        if listid in WikiParser.BULLET_CHARS:
             type_ = 'ul'
         else:
             type_ = 'ol'
@@ -809,7 +854,7 @@ class Formatter(object):
                     start = ord(lstart) - ord('A') + 1
         self._set_list_depth(ldepth, type_, class_, start)
         return ''
-        
+
     def _get_list_depth(self):
         """Return the space offset associated to the deepest opened list."""
         if self._list_stack:
@@ -823,9 +868,9 @@ class Formatter(object):
             self.close_indentation() # FIXME: why not lists in quotes?
             self._list_stack.append((new_type, depth))
             self._set_tab(depth)
-            class_attr = (lclass and ' class="%s"' % lclass) or ''
-            start_attr = (start and ' start="%s"' % start) or ''
-            self.out.write('<'+new_type+class_attr+start_attr+'><li>')
+            class_attr = ' class="%s"' % lclass if lclass else ''
+            start_attr = ' start="%s"' % start if start else ''
+            self.out.write('<' + new_type + class_attr + start_attr + '><li>')
         def close_item():
             self.flush_tags()
             self.out.write('</li>')
@@ -863,7 +908,7 @@ class Formatter(object):
     # Definition Lists
 
     def _definition_formatter(self, match, fullmatch):
-        tmp = self.in_def_list and '</dd>' or '<dl class="wiki">'
+        tmp = '</dd>' if self.in_def_list else '<dl class="wiki">'
         definition = match[:match.find('::')]
         tmp += '<dt>%s</dt><dd>' % format_to_oneliner(self.env, self.context,
                                                       definition)
@@ -887,7 +932,7 @@ class Formatter(object):
                         self.in_list_item = True
                         self._set_list_depth(idepth)
                         return ''
-            elif idepth <= ldepth + (ltype == 'ol' and 3 or 2):
+            elif idepth <= ldepth + (3 if ltype == 'ol' else 2):
                 self.in_list_item = True
                 return ''
         if not self.in_def_list:
@@ -899,7 +944,7 @@ class Formatter(object):
 
     def _get_quote_depth(self):
         """Return the space offset associated to the deepest opened quote."""
-        return self._quote_stack and self._quote_stack[-1] or 0
+        return self._quote_stack[-1] if self._quote_stack else 0
 
     def _set_quote_depth(self, depth, citation=False):
         def open_quote(depth):
@@ -909,7 +954,7 @@ class Formatter(object):
             def open_one_quote(d):
                 self._quote_stack.append(d)
                 self._set_tab(d)
-                class_attr = citation and ' class="citation"' or ''
+                class_attr = ' class="citation"' if citation else ''
                 self.out.write('<blockquote%s>' % class_attr + os.linesep)
             if citation:
                 for d in range(quote_depth+1, depth+1):
@@ -946,7 +991,7 @@ class Formatter(object):
             self.in_quote = True
 
     # Table
-    
+
     def _table_cell_formatter(self, match, fullmatch):
         self.open_table()
         self.open_table_row()
@@ -987,7 +1032,7 @@ class Formatter(object):
             last_extra = after_sep + next_sep.start() - 1
             if first_extra < last_extra and \
                    self.line[first_extra] == self.line[last_extra] == ' ':
-                textalign = 'center'                
+                textalign = 'center'
         if textalign:
             attrs += ' style="text-align: %s"' % textalign
         td = '<%s%s>' % (cell, attrs)
@@ -1053,12 +1098,7 @@ class Formatter(object):
     # Code blocks
 
     def parse_processor_args(self, line):
-        args = WikiParser._processor_param_re.split(line)
-        del args[::3]
-        keys = [str(k) for k in args[::2]] # used as keyword parameters
-        values = [(v and v[0] in '"\'' and [v[1:-1]] or [v])[0]
-                  for v in args[1::2]]
-        return dict(zip(keys, values))
+        return parse_processor_args(line)
 
     def handle_code_block(self, line, startmatch=None):
         if startmatch:
@@ -1066,7 +1106,7 @@ class Formatter(object):
             if self.in_code_block == 1:
                 name = startmatch.group(2)
                 if name:
-                    args = self.parse_processor_args(line[startmatch.end():])
+                    args = parse_processor_args(line[startmatch.end():])
                     self.code_processor = WikiProcessor(self, name, args)
                 else:
                     self.code_processor = None
@@ -1100,7 +1140,7 @@ class Formatter(object):
             if match:
                 self.code_prefix = match.group(1)
                 name = match.group(2)
-                args = self.parse_processor_args(line[match.end():])
+                args = parse_processor_args(line[match.end():])
                 self.code_processor = WikiProcessor(self, name, args)
             else:
                 self.code_buf.append(line)
@@ -1127,11 +1167,11 @@ class Formatter(object):
         if not self._quote_buffer and depth < self._get_list_depth():
             self.close_list(depth)
         self._quote_buffer.append(line[depth + 1:])
-        
+
     def close_quote_block(self, escape_newlines):
         if self._quote_buffer:
             # avoid an extra <blockquote> when there's consistently one space
-            # after the '>' 
+            # after the '>'
             if all(not line or line[0] in '> ' for line in self._quote_buffer):
                 self._quote_buffer = [line[bool(line and line[0] == ' '):]
                                       for line in self._quote_buffer]
@@ -1142,7 +1182,7 @@ class Formatter(object):
             self._quote_buffer = []
 
     # -- Wiki engine
-    
+
     def handle_match(self, fullmatch):
         for itype, match in fullmatch.groupdict().items():
             if match and not itype in self.wikiparser.helper_patterns:
@@ -1187,13 +1227,13 @@ class Formatter(object):
         self.in_table_cell = ''
         self.paragraph_open = 0
         return source
-        
+
 
     def format(self, text, out=None, escape_newlines=False):
         text = self.reset(text, out)
         if isinstance(text, basestring):
             text = text.splitlines()
-            
+
         for line in text:
             # Detect start of code block (new block or embedded block)
             block_start_match = None
@@ -1301,15 +1341,15 @@ class OneLinerFormatter(Formatter):
     def _linebreak_wc_formatter(self, match, fullmatch):
         return ' '
 
-    def _macro_formatter(self, match, fullmatch, macro=None):
+    def _macro_formatter(self, match, fullmatch, macro):
         name = fullmatch.group('macroname')
         if name.lower() == 'br':
             return ' '
-        elif name == 'comment':
-            return ''
+        args = fullmatch.group('macroargs')
+        if macro.is_inline(args):
+            return Formatter._macro_formatter(self, match, fullmatch, macro)
         else:
-            args = fullmatch.group('macroargs')
-            return '[[%s%s]]' % (name,  args and '(...)' or '')
+            return '[[%s%s]]' % (name, '(...)' if args else '')
 
     def format(self, text, out, shorten=False):
         if not text:
@@ -1358,10 +1398,15 @@ class OneLinerFormatter(Formatter):
 class OutlineFormatter(Formatter):
     """Special formatter that generates an outline of all the headings."""
     flavor = 'outline'
-    
-    # Avoid the possible side-effects of rendering WikiProcessors
 
-    def _macro_formatter(self, match, fullmatch, macro=None):
+    # Avoid the possible side-effects of rendering WikiProcessors
+    def _macro_formatter(self, match, fullmatch, macro):
+        name = fullmatch.group('macroname')
+        if name.lower() == 'br':
+            return ' '
+        args = fullmatch.group('macroargs')
+        if macro.is_inline(args):
+            return Formatter._macro_formatter(self, match, fullmatch, macro)
         return ''
 
     def handle_code_block(self, line, startmatch=None):
@@ -1418,7 +1463,7 @@ class OutlineFormatter(Formatter):
 class LinkFormatter(OutlineFormatter):
     """Special formatter that focuses on TracLinks."""
     flavor = 'link'
-    
+
     def _heading_formatter(self, match, fullmatch):
         return ''
 
@@ -1437,7 +1482,7 @@ class HtmlFormatter(object):
     """Format parsed wiki text to HTML"""
 
     flavor = 'default'
-    
+
     def __init__(self, env, context, wikidom):
         self.env = env
         self.context = context
@@ -1462,7 +1507,7 @@ class InlineHtmlFormatter(object):
 
     Block level content will be disguarded or compacted.
     """
-    
+
     flavor = 'oneliner'
 
     def __init__(self, env, context, wikidom):
@@ -1517,33 +1562,36 @@ def extract_link(env, context, wikidom):
 
 def wiki_to_html(wikitext, env, req, db=None,
                  absurls=False, escape_newlines=False):
-    """deprecated in favor of format_to_html (will be removed in 0.13)"""
+    """deprecated in favor of format_to_html (will be removed in 1.0)"""
     if not wikitext:
         return Markup()
     abs_ref, href = (req or env).abs_href, (req or env).href
-    context = Context.from_request(req, absurls=absurls)
+    from trac.web.chrome import web_context
+    context = web_context(req, absurls=absurls)
     out = StringIO()
     Formatter(env, context).format(wikitext, out, escape_newlines)
     return Markup(out.getvalue())
 
 def wiki_to_oneliner(wikitext, env, db=None, shorten=False, absurls=False,
                      req=None):
-    """deprecated in favor of format_to_oneliner (will be removed in 0.13)"""
+    """:deprecated: in favor of format_to_oneliner (will be removed in 1.0)"""
     if not wikitext:
         return Markup()
     abs_ref, href = (req or env).abs_href, (req or env).href
-    context = Context.from_request(req, absurls=absurls)
+    from trac.web.chrome import web_context
+    context = web_context(req, absurls=absurls)
     out = StringIO()
     OneLinerFormatter(env, context).format(wikitext, out, shorten)
     return Markup(out.getvalue())
 
 def wiki_to_outline(wikitext, env, db=None,
                     absurls=False, max_depth=None, min_depth=None, req=None):
-    """deprecated (will be removed in 0.13 and replaced by something else)"""
+    """:deprecated: will be removed in 1.0 and replaced by something else"""
     if not wikitext:
         return Markup()
     abs_ref, href = (req or env).abs_href, (req or env).href
-    context = Context.from_request(req, absurls=absurls)
+    from trac.web.chrome import web_context
+    context = web_context(req, absurls=absurls)
     out = StringIO()
     OutlineFormatter(env, context).format(wikitext, out, max_depth, min_depth)
     return Markup(out.getvalue())
