@@ -13,6 +13,8 @@ wiki:TestPage
 wiki:TestPage/
 wiki:/TestPage
 [wiki:/TestPage]
+[wiki:/TestPage ]
+[wiki:/TestPage\u200B]
 [wiki:/TestPage /TestPage]
 wiki:"Space 1 23"
 wiki:"C'est l'\xe9t\xe9"
@@ -25,6 +27,8 @@ wiki:abc
 <a class="wiki" href="/wiki/TestPage">wiki:TestPage/</a>
 <a class="wiki" href="/wiki/TestPage">wiki:/TestPage</a>
 <a class="wiki" href="/wiki/TestPage">TestPage</a>
+<a class="wiki" href="/wiki/TestPage">TestPage</a>
+<a class="wiki" href="/wiki/TestPage">TestPage</a>
 <a class="wiki" href="/wiki/TestPage">/TestPage</a>
 <a class="wiki" href="/wiki/Space%201%2023">wiki:"Space 1 23"</a>
 <a class="wiki" href="/wiki/C'est%20l'%C3%A9t%C3%A9">wiki:"C'est l'\xe9t\xe9"</a>
@@ -220,10 +224,14 @@ This is a <a class="missing wiki" href="/wiki/Wiki?version=12" rel="nofollow">Wi
 ============================== WikiPageName with label
 See details of the [WikiPageNames wiki page name] syntax.
 Here's a [BadExample\fbad] example with special whitespace.
+We can also [WikiLabels '"use [quotes]"']
+or [WikiLabels "'use [quotes]'"]
 ------------------------------
 <p>
 See details of the <a class="missing wiki" href="/wiki/WikiPageNames" rel="nofollow">wiki page name?</a> syntax.
 Here's a <a class="missing wiki" href="/wiki/BadExample" rel="nofollow">bad?</a> example with special whitespace.
+We can also <a class="missing wiki" href="/wiki/WikiLabels" rel="nofollow">"use [quotes]"?</a>
+or <a class="missing wiki" href="/wiki/WikiLabels" rel="nofollow">'use [quotes]'?</a>
 </p>
 ------------------------------
 ============================== WikiPageName with label should be strict...
@@ -271,6 +279,9 @@ complex link complex:a:test with positional arguments
 complex link complex:a (not enough arguments)
 complex link complex:a:test:more (too many arguments)
 
+in trac.ini inter:b:resource
+in trac.ini over:c:something overrides wiki
+
 NoLink:ignored
 NoLink:
 NoLink: ...
@@ -285,6 +296,10 @@ complex link <a class="ext-link" href="http://server/a/page/?format=txt" title="
 complex link <a class="ext-link" href="http://server/a/page/test:more?format=txt" title="resource test:more in a"><span class="icon"></span>complex:a:test:more</a> (too many arguments)
 </p>
 <p>
+in trac.ini <a class="ext-link" href="http://inter/b/page/resource" title="Resource resource in b"><span class="icon"></span>inter:b:resource</a>
+in trac.ini <a class="ext-link" href="http://over/c/page/something" title="c:something in over"><span class="icon"></span>over:c:something</a> overrides wiki
+</p>
+<p>
 NoLink:ignored
 <a class="missing wiki" href="/wiki/NoLink" rel="nofollow">NoLink?</a>:
 <a class="missing wiki" href="/wiki/NoLink" rel="nofollow">NoLink?</a>: ...
@@ -663,29 +678,34 @@ This is the InterMapTxt
 MeatBall 	http://www.usemod.com/cgi-bin/mb.pl? # $1 in MeatBall...
 tsvn            tsvn:
 complex         http://server/$1/page/$2?format=txt  # resource $2 in $1
+over        http://unused/? # Overridden in trac.ini
 }}}
 ----
 {{{
 nolink          http://noweb
 }}}
-""" 
+"""
     imt.save('joe', 'test InterWiki links', '::1', now)
+    tc.env.config.set('interwiki', 'inter',
+                      'http://inter/$1/page/$2 Resource $2 in $1')
+    tc.env.config.set('interwiki', 'over',
+                      'http://over/$1/page/$2')
 
     w = WikiPage(tc.env)
     w.name = 'FirstLevel'
     w.text = '--'
     w.save('joe', 'first level of hierarchy', '::1', now)
-    
+
     w = WikiPage(tc.env)
     w.name = 'FirstLevel/SecondLevel'
     w.text = '--'
     w.save('joe', 'second level of hierarchy', '::1', now)
-    
+
     w = WikiPage(tc.env)
     w.name = 'FirstLevel/SecondLevel/ThirdLevel'
     w.text = '--'
     w.save('joe', 'third level of hierarchy', '::1', now)
-    
+
     w = WikiPage(tc.env)
     w.name = 'FirstLevel/SecondLevel/OtherThirdLevel'
     w.text = '--'
@@ -713,9 +733,9 @@ def suite():
                                   context=('wiki', 'Main/Sub')))
     suite.addTest(formatter.suite(SCOPED_LINKS_TESTS, wiki_setup, __file__,
                                   wiki_teardown,
-                                  context=('wiki', 
+                                  context=('wiki',
                                       'FirstLevel/SecondLevel/ThirdLevel')))
     return suite
 
 if __name__ == '__main__':
-    unittest.main(defaultTest='suite') 
+    unittest.main(defaultTest='suite')
