@@ -16,6 +16,8 @@
 # Author: Jonas Borgström <jonas@edgewall.com>
 #         Christopher Lenz <cmlenz@gmx.de>
 
+from __future__ import with_statement
+
 import pkg_resources
 import re
 
@@ -24,7 +26,7 @@ from genshi.builder import tag
 from trac.attachment import AttachmentModule
 from trac.config import IntOption
 from trac.core import *
-from trac.mimeview.api import Mimeview, IContentConverter, Context
+from trac.mimeview.api import IContentConverter, Mimeview
 from trac.perm import IPermissionRequestor
 from trac.resource import *
 from trac.search import ISearchSource, search_to_sql, shorten_result
@@ -34,20 +36,22 @@ from trac.util.datefmt import from_utimestamp, to_utimestamp
 from trac.util.text import shorten_line
 from trac.util.translation import _, tag_
 from trac.versioncontrol.diff import get_diff_options, diff_blocks
-from trac.web.chrome import add_ctxtnav, add_link, add_notice, add_script, \
-                            add_stylesheet, add_warning, prevnext_nav, \
-                            Chrome, INavigationContributor, ITemplateProvider
 from trac.web.api import IRequestHandler
+from trac.web.chrome import (Chrome, INavigationContributor, ITemplateProvider,
+                             add_ctxtnav, add_link, add_notice, add_script,
+                             add_stylesheet, add_warning, prevnext_nav,
+                             web_context)
 from trac.wiki.api import IWikiPageManipulator, WikiSystem, validate_page_name
 from trac.wiki.formatter import format_to, OneLinerFormatter
 from trac.wiki.model import WikiPage
- 
+
+
 class InvalidWikiPage(TracError):
     """Exception raised when a Wiki page fails validation.
-    
+
     :deprecated: Not used anymore since 0.11
     """
- 
+
 
 class WikiModule(Component):
 
@@ -64,6 +68,7 @@ class WikiModule(Component):
     DEFAULT_PAGE_TEMPLATE = 'DefaultPage'
 
     # IContentConverter methods
+
     def get_supported_conversions(self):
         yield ('txt', _('Plain Text'), 'txt', 'text/x-trac-wiki', 'text/plain',
                9)
@@ -131,7 +136,7 @@ class WikiModule(Component):
             if action == 'edit':
                 if 'cancel' in req.args:
                     req.redirect(req.href.wiki(page.name))
-                
+
                 has_collision = int(version) != page.version
                 for a in ('preview', 'diff', 'merge'):
                     if a in req.args:
@@ -155,11 +160,11 @@ class WikiModule(Component):
                                            version=version,
                                            contextall=contextall or None))
         elif action == 'delete':
-            return self._render_confirm_delete(req, versioned_page)
+            return self._render_confirm_delete(req, page)
         elif action == 'rename':
             return self._render_confirm_rename(req, page)
         elif action == 'edit':
-            return self._render_editor(req, versioned_page)
+            return self._render_editor(req, page)
         elif action == 'diff':
             return self._render_diff(req, versioned_page)
         elif action == 'history':
@@ -184,7 +189,7 @@ class WikiModule(Component):
 
     def _validate(self, req, page):
         valid = True
-        
+
         # Validate page size
         if len(req.args.get('text', '')) > self.max_size:
             add_warning(req, _('The wiki page is too long (must be less '
@@ -228,9 +233,9 @@ class WikiModule(Component):
         def version_info(v, last=0):
             return {'path': get_resource_name(self.env, page.resource),
                     # TRANSLATOR: wiki page
-                    'rev': v or _('currently edited'), 
+                    'rev': v or _('currently edited'),
                     'shortrev': v or last + 1,
-                    'href': v and req.href.wiki(page.name, version=v) or None}
+                    'href': req.href.wiki(page.name, version=v) if v else None}
         changes = [{'diffs': diffs, 'props': [],
                     'new': version_info(new_version, old_version),
                     'old': version_info(old_version)}]
@@ -251,8 +256,7 @@ class WikiModule(Component):
         version = int(req.args.get('version', 0)) or None
         old_version = int(req.args.get('old_version', 0)) or version
 
-        @self.env.with_transaction()
-        def do_delete(db):
+        with self.env.db_transaction as db:
             if version and old_version and version > old_version:
                 # delete from `old_version` exclusive to `version` inclusive:
                 for v in range(old_version, version):
@@ -262,7 +266,7 @@ class WikiModule(Component):
                 page.delete(version, db)
 
         if not page.exists:
-            add_notice(req, _('The page %(name)s has been deleted.',
+            add_notice(req, _("The page %(name)s has been deleted.",
                               name=page.name))
             req.redirect(req.href.wiki())
         else:
@@ -281,32 +285,31 @@ class WikiModule(Component):
             req.perm(page.resource).require('WIKI_ADMIN')
         else:
             req.perm(page.resource).require('WIKI_RENAME')
- 	 
+
         if 'cancel' in req.args:
             req.redirect(get_resource_url(self.env, page.resource, req.href))
- 	 
+
         old_name, old_version = page.name, page.version
         new_name = req.args.get('new_name', '')
         new_name = re.sub(r'/{2,}', '/', new_name.strip('/'))
         redirect = req.args.get('redirect')
- 	 
+
         # verify input parameters
         warn = None
         if not new_name:
-            warn = _('A new name is mandatory for a rename.')
+            warn = _("A new name is mandatory for a rename.")
         elif not validate_page_name(new_name):
             warn = _("The new name is invalid (a name which is separated "
                      "with slashes cannot be '.' or '..').")
         elif new_name == old_name:
-            warn = _('The new name must be different from the old name.')
+            warn = _("The new name must be different from the old name.")
         elif WikiPage(self.env, new_name).exists:
-            warn = _('The page %(name)s already exists.', name=new_name)
+            warn = _("The page %(name)s already exists.", name=new_name)
         if warn:
             add_warning(req, warn)
             return self._render_confirm_rename(req, page, new_name)
 
-        @self.env.with_transaction()
-        def do_rename(db):
+        with self.env.db_transaction as db:
             page.rename(new_name)
             if redirect:
                 redirection = WikiPage(self.env, old_name, db=db)
@@ -315,8 +318,8 @@ class WikiModule(Component):
                 comment = u'[wiki:"%s@%d" %s] \u2192 [wiki:"%s"].' % (
                           new_name, old_version, old_name, new_name)
                 redirection.save(author, comment, req.remote_addr)
-        
-        req.redirect(req.href.wiki(redirect and old_name or new_name))
+
+        req.redirect(req.href.wiki(old_name if redirect else new_name))
 
     def _do_save(self, req, page):
         if page.readonly:
@@ -334,8 +337,8 @@ class WikiModule(Component):
         try:
             page.save(get_reporter_id(req, 'author'), req.args.get('comment'),
                       req.remote_addr)
-            add_notice(req, _('Your changes have been saved in version '
-                              '%(version)s.', version=page.version))
+            add_notice(req, _("Your changes have been saved in version "
+                              "%(version)s.", version=page.version))
             req.redirect(get_resource_url(self.env, page.resource, req.href,
                                           version=None))
         except TracError:
@@ -353,17 +356,28 @@ class WikiModule(Component):
             version = int(req.args.get('version', 0))
         old_version = int(req.args.get('old_version') or 0) or version
 
+        what = 'multiple' if version and old_version \
+                             and version - old_version > 1 \
+               else 'single' if version else 'page'
+
+        num_versions = 0
+        new_date = None
+        old_date = None
+        for v, t, author, comment, ipnr in page.get_history():
+            if (v <= version or what == 'page') and new_date is None:
+                new_date = t
+            if (v <= old_version and what == 'multiple' or
+                num_versions > 1 and what == 'single'):
+                break
+            num_versions += 1
+            old_date = t
+
         data = self._page_data(req, page, 'delete')
-        data.update({'new_version': None, 'old_version': None,
-                     'num_versions': 0})
+        data.update({'what': what, 'new_version': None, 'old_version': None,
+                     'num_versions': num_versions, 'new_date': new_date,
+                     'old_date': old_date})
         if version is not None:
-            num_versions = 0
-            for v, t, author, comment, ipnr in page.get_history():
-                num_versions += 1
-                if num_versions > 1:
-                    break
-            data.update({'new_version': version, 'old_version': old_version,
-                         'num_versions': num_versions})
+            data.update({'new_version': version, 'old_version': old_version})
         self._wiki_ctxtnav(req, page)
         return 'wiki_delete.html', data, None
 
@@ -372,12 +386,12 @@ class WikiModule(Component):
             req.perm(page.resource).require('WIKI_ADMIN')
         else:
             req.perm(page.resource).require('WIKI_RENAME')
-           
+
         data = self._page_data(req, page, 'rename')
-        data['new_name'] = new_name is None and page.name or new_name
+        data['new_name'] = new_name if new_name is not None else page.name
         self._wiki_ctxtnav(req, page)
         return 'wiki_rename.html', data, None
-        
+
     def _render_diff(self, req, page):
         if not page.exists:
             raise TracError(_('Version %(num)s of page "%(name)s" does not '
@@ -441,7 +455,7 @@ class WikiModule(Component):
                      _('Version %(num)s', num=next_version))
 
         data = self._page_data(req, page, 'diff')
-        data.update({ 
+        data.update({
             'change': {'date': date, 'author': author, 'ipnr': ipnr,
                        'comment': comment},
             'new_version': new_version, 'old_version': old_version,
@@ -451,7 +465,7 @@ class WikiModule(Component):
             'changes': changes,
             'diff': diff_data,
         })
-        prevnext_nav(req, _('Previous Change'), _('Next Change'), 
+        prevnext_nav(req, _('Previous Change'), _('Next Change'),
                      _('Wiki History'))
         return 'wiki_diff.html', data, None
 
@@ -468,6 +482,7 @@ class WikiModule(Component):
         else:
             req.perm(page.resource).require('WIKI_MODIFY')
         original_text = page.text
+        comment = req.args.get('comment', '')
         if 'text' in req.args:
             page.text = req.args.get('text')
         elif 'template' in req.args:
@@ -476,12 +491,17 @@ class WikiModule(Component):
             if template_page and template_page.exists and \
                    'WIKI_VIEW' in req.perm(template_page.resource):
                 page.text = template_page.text
+        elif 'version' in req.args:
+            old_page = WikiPage(self.env, page.name,
+                                version=int(req.args['version']))
+            req.perm(page.resource).require('WIKI_VIEW')
+            page.text = old_page.text
+            comment = _("Reverted to version %(version)s.",
+                        version=req.args['version'])
         if action in ('preview', 'diff'):
             page.readonly = 'readonly' in req.args
 
         author = get_reporter_id(req, 'author')
-        comment = req.args.get('comment', '')
-
         defaults = {'editrows': 20}
         prefs = dict((key, req.session.get('wiki_%s' % key, defaults.get(key)))
                      for key in ('editrows', 'sidebyside'))
@@ -494,7 +514,7 @@ class WikiModule(Component):
             sidebyside = prefs['sidebyside']
 
         if sidebyside:
-            editrows = max(int(prefs['editrows']), 
+            editrows = max(int(prefs['editrows']),
                            len(page.text.splitlines()) + 1)
         else:
             editrows = req.args.get('editrows')
@@ -506,16 +526,18 @@ class WikiModule(Component):
                 editrows = prefs['editrows']
 
         data = self._page_data(req, page, action)
+        context = web_context(req, page.resource)
         data.update({
             'author': author,
             'comment': comment,
             'edit_rows': editrows, 'sidebyside': sidebyside,
             'scroll_bar_pos': req.args.get('scroll_bar_pos', ''),
             'diff': None,
+            'attachments': AttachmentModule(self.env).attachment_data(context),
         })
         if action in ('diff', 'merge'):
-            old_text = original_text and original_text.splitlines() or []
-            new_text = page.text and page.text.splitlines() or []
+            old_text = original_text.splitlines() if original_text else []
+            new_text = page.text.splitlines() if page.text else []
             diff_data, changes = self._prepare_diff(
                 req, page, old_text, new_text, page.version, '')
             data.update({'diff': diff_data, 'changes': changes,
@@ -523,10 +545,11 @@ class WikiModule(Component):
                          'longcol': 'Version', 'shortcol': 'v'})
         elif sidebyside and action != 'collision':
             data['action'] = 'preview'
-        
+
         self._wiki_ctxtnav(req, page)
         Chrome(self.env).add_wiki_toolbars(req)
         Chrome(self.env).add_auto_preview(req)
+        add_script(req, 'common/js/folding.js')
         return 'wiki_edit.html', data, None
 
     def _render_history(self, req, page):
@@ -574,7 +597,7 @@ class WikiModule(Component):
             data['title'] = ''
 
         ws = WikiSystem(self.env)
-        context = Context.from_request(req, page.resource)
+        context = web_context(req, page.resource)
         higher, related = [], []
         if not page.exists:
             if 'WIKI_CREATE' not in req.perm(page.resource):
@@ -617,7 +640,7 @@ class WikiModule(Component):
                             next_version = v
             except ValueError:
                 version = None
-            
+
         prefix = self.PAGE_TEMPLATES_PREFIX
         templates = [template[len(prefix):]
                      for template in ws.get_pages(prefix)
@@ -637,7 +660,7 @@ class WikiModule(Component):
             parent = page.name[:page.name.rindex('/')]
             add_link(req, 'up', req.href.wiki(parent, version=None),
                      _("View parent page"))
-        
+
         if next_version:
             add_link(req, 'next',
                      req.href.wiki(page.name, version=next_version),
@@ -671,13 +694,13 @@ class WikiModule(Component):
         })
         add_script(req, 'common/js/folding.js')
         return 'wiki_view.html', data, None
-    
+
     def _wiki_ctxtnav(self, req, page):
         """Add the normal wiki ctxtnav entries."""
         add_ctxtnav(req, _('Start Page'), req.href.wiki('WikiStart'))
         add_ctxtnav(req, _('Index'), req.href.wiki('TitleIndex'))
         if page.exists:
-            add_ctxtnav(req, _('History'), req.href.wiki(page.name, 
+            add_ctxtnav(req, _('History'), req.href.wiki(page.name,
                                                          action='history'))
 
     # ITimelineEventProvider methods
@@ -687,14 +710,12 @@ class WikiModule(Component):
             yield ('wiki', _('Wiki changes'))
 
     def get_timeline_events(self, req, start, stop, filters):
-        db = self.env.get_db_cnx()
         if 'wiki' in filters:
             wiki_realm = Resource('wiki')
-            cursor = db.cursor()
-            cursor.execute("SELECT time,name,comment,author,version "
-                           "FROM wiki WHERE time>=%s AND time<=%s",
-                           (to_utimestamp(start), to_utimestamp(stop)))
-            for ts, name, comment, author, version in cursor:
+            for ts, name, comment, author, version in self.env.db_query("""
+                    SELECT time, name, comment, author, version FROM wiki
+                    WHERE time>=%s AND time<=%s
+                    """, (to_utimestamp(start), to_utimestamp(stop))):
                 wiki_page = wiki_realm(id=name, version=version)
                 if 'WIKI_VIEW' not in req.perm(wiki_page):
                     continue
@@ -717,8 +738,8 @@ class WikiModule(Component):
             else:
                 return tag_('%(page)s created', page=name)
         elif field == 'description':
-            markup = format_to(self.env, None, context(resource=wiki_page),
-                               comment)
+            markup = format_to(self.env, None,
+                               context.child(resource=wiki_page), comment)
             if wiki_page.version > 1:
                 diff_href = context.href.wiki(
                     wiki_page.id, version=wiki_page.version, action='diff')
@@ -735,26 +756,23 @@ class WikiModule(Component):
     def get_search_results(self, req, terms, filters):
         if not 'wiki' in filters:
             return
-        db = self.env.get_db_cnx()
-        sql_query, args = search_to_sql(db, ['w1.name', 'w1.author', 'w1.text'],
-                                        terms)
-        cursor = db.cursor()
-        cursor.execute("SELECT w1.name,w1.time,w1.author,w1.text "
-                       "FROM wiki w1,"
-                       "(SELECT name,max(version) AS ver "
-                       "FROM wiki GROUP BY name) w2 "
-                       "WHERE w1.version = w2.ver AND w1.name = w2.name "
-                       "AND " + sql_query, args)
+        with self.env.db_query as db:
+            sql_query, args = search_to_sql(db, ['w1.name', 'w1.author',
+                                                 'w1.text'], terms)
+            wiki_realm = Resource('wiki')
+            for name, ts, author, text in db("""
+                    SELECT w1.name, w1.time, w1.author, w1.text
+                    FROM wiki w1,(SELECT name, max(version) AS ver
+                                  FROM wiki GROUP BY name) w2
+                    WHERE w1.version = w2.ver AND w1.name = w2.name
+                    AND """ + sql_query, args):
+                page = wiki_realm(id=name)
+                if 'WIKI_VIEW' in req.perm(page):
+                    yield (get_resource_url(self.env, page, req.href),
+                           '%s: %s' % (name, shorten_line(text)),
+                           from_utimestamp(ts), author,
+                           shorten_result(text, terms))
 
-        wiki_realm = Resource('wiki')
-        for name, ts, author, text in cursor:
-            page = wiki_realm(id=name)
-            if 'WIKI_VIEW' in req.perm(page):
-                yield (get_resource_url(self.env, page, req.href),
-                       '%s: %s' % (name, shorten_line(text)),
-                       from_utimestamp(ts), author,
-                       shorten_result(text, terms))
-        
         # Attachments
         for result in AttachmentModule(self.env).get_search_results(
             req, wiki_realm, terms):
